
DECLARE 
	@del_units int,
	@count int,
	@timeProt int,
	@timeMain int,
	@timeMin datetime,
	@timeLimits datetime,
	@caseProt int,
	@caseMain int

SELECT 
	-- Number of transmissions to be deleted on a regular basis
	@del_units = (SELECT UnitsToDelete FROM DeleteParameters),
	-- Factor that informs whether TimeLimitOfTypes needs to be updated
	@count = (SELECT RenewLimits FROM DeleteParameters),
	-- Time limit for deleting from ProtocolData
	@timeProt = (SELECT TimeLimitProtocol FROM dbo.DeleteParameters),
	-- Time limit for deleting from MainData
	@timeMain = (SELECT TimeLimitMain FROM dbo.DeleteParameters),
	-- Time minimum from MainData
	@timeMin = (SELECT MIN(Time) FROM dbo.MainData),
	-- Factor that informs if and how to delete from ProtocolData 
	@caseProt = (SELECT CaseProtocol FROM DeleteParameters),
	-- Factor that informs if and how to delete from MainData 
	@caseMain = (SELECT CaseMain FROM DeleteParameters)

-- If only deletion from ProtocolData is activated, @timeLimits is the set to TimeLimitProtocol
IF (@caseMain = 0 AND @caseProt = 4) SELECT @timeLimits = (SELECT DATEADD("Month",-(@timeProt),GETDATE()))
-- If only deletion from MainData is activated, @timeLimits is the set to TimeLimitMain
IF (@caseMain = 1 AND @caseProt = 0) SELECT @timeLimits = (SELECT DATEADD("Month",-(@timeMain),GETDATE()))
-- If both deletion from MainData and ProtocolData is activated, @timeLimits is the set to th most recent of the two time limits (ProtocolData, MainData)
IF (@caseMain = 1 AND @caseProt = 4) SELECT @timeLimits = MAX(minTime) FROM (SELECT DATEADD("Month",-(@timeProt),GETDATE()) as minTime UNION SELECT DATEADD("Month",-(@timeMain),GETDATE())) as minTime

IF ((SELECT COUNT(Id) FROM TimeLimitsOfTypes) <> 0)
BEGIN
	IF ((SELECT name FROM sys.columns WHERE object_id = (SELECT object_id FROM sys.objects WHERE name = 'TimeLimitsOfTypes') AND name = 'BenchId') IS NOT NULL)
	BEGIN
		---------------------------------------------------------------------------------------------------------------------------------------------------------
		-- If column BenchId is filled with entries then execute stored procedures which are type and bench specific
		---------------------------------------------------------------------------------------------------------------------------------------------------------
		IF ((SELECT Count(BenchId) FROM TimeLimitsOfTypes WHERE BenchId IS NOT NULL) <> 0)
		BEGIN
			-- If delete options include only the time as the limiting factor, execute stored procedures only if data exists before the most recent of time limits.
			IF (@caseMain = 1 OR @caseProt = 4) BEGIN
				IF (@timeMin < @timeLimits) BEGIN
					-- Checks whether TimeLimitOfTypes needs to be updated
					If (@count = 1) EXEC dbo.TimeLimitsRenew_TypeAndBench
					-- Execute procedure 'DeleteData'
					EXEC dbo.DeleteData_TypeAndBench @del_units;
				END
				ELSE PRINT 'Database contains no data prior to the dates specified by TimeLimitProtocol and/or TimeLimitMain.' 
			END

			-- Execute stored procedures if protocol delete options are those cases that do not solely include the time as the limiting factor. 
			IF (@caseProt > 0 AND @caseProt < 4) BEGIN
				-- Checks whether TimeLimitOfTypes needs to be updated
				If (@count = 1) EXEC dbo.TimeLimitsRenew_TypeAndBench
				-- Execute procedure 'DeleteData'
				EXEC dbo.DeleteData_TypeAndBench @del_units;
			END
		END
		---------------------------------------------------------------------------------------------------------------------------------------------------------
		-- If column BenchId is NOT filled with entries then execute stored procedures which are only type specific
		---------------------------------------------------------------------------------------------------------------------------------------------------------
		ELSE
		BEGIN
			-- If delete options include only the time as the limiting factor, execute stored procedures only if data exists before the most recent of time limits.
			IF (@caseMain = 1 OR @caseProt = 4) BEGIN
				IF (@timeMin < @timeLimits) BEGIN
					-- Checks whether TimeLimitOfTypes needs to be updated
					If (@count = 1) EXEC dbo.TimeLimitsRenew
					-- Execute procedure 'DeleteData'
					EXEC dbo.DeleteData @del_units;
				END
				ELSE PRINT 'Database contains no data prior to the dates specified by TimeLimitProtocol and/or TimeLimitMain.' 
			END

			-- Execute stored procedures if protocol delete options are those cases that do not solely include the time as the limiting factor. 
			IF (@caseProt > 0 AND @caseProt < 4) BEGIN
				-- Checks whether TimeLimitOfTypes needs to be updated
				If (@count = 1) EXEC dbo.TimeLimitsRenew
				-- Execute procedure 'DeleteData'
				EXEC dbo.DeleteData @del_units;
			END
		END
	END
	---------------------------------------------------------------------------------------------------------------------------------------------------------
	-- If column BenchId does not exist then execute stored procedures which are only type specific
	---------------------------------------------------------------------------------------------------------------------------------------------------------
	ELSE 
	BEGIN 
		-- If delete options include only the time as the limiting factor, execute stored procedures only if data exists before the most recent of time limits.
		IF (@caseMain = 1 OR @caseProt = 4) BEGIN
			IF (@timeMin < @timeLimits) BEGIN
				-- Checks whether TimeLimitOfTypes needs to be updated
				If (@count = 1) EXEC dbo.TimeLimitsRenew
				-- Execute procedure 'DeleteData'
				EXEC dbo.DeleteData @del_units;
			END
			ELSE PRINT 'Database contains no data prior to the dates specified by TimeLimitProtocol and/or TimeLimitMain.' 
		END

		-- Execute stored procedures if protocol delete options are those cases that do not solely include the time as the limiting factor. 
		IF (@caseProt > 0 AND @caseProt < 4) BEGIN
			-- Checks whether TimeLimitOfTypes needs to be updated
			If (@count = 1) EXEC dbo.TimeLimitsRenew
			-- Execute procedure 'DeleteData'
			EXEC dbo.DeleteData @del_units;
		END
	END
	---------------------------------------------------------------------------------------------------------------------------------------------------------
END
ELSE PRINT 'Table TimeLimitsOfTypes does not contain any entries. Please calculate time limits.'

