
------------------------------------------------------------------------------------------------------------------------------------------
-- Script version number
------------------------------------------------------------------------------------------------------------------------------------------
-- After having made changes to the SQL scripts, adjust the script version number and describe the changes
-- The version number is the date of the changes in the format yyyy-mm-dd
DECLARE @version NVARCHAR(20)
-- Comment on the changes (this will be written to the database)
DECLARE @comment NVARCHAR(500)

-- Everytime the CreateDb/UpdateDb scripts are executed, this will insert the following line into the table 'DatabaseVersion'
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[DatabaseVersion]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
	IF NOT EXISTS (SELECT * FROM DatabaseVersion WHERE Version = @version AND Comment = @comment) 
	BEGIN
		SELECT @version = '2016-11-13'
		SELECT @comment = 'Database change log introduced'
		INSERT INTO [DatabaseVersion] (Version, Date, Comment) VALUES (@version, GetDate(), @comment)	
	END
	IF NOT EXISTS (SELECT * FROM DatabaseVersion WHERE Version = @version AND Comment = @comment) 
	BEGIN
		SELECT @version = '2019-04-29'
		SELECT @comment = 'Second datafile and filegroup (Protocol) for ProtocolData introduced. Float-columns of ProtocolData table changed to real.'
		INSERT INTO [DatabaseVersion] (Version, Date, Comment) VALUES (@version, GetDate(), @comment)	
	END
END
GO	
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
-- Change descriptions (describe the changes in more detail than for the comment above) This will not be written to the database.  
------------------------------------------------------------------------------------------------------------------------------------------
-- 2016-11-13: Table DatabaseChangeLog added to the tables' list. This table contains the log info of the SQL scripts (shown 
-- in DbControl after executing the CreateDb/UpdateDb scripts). The table DatabaseChangeLog is linked to the table 
-- DatabaseVersion where the date of the script execution, the version number, and the result is logged   
------------------------------------------------------------------------------------------------------------------------------------------

