
------------------------------------------------------------------------------------------------------------------------------------------
------------ DatabaseVersion: Check existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
-- Table DatabaseVersion contains the main info of the SQL script execution

IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[DatabaseVersion]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
	CREATE TABLE [dbo].[DatabaseVersion] (
		[Id]		[int]			PRIMARY KEY CLUSTERED IDENTITY (1, 1),
		[Version]	[nvarchar] (20)		NOT NULL,
		[Date]		[datetime]		NOT NULL,
		[Comment]	[nvarchar] (500)	NULL 
	) ON [PRIMARY]
END
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ DatabaseChangeLog: Check existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
-- Table DatabaseChangeLog contains the detailed log info of the CreateDb/UpdateDb SQL scripts

IF NOT EXISTS (select * from dbo.sysobjects where id = object_id(N'[dbo].[DatabaseChangeLog]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN

	CREATE TABLE [dbo].[DatabaseChangeLog](
		[Id] [int] IDENTITY(1,1) NOT NULL,
		[DbVersionId] [int] NOT NULL,
		[Message] [nvarchar](256) NOT NULL,
		[Type] [nvarchar](20) NOT NULL
	 CONSTRAINT [PK_DatabaseChangeLog] PRIMARY KEY CLUSTERED 
	(
		[Id] ASC
	)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
	) ON [PRIMARY]

END
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ DatabaseChangeLog: Check foreign key constraint existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_DatabaseChangeLog_DatabaseVersion')
BEGIN
	ALTER TABLE [dbo].[DatabaseChangeLog] 
	ADD CONSTRAINT [FK_DatabaseChangeLog_DatabaseVersion] FOREIGN KEY([DbVersionId])
	REFERENCES [dbo].[DatabaseVersion]([Id]) 
	ON UPDATE CASCADE 
	ON DELETE CASCADE

END
GO
------------------------------------------------------------------------------------------------------------------------------------------