

/* ------------- THIS SCRIPT IS ONLY ACTIVE FOR SQL SERVER 2008 AND HIGHER --------------------- */

IF (SELECT  SUBSTRING(ver, 1, CHARINDEX('.', ver) - 1)
FROM (SELECT CAST(serverproperty('ProductVersion') AS nvarchar) ver) as t) < '10'
RETURN
GO

-- set the database compatability level at least to SQL Server 2008
IF (SELECT compatibility_level FROM sys.databases WHERE name = '{DbName}') < '100'
ALTER DATABASE {DbName} SET COMPATIBILITY_LEVEL = 100;
GO


------------------------------------------------------------------------------------------------------------------------------------------
------------ Type: TopErrorGroupTableType
------------------------------------------------------------------------------------------------------------------------------------------

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[TopErrorGroupVarTable]') AND [type] = 'FN')
	DROP FUNCTION [dbo].[TopErrorGroupVarTable]
GO

IF EXISTS (SELECT * FROM sys.types WHERE is_table_type = 1 AND name ='TopErrorGroupTableType') 
	DROP TYPE [dbo].[TopErrorGroupTableType]
GO

CREATE TYPE TopErrorGroupTableType AS TABLE( 
	[GroupId] 	[int]	NOT NULL,  -- Id of error selection group
	[Nr]		[int]	NOT NULL,  -- Nr of entry
	[StartErrorId]	[int]	NOT NULL,  -- start of error id that belongs to this group
	[EndErrorId]	[int]	NOT NULL   -- end of error id that belongs to this group

)
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------
------------ Function: TopErrorGroupVarTable
------------------------------------------------------------------------------------------------------------------------------------------
-- =============================================
-- Author:		Thomas Lewien (modified by Pyare Pueschel)
-- Create date: 2015-06-23 (2017-11-24)
-- Description:	Returns error group. Accepts a table with the error id's
-- =============================================
CREATE FUNCTION [dbo].[TopErrorGroupVarTable]
(
-- Add the parameters for the function here
@pkIdent int,
@Teg TopErrorGroupTableType READONLY,
@TopN INT,
@FirstOrNErrors bit
)
RETURNS int
AS
BEGIN

DECLARE @GroupId int

IF @FirstOrNErrors = 1 
BEGIN
	DECLARE @errorCode int
	SET @errorCode = dbo.FirstError(@pkIdent)

	IF (@errorCode = -1) RETURN -1	-- good units

	SET @GroupId = (
	SELECT TOP 1 GroupId from @Teg
	WHERE GroupId = @errorCode
	ORDER BY GroupId ASC
	)

	-- Return the result of the function or -2 for all not assigned groups
	RETURN ISNULL(@GroupId,-2)

END
ELSE BEGIN
	DECLARE @TempTable table (Id INT)
	INSERT INTO @TempTable 
	SELECT * FROM dbo.FirstAndNErrors(@pkIdent, @TopN)

	IF (SELECT MIN(Id) FROM @TempTable) = -1 RETURN -1	-- good units

	SET @GroupId = (
		SELECT TOP 1 GroupId from @Teg
		WHERE GroupId IN (SELECT Id FROM @TempTable)
		ORDER BY GroupId ASC
		)

	-- Return the result of the function or -2 for all not assigned groups
	RETURN ISNULL(@GroupId,-2)
END

RETURN @GroupId

END
GO
------------------------------------------------------------------------------------------------------------------------------------------

