
------------------------------------------------------------------------------------------------------------------------------------------
------------ Application: Check existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[Application]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN

CREATE TABLE [dbo].[Application] (
	[Id]		[int]		PRIMARY KEY CLUSTERED IDENTITY (1, 1) NOT NULL,
	[Name]		[nvarchar] (200) NULL
) ON [PRIMARY]

PRINT 'Added table Application'
END
ELSE PRINT 'Table Application already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ User: Check existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[User]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN

CREATE TABLE [dbo].[User] (
	[Id]		[int]		PRIMARY KEY CLUSTERED IDENTITY (1, 1) NOT NULL,
	[Name]		[nvarchar] (200)NULL
) ON [PRIMARY]

PRINT 'Added table User'
END
ELSE PRINT 'Table User already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ StorageClass: Check existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[StorageClass]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN

CREATE TABLE [dbo].[StorageClass] (
	[Id]		[int]		PRIMARY KEY CLUSTERED IDENTITY (1, 1) NOT NULL,
	[Name]		[nvarchar] (200)NULL
) ON [PRIMARY]

PRINT 'Added table StorageClass'
END
ELSE PRINT 'Table StorageClass already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ Instance: Check existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[Instance]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN

CREATE TABLE [dbo].[Instance] (
	[Id]		[int]		PRIMARY KEY CLUSTERED IDENTITY (1, 1) NOT NULL,
	[Name]		[nvarchar] (200)NULL
) ON [PRIMARY]

PRINT 'Added table Instance'
END
ELSE PRINT 'Table Instance already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ MainData: Check existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[MainData]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN

	CREATE TABLE [dbo].[MainData] (
		[Id]			[int]		PRIMARY KEY CLUSTERED IDENTITY (1, 1) NOT NULL,
		[ApplicationId]	[int]		NOT NULL,
		[UserId]		[int]		NOT NULL,
		[StorageClassId][int]		NOT NULL,
		[InstanceId]	[int]		NOT NULL,
		[Accessed]		[datetime]	NOT NULL,
	) ON [PRIMARY]

PRINT 'Added table MainData'
END
ELSE PRINT 'Table MainData already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------
	
------------------------------------------------------------------------------------------------------------------------------------------
------------ MainData: Check foreign key constraint existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_MainData_Application')
BEGIN
	ALTER TABLE [dbo].[MainData] 
	ADD CONSTRAINT [FK_MainData_Application] FOREIGN KEY([ApplicationId])
	REFERENCES [dbo].[Application]([Id])

PRINT 'Added FK constraint FK_MainData_Application'
END
ELSE PRINT 'FK constraint FK_MainData_Application already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_MainData_User')
BEGIN
	ALTER TABLE [dbo].[MainData] 
	ADD CONSTRAINT [FK_MainData_User] FOREIGN KEY([UserId])
	REFERENCES [dbo].[User]([Id])
	ON UPDATE CASCADE
	ON DELETE CASCADE

	PRINT 'Added FK constraint FK_MainData_User'
END
ELSE PRINT 'FK constraint FK_MainData_User already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_MainData_StorageClass')
BEGIN
	ALTER TABLE [dbo].[MainData] 
	ADD CONSTRAINT [FK_MainData_StorageClass] FOREIGN KEY([StorageClassId])
	REFERENCES [dbo].[StorageClass]([Id])

PRINT 'Added FK constraint FK_MainData_StorageClass'
END
ELSE PRINT 'FK constraint FK_MainData_StorageClass already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_MainData_PropertyID')
BEGIN
	ALTER TABLE [dbo].[MainData] 
	ADD CONSTRAINT [FK_MainData_PropertyID] FOREIGN KEY([InstanceId])
	REFERENCES [dbo].[Instance]([Id])
		
	ON DELETE CASCADE
	ON UPDATE CASCADE

PRINT 'Added FK constraint FK_MainData_PropertyID'
END
ELSE PRINT 'FK constraint FK_MainData_PropertyID already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ XmlStorage: Check existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[XmlStorage]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN

	CREATE TABLE [dbo].[XmlStorage] (
		[Id]			[int]		PRIMARY KEY  CLUSTERED NOT NULL,
		[Data]			[ntext]		NULL
	) ON [PRIMARY]
	
PRINT 'Added table XmlStorage'
END
ELSE PRINT 'Table XmlStorage already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ XmlStorage: Check foreign key constraint existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_XmlStorage_Id')
BEGIN
	ALTER TABLE [dbo].[XmlStorage] 
	ADD CONSTRAINT [FK_XmlStorage_Id] FOREIGN KEY([Id])
	REFERENCES [dbo].[MainData]([Id])
		
	ON DELETE CASCADE
	ON UPDATE CASCADE

PRINT 'Added FK constraint FK_XmlStorage_Id'
END
ELSE PRINT 'FK constraint FK_XmlStorage_Id already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------

sp_tableoption N'XmlStorage', 'text in row', '7000'
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
-- Insert Stored Procedure to Get or Create an ItemIndex
------------------------------------------------------------------------------------------------------------------------------------------
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[spGetOrCreateItemIndex]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
DROP PROCEDURE [dbo].[spGetOrCreateItemIndex]
GO

CREATE PROC [dbo].[spGetOrCreateItemIndex]
	@Application 	nvarchar(200),
	@User		 	nvarchar(200),
	@StorageClass 	nvarchar(200),
	@Instance		nvarchar(200),
	
	@Id				int OUTPUT
AS

DECLARE @ApplicationId	int
DECLARE @UserId			int
DECLARE @StorageClassId	int
DECLARE @InstanceId		int

BEGIN TRAN	--wrap to make sure that nobody inserts new Id's while we are active.

SET @ApplicationId = (SELECT Id FROM [dbo].[Application] WHERE [Name] = @Application)
IF @ApplicationId IS NULL
BEGIN
	INSERT INTO [dbo].[Application] VALUES (@Application)
 	SELECT @ApplicationId = @@IDENTITY
END

SET @UserId = (SELECT Id FROM [dbo].[User] WHERE [Name] = @User)
IF @UserId IS NULL
BEGIN
	INSERT INTO [dbo].[User] VALUES (@User)
 	SELECT @UserId = @@IDENTITY
END

SET @StorageClassId = (SELECT Id FROM [dbo].[StorageClass] WHERE [Name] = @StorageClass)
IF @StorageClassId IS NULL
BEGIN
	INSERT INTO [dbo].[StorageClass] VALUES (@StorageClass)
 	SELECT @StorageClassId = @@IDENTITY
END

SET @InstanceId = (SELECT Id FROM [dbo].[Instance] WHERE [Name] = @Instance)
IF @InstanceId IS NULL
BEGIN
	INSERT INTO [dbo].[Instance] VALUES (@Instance)
 	SELECT @InstanceId = @@IDENTITY
END

SET @Id = 
	(
		SELECT Id FROM [dbo].[MainData] 
		WHERE	[ApplicationId] = @ApplicationId
		AND		[UserId] = @UserId
		AND		[StorageClassId] = @StorageClassId
		AND		[InstanceId] = @InstanceId
	)

IF @Id IS NULL
BEGIN
	INSERT INTO MainData 
	VALUES (
		@ApplicationId,
		@UserId,
		@StorageClassId,
		@InstanceId,
		GETDATE()
	)
	SELECT @Id = @@IDENTITY
END 
ELSE
BEGIN
	UPDATE MainData SET [Accessed]=GETDATE() WHERE Id = @Id
END
COMMIT TRAN
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
-- Insert Stored Procedure to Get an ItemIndex
------------------------------------------------------------------------------------------------------------------------------------------
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[spGetItemIndex]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
DROP PROCEDURE [dbo].[spGetItemIndex]
GO

CREATE PROC [dbo].[spGetItemIndex]
	@Application 	nvarchar(200),
	@User		 	nvarchar(200),
	@StorageClass 	nvarchar(200),
	@Instance		nvarchar(200),
	
	@Id				int OUTPUT
AS

SET @Id = 
	(
		SELECT [dbo].[MainData].[Id] FROM [dbo].[MainData] 
		INNER JOIN [dbo].[Application] ON [dbo].[Application].[Id] = [dbo].[MainData].[ApplicationId]
		INNER JOIN [dbo].[User] ON [dbo].[User].[Id] = [dbo].[MainData].[UserId]
		INNER JOIN [dbo].[StorageClass] ON [dbo].[StorageClass].[Id] = [dbo].[MainData].[StorageClassId]
		INNER JOIN [dbo].[Instance] ON [dbo].[Instance].[Id] = [dbo].[MainData].[InstanceId]
		WHERE	[dbo].[Application].[Name] = @Application
		AND		[dbo].[User].[Name] = @User
		AND		[dbo].[StorageClass].[Name] = @StorageClass
		AND		[dbo].[Instance].[Name] = @Instance
	)

IF @Id IS NULL
BEGIN
	SELECT @Id = -1
END 

GO
------------------------------------------------------------------------------------------------------------------------------------------

