

---------------------------------------------------------------------------------------------------------------------------
/* 1. Drop all Foreign Key constraints */
---------------------------------------------------------------------------------------------------------------------------

DECLARE @name VARCHAR(128)
DECLARE @constraint VARCHAR(254)
DECLARE @SQL VARCHAR(254)

SELECT @name = (SELECT TOP 1 TABLE_NAME FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS WHERE constraint_catalog=DB_NAME() AND CONSTRAINT_TYPE = 'FOREIGN KEY' ORDER BY TABLE_NAME)

WHILE @name is not null
BEGIN
    SELECT @constraint = (SELECT TOP 1 CONSTRAINT_NAME FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS WHERE constraint_catalog=DB_NAME() AND CONSTRAINT_TYPE = 'FOREIGN KEY' AND TABLE_NAME = @name ORDER BY CONSTRAINT_NAME)
    WHILE @constraint IS NOT NULL
    BEGIN
        SELECT @SQL = 'ALTER TABLE [dbo].[' + RTRIM(@name) +'] DROP CONSTRAINT [' + RTRIM(@constraint) +']'
		EXEC (@SQL)
		PRINT 'Dropped FK Constraint: ' + @constraint + ' on ' + @name
        
		SELECT @constraint = (SELECT TOP 1 CONSTRAINT_NAME FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS WHERE constraint_catalog=DB_NAME() AND CONSTRAINT_TYPE = 'FOREIGN KEY' AND CONSTRAINT_NAME <> @constraint AND TABLE_NAME = @name ORDER BY CONSTRAINT_NAME)
    END
SELECT @name = (SELECT TOP 1 TABLE_NAME FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS WHERE constraint_catalog=DB_NAME() AND CONSTRAINT_TYPE = 'FOREIGN KEY' ORDER BY TABLE_NAME)
END
GO

---------------------------------------------------------------------------------------------------------------------------
/* 2. Truncate all tables except DeleteParameters */
---------------------------------------------------------------------------------------------------------------------------

DECLARE @name VARCHAR(128)
DECLARE @SQL VARCHAR(254)

SELECT @name = (SELECT TOP 1 [name] FROM sysobjects WHERE [type] = 'U' AND category = 0 ORDER BY [name])

WHILE @name IS NOT NULL
BEGIN
    SELECT @SQL = 'TRUNCATE TABLE [dbo].[' + RTRIM(@name) +']'
	EXEC (@SQL)
	PRINT 'Truncated Table: ' + @name
	SELECT @name = (SELECT TOP 1 [name] FROM sysobjects WHERE [type] = 'U' AND category = 0 AND [name] > @name ORDER BY [name])
END
GO

---------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------
/* 3. Re-establish foreign key constraints */
---------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ MainData: Check foreign key constraint existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_MainData_Application')
BEGIN
	ALTER TABLE [dbo].[MainData] 
	ADD CONSTRAINT [FK_MainData_Application] FOREIGN KEY([ApplicationId])
	REFERENCES [dbo].[Application]([Id])

PRINT 'Added FK constraint FK_MainData_Application'
END
ELSE PRINT 'FK constraint FK_MainData_Application already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_MainData_User')
BEGIN
	ALTER TABLE [dbo].[MainData] 
	ADD CONSTRAINT [FK_MainData_User] FOREIGN KEY([UserId])
	REFERENCES [dbo].[User]([Id])
	ON UPDATE CASCADE
	ON DELETE CASCADE
PRINT 'Added FK constraint FK_MainData_User'
END
ELSE PRINT 'FK constraint FK_MainData_User already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_MainData_StorageClass')
BEGIN
	ALTER TABLE [dbo].[MainData] 
	ADD CONSTRAINT [FK_MainData_StorageClass] FOREIGN KEY([StorageClassId])
	REFERENCES [dbo].[StorageClass]([Id])

PRINT 'Added FK constraint FK_MainData_StorageClass'
END
ELSE PRINT 'FK constraint FK_MainData_StorageClass already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_MainData_PropertyID')
BEGIN
	ALTER TABLE [dbo].[MainData] 
	ADD CONSTRAINT [FK_MainData_PropertyID] FOREIGN KEY([InstanceId])
	REFERENCES [dbo].[Instance]([Id])
		
	ON DELETE CASCADE
	ON UPDATE CASCADE

PRINT 'Added FK constraint FK_MainData_PropertyID'
END
ELSE PRINT 'FK constraint FK_MainData_PropertyID already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ XmlStorage: Check foreign key constraint existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_XmlStorage_Id')
BEGIN
	ALTER TABLE [dbo].[XmlStorage] 
	ADD CONSTRAINT [FK_XmlStorage_Id] FOREIGN KEY([Id])
	REFERENCES [dbo].[MainData]([Id])
		
	ON DELETE CASCADE
	ON UPDATE CASCADE

PRINT 'Added FK constraint FK_XmlStorage_Id'
END
ELSE PRINT 'FK constraint FK_XmlStorage_Id already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------

sp_tableoption N'XmlStorage', 'text in row', '7000'
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
-- Insert Stored Procedure to Get or Create an ItemIndex
------------------------------------------------------------------------------------------------------------------------------------------
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[spGetOrCreateItemIndex]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
DROP PROCEDURE [dbo].[spGetOrCreateItemIndex]
GO

CREATE PROC [dbo].[spGetOrCreateItemIndex]
	@Application 	nvarchar(200),
	@User		 	nvarchar(200),
	@StorageClass 	nvarchar(200),
	@Instance		nvarchar(200),
	
	@Id				int OUTPUT
AS

DECLARE @ApplicationId	int
DECLARE @UserId			int
DECLARE @StorageClassId	int
DECLARE @InstanceId		int

BEGIN TRAN	--wrap to make sure that nobody inserts new Id's while we are active.

SET @ApplicationId = (SELECT Id FROM [dbo].[Application] WHERE [Name] = @Application)
IF @ApplicationId IS NULL
BEGIN
	INSERT INTO [dbo].[Application] VALUES (@Application)
 	SELECT @ApplicationId = @@IDENTITY
END

SET @UserId = (SELECT Id FROM [dbo].[User] WHERE [Name] = @User)
IF @UserId IS NULL
BEGIN
	INSERT INTO [dbo].[User] VALUES (@User)
 	SELECT @UserId = @@IDENTITY
END

SET @StorageClassId = (SELECT Id FROM [dbo].[StorageClass] WHERE [Name] = @StorageClass)
IF @StorageClassId IS NULL
BEGIN
	INSERT INTO [dbo].[StorageClass] VALUES (@StorageClass)
 	SELECT @StorageClassId = @@IDENTITY
END

SET @InstanceId = (SELECT Id FROM [dbo].[Instance] WHERE [Name] = @Instance)
IF @InstanceId IS NULL
BEGIN
	INSERT INTO [dbo].[Instance] VALUES (@Instance)
 	SELECT @InstanceId = @@IDENTITY
END

SET @Id = 
	(
		SELECT Id FROM [dbo].[MainData] 
		WHERE	[ApplicationId] = @ApplicationId
		AND		[UserId] = @UserId
		AND		[StorageClassId] = @StorageClassId
		AND		[InstanceId] = @InstanceId
	)

IF @Id IS NULL
BEGIN
	INSERT INTO MainData 
	VALUES (
		@ApplicationId,
		@UserId,
		@StorageClassId,
		@InstanceId,
		GETDATE()
	)
	SELECT @Id = @@IDENTITY
END 
ELSE
BEGIN
	UPDATE MainData SET [Accessed]=GETDATE() WHERE Id = @Id
END
COMMIT TRAN
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
-- Insert Stored Procedure to Get an ItemIndex
------------------------------------------------------------------------------------------------------------------------------------------
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[spGetItemIndex]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
DROP PROCEDURE [dbo].[spGetItemIndex]
GO

CREATE PROC [dbo].[spGetItemIndex]
	@Application 	nvarchar(200),
	@User		 	nvarchar(200),
	@StorageClass 	nvarchar(200),
	@Instance		nvarchar(200),
	
	@Id				int OUTPUT
AS

SET @Id = 
	(
		SELECT [dbo].[MainData].[Id] FROM [dbo].[MainData] 
		INNER JOIN [dbo].[Application] ON [dbo].[Application].[Id] = [dbo].[MainData].[ApplicationId]
		INNER JOIN [dbo].[User] ON [dbo].[User].[Id] = [dbo].[MainData].[UserId]
		INNER JOIN [dbo].[StorageClass] ON [dbo].[StorageClass].[Id] = [dbo].[MainData].[StorageClassId]
		INNER JOIN [dbo].[Instance] ON [dbo].[Instance].[Id] = [dbo].[MainData].[InstanceId]
		WHERE	[dbo].[Application].[Name] = @Application
		AND		[dbo].[User].[Name] = @User
		AND		[dbo].[StorageClass].[Name] = @StorageClass
		AND		[dbo].[Instance].[Name] = @Instance
	)

IF @Id IS NULL
BEGIN
	SELECT @Id = -1
END 

GO
------------------------------------------------------------------------------------------------------------------------------------------

