
------------------------------------------------------------------------------------------------------------------------------------------
------------ Procedure: spCurrentRejectGroup
------------------------------------------------------------------------------------------------------------------------------------------

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[spCurrentRejectGroup]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
	DROP PROCEDURE [dbo].spCurrentRejectGroup
GO

/* 
	
*/ 
CREATE PROCEDURE [dbo].spCurrentRejectGroup 
(
	 @startTime datetime,
	 @endTime datetime,
	 @fixDaysOffset int,
	 @weekNr int,
	 @isUpdate int		-- --0: insert, 1: update


)
 AS
BEGIN

SET NOCOUNT ON
DECLARE 
	@fixWeekTime datetime,
	@thisCountType int
	
	DECLARE @TempDetails TABLE 
	(
			[WeekNr]		[int]		NOT NULL,
			[Type]			[int]		NOT NULL,	
			[TestBench]		[int]		NOT NULL,	
			[CountType]		[int]		NOT NULL,	
			[All]			[int]		NULL,
			[Good]			[int]		NULL,
			[Bad]			[int]		NULL
	)

	SELECT @thisCountType = [Id] FROM [dbo].[ResultCountType] WHERE [Name] = 'CurrentReject'

	SET @fixWeekTime   = DATEADD(day,@fixDaysOffset,@endTime)

	INSERT INTO @TempDetails (
		[WeekNr], 
		[Type],		
		[TestBench],	
		[CountType],	
		[All],		
		[Good],		
		[Bad]
	)
		SELECT 
		@weekNr,
		t.Id,
		b.Id,
		@thisCountType,
		COUNT(*),  
		SUM(dbo.HasResult(MainData.Result,1)), 
		SUM(dbo.HasResult(MainData.Result,2)) + SUM(dbo.HasResult(MainData.Result,4)) 
	FROM 
		Maindata  
		INNER JOIN dbo.TypeID t ON MainData.Type = t.Id
		INNER JOIN dbo.BenchId b ON MainData.TestBench = b.Id

		WHERE  
		t.Id > 0 AND b.Id > 0 -- not the '*' entries
		AND MainData.Serial NOT IN ( SELECT [Name] FROM [ReferenceSerials])  
		AND t.Name NOT IN ( SELECT [Name] FROM [ReferenceTypes]) 
		AND (MainData.TestKind = 1)
		
		AND  ((MainData.Time >= @startTime)  AND  (MainData.Time < @endTime) ) 
		AND
		-- Only Test 
		( ((MainData.ThisLast = 1) AND (MainData.Ordinal = 1) ) 
		
		OR  
		-- First test and same as any test there after
		(  (MainData.ThisLast = 0) AND (MainData.Ordinal = 1)   
		AND  ( SELECT COUNT(*) FROM  MainData e 
				WHERE 
					e.Serial = MainData.Serial  
					AND e.Type = MainData.Type  
					AND e.Time > = MainData.Time  
					AND e.Time <  @fixWeekTime  
					AND e.Result <> MainData.Result)  = 0  )  
		 
		  OR
		   -- Last Test:  different from test before	
		  (
			(MainData.Ordinal > 1) AND (MainData.ThisLast = 1) AND
			(
				-- no earlier tests of same kind
				(
					(Select Count(*) FROM MainData o   WHERE o.Serial = MainData.Serial   
													AND o.Type = MainData.Type 
													AND o.TestKind = MainData.TestKind
													AND o.Time < MainData.Time)
					= 0
				)
				OR
				-- different from last test
				( SELECT COUNT(*) FROM  MainData e 
				WHERE 
					e.Serial = MainData.Serial   
					AND e.Type = MainData.Type  
					AND e.Ordinal =  
					-- (MainData.Ordinal-1) for the same TestKind 
					(Select Max(Ordinal) FROM MainData o   WHERE o.Serial = MainData.Serial   
											AND o.Type = MainData.Type AND o.TestKind = MainData.TestKind
											AND o.Time < MainData.Time
					)
					AND e.Result <> MainData.Result)  > 0   
			)
		  )
					 
		 
					 
		  OR  
		  -- Any test: different from test before and not different from any test thereafter
		  (	MainData.Ordinal > 1 AND
			(				
				(					
					-- no earlier tests of same kind
					(Select Count(*) FROM MainData o   WHERE o.Serial = MainData.Serial   
													AND o.Type = MainData.Type 
													AND o.TestKind = MainData.TestKind
													AND o.Time < MainData.Time)
					= 0
					
					OR  -- different from last test
					( SELECT COUNT(*) FROM  MainData e 
						WHERE 
							e.Serial = MainData.Serial   
							AND e.Type = MainData.Type  
							AND e.Ordinal = 
								-- (MainData.Ordinal-1) for the same TestKind 
								(Select Max(Ordinal) FROM MainData o   WHERE o.Serial = MainData.Serial   
														AND o.Type = MainData.Type 
														AND o.TestKind = MainData.TestKind
														AND o.Time < MainData.Time
								)
							AND e.Result <> MainData.Result)  > 0
				)
				-- no later result that is different
				AND   ( SELECT COUNT(*) FROM  MainData e 
					WHERE e.Serial = MainData.Serial  
						AND e.Type = MainData.Type  
						AND e.Time > = MainData.Time  
						AND  e.Time <  @fixWeekTime  
						AND e.Result <> MainData.Result)  
				= 0  
				 
			)
		  )			
    ) 
					
	GROUP BY t.Id, b.Id WITH ROLLUP -- cube
	HAVING GROUPING (t.Id) = 0 AND  GROUPING (b.Id) = 0 -- avoid null entries
	
	IF (@isUpdate = 1)
	BEGIN
		BEGIN TRAN
			DELETE FROM [WeeklyResultDetails] WHERE [WeeklyResultDetails].[WeekNr] = @weekNr AND [CountType] = @thisCountType
	 		INSERT INTO [WeeklyResultDetails] SELECT * FROM @TempDetails	
		COMMIT
	END ELSE
		INSERT INTO [WeeklyResultDetails] SELECT * FROM @TempDetails	
	

END -- procedure spCurrentRejectGroup
GO
------------------------------------------------------------------------------------------------------------------------------------------

PRINT 'Stored procedure spCurrentRejectGroup added'
GO
