
------------------------------------------------------------------------------------------------------------------------------------------
------------ Procedure: spDirectTestGroup
------------------------------------------------------------------------------------------------------------------------------------------

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[spDirectTestGroup]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
	DROP PROCEDURE [dbo].spDirectTestGroup
GO


IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[spDirectTestGroup]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
	DROP PROCEDURE [dbo].spDirectTestGroup
GO

/* 
	
*/ 

CREATE PROCEDURE [dbo].[spDirectTestGroup]
(
@startTime datetime,
@endTime datetime,
@fixDaysOffset int,
@weekNr int,
@isUpdate int,		-- --0: insert, 1: update
@maxMinutes int, 
@maxOrdinal int
)
AS
BEGIN
SET NOCOUNT ON
DECLARE
@fixWeekTime datetime,
@thisCountType int
DECLARE @TempDetails TABLE
(
[WeekNr]		[int]		NOT NULL,
[Type]			[int]		NOT NULL,
[TestBench]		[int]		NOT NULL,
[CountType]		[int]		NOT NULL,
[All]			[int]		NULL,
[Good]			[int]		NULL,
[Bad]			[int]		NULL
)
SELECT @thisCountType = [Id] FROM [dbo].[ResultCountType] WHERE [Name] = 'Direct'
SET @fixWeekTime   = DATEADD(day,@fixDaysOffset,@endTime)

-- Both conditions are met
IF (@maxOrdinal > 0 AND @maxMinutes > 0)
BEGIN
		 INSERT INTO @TempDetails (
		[WeekNr],
		[Type],
		[TestBench],
		[CountType],
		[All],
		[Good],
		[Bad]
		)
		SELECT
		@weekNr,
		t.Id,
		b.Id,
		@thisCountType,
		COUNT(*),
		SUM(dbo.HasResult(MainData.Result,1)),
		SUM(dbo.HasResult(MainData.Result,2)) + SUM(dbo.HasResult(MainData.Result,4))
		FROM
		Maindata
		INNER JOIN dbo.TypeID t ON MainData.Type = t.Id
		INNER JOIN dbo.BenchId b ON MainData.TestBench = b.Id
		WHERE
		t.Id > 0 AND b.Id > 0 -- not the '*' entries
		AND MainData.Serial NOT IN ( SELECT [Name] FROM [ReferenceSerials])
		AND t.Name NOT IN ( SELECT [Name] FROM [ReferenceTypes])
		AND (MainData.TestKind = 1)
		AND  ((MainData.Time >= @startTime)  AND  (MainData.Time < @endTime))
		
		-- Only Test
		AND (((MainData.ThisLast = 1) AND (MainData.Ordinal = 1))
		
            -- first test is BEFORE the time frame and subsequent tests are WITHIN the time frame  
            -- if the result HAS changed count the subsequent test
            -- last test within maxHours is also the last test overall 
            OR PkIdent IN (
                 SELECT b.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result <> a.Result 
                 WHERE 
                 a.Ordinal = 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <=  @maxMinutes
                 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time < @startTime 
                 AND ((b.Ordinal = @maxOrdinal )
                 OR (b.Ordinal > 1 AND b.Ordinal < @maxOrdinal AND b.ThisLast = 1))
                 ) 

            -- first test is BEFORE the time frame and subsequent tests are WITHIN the time frame  
            -- if the result HAS changed count the subsequent test
            -- last test within maxHours which is NOT the last test overall
			-- Get the subsequent test which equals the maxOrdinal and is within the time frame OR is < maxOrdinal and the last one within the time frame
            OR PkIdent IN (
                 SELECT b.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result <> a.Result 
                 WHERE 
                 a.Ordinal = 1 
				 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time < @startTime
				 AND ((b.Ordinal = @maxOrdinal )
                 OR (b.Ordinal > 1 AND b.Ordinal < @maxOrdinal AND b.ThisLast = 1))
                 )

			-- first test is WITHIN the time frame and subsequent tests are AFTER the time frame  
            -- if the result has NOT changed count the first test
            -- last test within maxHours which is also the last test overall
			-- Get the subsequent test which equals the maxOrdinal and is within the time frame OR is < maxOrdinal and the last one within the time frame
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result = a.Result 
                 WHERE 
                 a.Ordinal = 1 
				 AND a.Time > = @startTime AND a.Time < @endTime
				 AND b.Time > = @endTime 
                 AND ((b.Ordinal = @maxOrdinal )
                 OR (b.Ordinal > 1 AND b.Ordinal < @maxOrdinal AND b.ThisLast = 1)) 
                 ) 

            -- first test is WITHIN the time frame and subsequent tests are AFTER the time frame  
            -- if the result has NOT changed count the first test
            -- last test within maxHours which is NOT the last test overall
			-- Get the subsequent test which equals the maxOrdinal and is within the time frame OR is < maxOrdinal and the last one within the time frame
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result = a.Result 
                 WHERE 
                 a.Ordinal = 1 
				 AND a.Time > = @startTime AND a.Time < @endTime
				 AND b.Time > = @endTime
				 AND ((b.Ordinal = @maxOrdinal )
                 OR (b.Ordinal > 1 AND b.Ordinal < @maxOrdinal AND b.ThisLast = 1)) 
                 )

			-- first and subsequent tests are WITHIN the time frame 
            -- if the result has NOT changed count the first test
            -- last test within maxHours which is also the last test overall
			-- Get the subsequent test which equals the maxOrdinal and is within the time frame OR is < maxOrdinal and the last one within the time frame
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result = a.Result 
				 WHERE 
                 a.Ordinal = 1 
				 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time > = @startTime AND a.Time < @endTime
                 AND ((b.Ordinal = @maxOrdinal )
                 OR (b.Ordinal > 1 AND b.Ordinal < @maxOrdinal AND b.ThisLast = 1))
                 ) 

            -- first and subsequent tests are WITHIN the time frame 
            -- if the result has NOT changed count the first test
            -- last test within maxHours which is NOT the last test overall
			-- Get the subsequent test which equals the maxOrdinal and is within the time frame OR is < maxOrdinal and the last one within the time frame
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result = a.Result 
				 WHERE 
                 a.Ordinal = 1 
				 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time > = @startTime AND a.Time < @endTime
				 AND ((b.Ordinal = @maxOrdinal )
                 OR (b.Ordinal > 1 AND b.Ordinal < @maxOrdinal AND b.ThisLast = 1))
                 )

			-- first and subsequent tests are WITHIN the time frame 
            -- if the result HAS changed count the subsequent test
            -- last test within maxHours which is also the last test overall
			-- Get the subsequent test which equals the maxOrdinal and is within the time frame OR is < maxOrdinal and the last one within the time frame
            OR PkIdent IN (
                 SELECT b.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result <> a.Result 
				 WHERE 
                 a.Ordinal = 1 
				 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time > = @startTime AND a.Time < @endTime
                 AND ((b.Ordinal = @maxOrdinal )
                 OR (b.Ordinal > 1 AND b.Ordinal < @maxOrdinal AND b.ThisLast = 1))
                 ) 

            -- first and subsequent tests are WITHIN the time frame 
            -- if the result HAS changed count the subsequent test
            -- last test within maxHours which is NOT the last test overall
			-- Get the subsequent test which equals the maxOrdinal and is within the time frame OR is < maxOrdinal and the last one within the time frame
            OR PkIdent IN (
                 SELECT b.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result <> a.Result 
				 WHERE 
                 a.Ordinal = 1 
				 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time > = @startTime AND a.Time < @endTime
				 AND ((b.Ordinal = @maxOrdinal )
                 OR (b.Ordinal > 1 AND b.Ordinal < @maxOrdinal AND b.ThisLast = 1))
                 ))

		GROUP BY t.Id, b.Id WITH ROLLUP -- cube
		HAVING GROUPING (t.Id) = 0 AND  GROUPING (b.Id) = 0 -- avoid null entries
END
-- Only the max ordinal condition is met
ELSE IF (@maxOrdinal > 0 AND @maxMinutes = 0)
BEGIN
		 INSERT INTO @TempDetails (
		[WeekNr],
		[Type],
		[TestBench],
		[CountType],
		[All],
		[Good],
		[Bad]
		)
		SELECT
		@weekNr,
		t.Id,
		b.Id,
		@thisCountType,
		COUNT(*),
		SUM(dbo.HasResult(MainData.Result,1)),
		SUM(dbo.HasResult(MainData.Result,2)) + SUM(dbo.HasResult(MainData.Result,4))
		FROM
		Maindata
		INNER JOIN dbo.TypeID t ON MainData.Type = t.Id
		INNER JOIN dbo.BenchId b ON MainData.TestBench = b.Id
		WHERE
		t.Id > 0 AND b.Id > 0 -- not the '*' entries
		AND MainData.Serial NOT IN ( SELECT [Name] FROM [ReferenceSerials])
		AND t.Name NOT IN ( SELECT [Name] FROM [ReferenceTypes])
		AND (MainData.TestKind = 1)
		AND  ((MainData.Time >= @startTime)  AND  (MainData.Time < @endTime))
		
		-- Only Test
		AND (((MainData.ThisLast = 1) AND (MainData.Ordinal = 1))
		
			-- First test within the time slot but 2nd test outside the maxHours
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind 
                 WHERE 
                 a.Ordinal = 1 AND b.Ordinal = 2 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) >  @maxMinutes
                 AND a.Time > = @startTime AND a.Time < @endTime 
                 ) 

            -- first test is BEFORE the time frame and subsequent tests are WITHIN the time frame  
            -- if the result HAS changed count the subsequent test
            -- last test within maxHours is also the last test overall 
            OR PkIdent IN (
                 SELECT b.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result <> a.Result 
                 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <=  @maxMinutes
                 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time < @startTime 
                 AND b.ThisLast = 1 
                 ) 

            -- first test is BEFORE the time frame and subsequent tests are WITHIN the time frame  
            -- if the result HAS changed count the subsequent test
            -- last test within maxHours which is NOT the last test overall
            OR PkIdent IN (
                 SELECT b.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result <> a.Result 
                 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <= @maxMinutes
                 AND DATEDIFF(mi, a.[Time], b.[TimeOfNextMeasurement]) >  @maxMinutes
                 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time < @startTime
                 )

			-- first test is WITHIN the time frame and subsequent tests are AFTER the time frame  
            -- if the result has NOT changed count the first test
            -- last test within maxHours which is also the last test overall
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result = a.Result 
                 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <=  @maxMinutes
                 AND a.Time > = @startTime AND a.Time < @endTime
				 AND b.Time > = @endTime 
                 AND b.ThisLast = 1 
                 ) 

            -- first test is WITHIN the time frame and subsequent tests are AFTER the time frame  
            -- if the result has NOT changed count the first test
            -- last test within maxHours which is NOT the last test overall
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result = a.Result 
                 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <= @maxMinutes
                 AND DATEDIFF(mi, a.[Time], b.[TimeOfNextMeasurement]) >  @maxMinutes
                 AND a.Time > = @startTime AND a.Time < @endTime
				 AND b.Time > = @endTime 
                 )

			-- first and subsequent tests are WITHIN the time frame 
            -- if the result has NOT changed count the first test
            -- last test within maxHours which is also the last test overall
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result = a.Result 
				 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <=  @maxMinutes
                 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time > = @startTime AND a.Time < @endTime
                 AND b.ThisLast = 1 
                 ) 

            -- first and subsequent tests are WITHIN the time frame 
            -- if the result has NOT changed count the first test
            -- last test within maxHours which is NOT the last test overall
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result = a.Result 
				 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <= @maxMinutes
                 AND DATEDIFF(mi, a.[Time], b.[TimeOfNextMeasurement]) >  @maxMinutes
                 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time > = @startTime AND a.Time < @endTime
                 )

			-- first and subsequent tests are WITHIN the time frame 
            -- if the result HAS changed count the subsequent test
            -- last test within maxHours which is also the last test overall
            OR PkIdent IN (
                 SELECT b.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result <> a.Result 
				 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <=  @maxMinutes
                 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time > = @startTime AND a.Time < @endTime
                 AND b.ThisLast = 1 
                 ) 

            -- first and subsequent tests are WITHIN the time frame 
            -- if the result HAS changed count the subsequent test
            -- last test within maxHours which is NOT the last test overall
            OR PkIdent IN (
                 SELECT b.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result <> a.Result 
				 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <= @maxMinutes
                 AND DATEDIFF(mi, a.[Time], b.[TimeOfNextMeasurement]) >  @maxMinutes
                 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time > = @startTime AND a.Time < @endTime
                 ))

		GROUP BY t.Id, b.Id WITH ROLLUP -- cube
		HAVING GROUPING (t.Id) = 0 AND  GROUPING (b.Id) = 0 -- avoid null entries

END
--Only the max hours condition is met
ELSE IF (@maxOrdinal = 0 AND @maxMinutes > 0)
BEGIN
		 INSERT INTO @TempDetails (
		[WeekNr],
		[Type],
		[TestBench],
		[CountType],
		[All],
		[Good],
		[Bad]
		)
		SELECT
		@weekNr,
		t.Id,
		b.Id,
		@thisCountType,
		COUNT(*),
		SUM(dbo.HasResult(MainData.Result,1)),
		SUM(dbo.HasResult(MainData.Result,2)) + SUM(dbo.HasResult(MainData.Result,4))
		FROM
		Maindata
		INNER JOIN dbo.TypeID t ON MainData.Type = t.Id
		INNER JOIN dbo.BenchId b ON MainData.TestBench = b.Id
		WHERE
		t.Id > 0 AND b.Id > 0 -- not the '*' entries
		AND MainData.Serial NOT IN ( SELECT [Name] FROM [ReferenceSerials])
		AND t.Name NOT IN ( SELECT [Name] FROM [ReferenceTypes])
		AND (MainData.TestKind = 1)
		AND  ((MainData.Time >= @startTime)  AND  (MainData.Time < @endTime))
		
		-- Only Test
		AND (((MainData.ThisLast = 1) AND (MainData.Ordinal = 1))
		
			-- First test within the time slot but 2nd test outside the maxHours
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind 
                 WHERE 
                 a.Ordinal = 1 AND b.Ordinal = 2 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) >  @maxMinutes
                 AND a.Time > = @startTime AND a.Time < @endTime 
                 ) 

            -- first test is BEFORE the time frame and subsequent tests are WITHIN the time frame  
            -- if the result HAS changed count the subsequent test
            -- last test within maxHours is also the last test overall 
            OR PkIdent IN (
                 SELECT b.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result <> a.Result 
                 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <=  @maxMinutes
                 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time < @startTime 
                 AND b.ThisLast = 1 
                 ) 

            -- first test is BEFORE the time frame and subsequent tests are WITHIN the time frame  
            -- if the result HAS changed count the subsequent test
            -- last test within maxHours which is NOT the last test overall
            OR PkIdent IN (
                 SELECT b.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result <> a.Result 
                 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <= @maxMinutes
                 AND DATEDIFF(mi, a.[Time], b.[TimeOfNextMeasurement]) >  @maxMinutes
                 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time < @startTime
                 )

			-- first test is WITHIN the time frame and subsequent tests are AFTER the time frame  
            -- if the result has NOT changed count the first test
            -- last test within maxHours which is also the last test overall
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result = a.Result 
                 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <=  @maxMinutes
                 AND a.Time > = @startTime AND a.Time < @endTime
				 AND b.Time > = @endTime 
                 AND b.ThisLast = 1 
                 ) 

            -- first test is WITHIN the time frame and subsequent tests are AFTER the time frame  
            -- if the result has NOT changed count the first test
            -- last test within maxHours which is NOT the last test overall
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result = a.Result 
                 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <= @maxMinutes
                 AND DATEDIFF(mi, a.[Time], b.[TimeOfNextMeasurement]) >  @maxMinutes
                 AND a.Time > = @startTime AND a.Time < @endTime
				 AND b.Time > = @endTime 
                 )

			-- first and subsequent tests are WITHIN the time frame 
            -- if the result has NOT changed count the first test
            -- last test within maxHours which is also the last test overall
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result = a.Result 
				 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <=  @maxMinutes
                 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time > = @startTime AND a.Time < @endTime
                 AND b.ThisLast = 1 
                 ) 

            -- first and subsequent tests are WITHIN the time frame 
            -- if the result has NOT changed count the first test
            -- last test within maxHours which is NOT the last test overall
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result = a.Result 
				 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <= @maxMinutes
                 AND DATEDIFF(mi, a.[Time], b.[TimeOfNextMeasurement]) >  @maxMinutes
                 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time > = @startTime AND a.Time < @endTime
                 )

			-- first and subsequent tests are WITHIN the time frame 
            -- if the result HAS changed count the subsequent test
            -- last test within maxHours which is also the last test overall
            OR PkIdent IN (
                 SELECT b.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result <> a.Result 
				 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <=  @maxMinutes
                 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time > = @startTime AND a.Time < @endTime
                 AND b.ThisLast = 1 
                 ) 

            -- first and subsequent tests are WITHIN the time frame 
            -- if the result HAS changed count the subsequent test
            -- last test within maxHours which is NOT the last test overall
            OR PkIdent IN (
                 SELECT b.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result <> a.Result 
				 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <= @maxMinutes
                 AND DATEDIFF(mi, a.[Time], b.[TimeOfNextMeasurement]) >  @maxMinutes
                 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time > = @startTime AND a.Time < @endTime
                 ))

		GROUP BY t.Id, b.Id WITH ROLLUP -- cube
		HAVING GROUPING (t.Id) = 0 AND  GROUPING (b.Id) = 0 -- avoid null entries

END

IF (@isUpdate = 1)
BEGIN
BEGIN TRAN
DELETE FROM [WeeklyResultDetails] WHERE [WeeklyResultDetails].[WeekNr] = @weekNr AND [CountType] = @thisCountType
INSERT INTO [WeeklyResultDetails] SELECT * FROM @TempDetails
COMMIT
END ELSE
INSERT INTO [WeeklyResultDetails] SELECT * FROM @TempDetails
END -- procedure spDirectTestGroup

GO

------------------------------------------------------------------------------------------------------------------------------------------

PRINT 'Stored procedure spDirectTestGroup added'
GO
