
------------------------------------------------------------------------------------------------------------------------------------------
------------ Procedure: spDsFillWeeklyData
------------------------------------------------------------------------------------------------------------------------------------------

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[spDsFillWeeklyData]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
	DROP PROCEDURE [dbo].spDsFillWeeklyData
GO

/*
Fill WeeklyResults table
A week starts with the first shift offset from Sunday 0:0 + @shiftStartMinutes
and ends with next weeks Sunday + @shiftEndMinutes
IsoWeek And IsoYear are entered per calculation based on the (first) @weekday
Weeks are counted from the base of times (which is 1990-01-01)
*/
CREATE PROCEDURE [dbo].[spDsFillWeeklyData] (@startTime datetime, @endTime datetime, @fixDaysOffset int, @maxMinutes int, @maxOrdinal int) AS
BEGIN
SET NOCOUNT ON
DECLARE
@weekCnt int,
@startWeek int,
@endWeek int,
@shiftStartMinutes int,
@shiftEndMinutes int,
@now datetime,
@update int
set @now = GETDATE()
EXEC dbo.[parSetTime] @key ='WeekData.LastRun',  @setTime =  @now
-- shift start and end minutes, end minutes minus one day
SELECT  @shiftStartMinutes = MIN([Day]*24*60 + [Start]) from ShiftMinutes
SELECT  @shiftEndMinutes =   MAX(([Day]-1)*24*60 + [Stop]) from ShiftMinutes
set @startWeek = DATEDIFF(wk, 0,  @startTime)
set @endWeek =   DATEDIFF(wk, 0,  @endTime)
set @weekCnt = @startWeek
DECLARE @start datetime, @end datetime, @weekday datetime
DECLARE @totalCnt int
WHILE @weekCnt <= @endWeek
BEGIN
set @end =   DATEADD(minute,@shiftEndMinutes,DATEADD(wk,1,(7*@weekCnt)-1))
IF ( ( ( SELECT COUNT(*) FROM [WeeklyResults] WHERE [WeeklyResults].[WeekNr] = @weekCnt) = 0 )
OR ( DATEADD(day,@fixDaysOffset,@end) > @now)
)
BEGIN
-- There is no entry in [WeeklyResults] or it can still be changed
set @start = DATEADD(minute,@shiftStartMinutes,DATEADD(wk,0,(7*@weekCnt)-1))		-- StartTime: Day 0, add shift time
set @weekday =  DATEADD(day,1,@start)

-- Account for negative @shiftStartMinutes (otherwise the IsoWeek determined will NOT be the current calendar week)  
IF (@shiftStartMinutes < 0) set @weekday =  DATEADD(day,1,DATEADD(minute,0,DATEADD(wk,0,(7*@weekCnt)-1)))	
	
IF (( (DATEADD(day,@fixDaysOffset,@end) > @now)   AND ( SELECT COUNT(*) FROM [WeeklyResults] WHERE [WeeklyResults].[WeekNr] = @weekCnt) > 0 ) )
BEGIN
-- an update operation
SET @update = 1
UPDATE [WeeklyResults] SET
[Year] = dbo.IsoYear(@weekday),
[WeekInYear] =  dbo.IsoWeek(@weekday),
[StartTime] = @start,
[EndTime] = @end
WHERE WeekNr = @weekCnt
END ELSE
BEGIN
-- an insert operation
SET @update = 0
INSERT INTO [WeeklyResults] ([WeekNr],[Year],[WeekInYear],[StartTime],[EndTime])
VALUES (
@weekCnt,
dbo.IsoYear(@weekday),
dbo.IsoWeek(@weekday),
@start,
@end
)
END
-- generate sum terms for test stand and type groups for the different counting methods
EXEC dbo.spAllTestsGroup
@startTime =@start,
@endTime = @end,
@fixDaysOffset = @fixDaysOffset,
@weekNr = @weekCnt,
@isUpdate = @update
EXEC dbo.spFirstTestGroup
@startTime =@start,
@endTime = @end,
@fixDaysOffset = @fixDaysOffset,
@weekNr = @weekCnt,
@isUpdate = @update
EXEC dbo.spLastTestGroup
@startTime =@start,
@endTime = @end,
@fixDaysOffset = @fixDaysOffset,
@weekNr = @weekCnt,
@isUpdate = @update
EXEC dbo.spCurrentRejectGroup
@startTime =@start,
@endTime = @end,
@fixDaysOffset = @fixDaysOffset,
@weekNr = @weekCnt,
@isUpdate = @update
EXEC dbo.spDirectTestGroup
@startTime =@start,
@endTime = @end,
@fixDaysOffset = @fixDaysOffset,
@weekNr = @weekCnt,
@isUpdate = @update,
@maxMinutes = @maxMinutes,
@maxOrdinal = @maxOrdinal
EXEC dbo.spTestRollup				-- generate sum terms for test stands and types
@weekNr = @weekCnt
SELECT @totalCnt = 0
SELECT @totalCnt = [All] FROM [WeeklyResultDetails] WHERE [Type]= 0 AND [TestBench] = 0 AND [WeekNr] = @weekCnt AND [CountType] = 2
IF (NOT @totalCnt IS NULL)
Print ('Units:' + CAST(@totalCnt AS nvarchar ) + ' Week: ' + CAST(@weekCnt AS nvarchar) + ' Start: ' + CAST(@start AS nvarchar) + ' End: ' + CAST(@end AS nvarchar))
END --there is no entry or it can still be changed
SET @weekCnt = @weekCnt+1
END
END -- procedure spDsFillWeeklyData

GO
------------------------------------------------------------------------------------------------------------------------------------------

PRINT 'Stored procedure spDsFillWeeklyData added'
GO