

---------------------------------------------------------------------------------------------------------------------------------------------
-- Stored procedure DeleteData
---------------------------------------------------------------------------------------------------------------------------------------------
-- Procedure which deletes from ProtocolData based on the table TimeLimitOfTypes
---------------------------------------------------------------------------------------------------------------------------------------------

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[DeleteData]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
	DROP PROCEDURE [dbo].[DeleteData]
GO
---------------------------------------------------------------------------------------------------------------------------------------------

CREATE PROCEDURE dbo.DeleteData @del int
AS
SET NOCOUNT ON;

DECLARE 
	@lim_id int, 
	@lim_name varchar(50), 
	@lim_actualTime dateTime,
	@lim_previousTime dateTime,
	@lim_number int,
	@lim_deleted int, 
	@lineMessage varchar(300),
	@del_Protocol int,
	@del_Start dateTime,
	@del_Stop time,
	@del_number int,
	@del_offset int, 
	@lim_del int,
	@temp_previousTime dateTime,
	@status int

-- Include only entries from TimeLimitOfTypes that are not flagged as already deleted.		
DECLARE lim_cursor CURSOR FOR 
SELECT Id, Name, ActualTimeLimit, PreviousTimeLimit, UnitsToDelete, Deleted
	FROM TimeLimitsOfTypes
	WHERE Deleted = 0   
	ORDER BY Id;

OPEN lim_cursor

FETCH NEXT FROM lim_cursor 
INTO @lim_id, @lim_name, @lim_actualTime, @lim_previousTime, @lim_number, @lim_deleted

SELECT @status = @@FETCH_STATUS

WHILE @@FETCH_STATUS = 0
BEGIN
	
---------------------------------------------------------------------------------------------------------------------------------------------
-- Deletion from MainData and its associated tables
---------------------------------------------------------------------------------------------------------------------------------------------
	IF (@lim_id = 0) BEGIN
		
		-- Local variable which stores for a specific type the number of units which are left after a routine deletion 
		-- Default = 0
		SELECT @lim_del = 0
		
		-- Local variable that stores the system time before the delete statement is submitted to the SQL-Server
		SELECT @del_Start = GETDATE()
	
		-- Number of units left after deleting @del number of units 
		SELECT @lim_del = @lim_number - @del	
		-- In case @del (number of units to be deleted) is greater than the actual number of units 
		-- to be deleted for a specific type, @del is set to this actual number.  
		IF (@lim_del < 0) SELECT @del = @del + @lim_del	 	 

		-- Actual delete statement
		DELETE FROM ErrorData
		FROM ErrorData INNER JOIN MainData ON ErrorData.FkIdent = MainData.PkIdent  
		WHERE PkIdent IN
			(SELECT TOP (@del) PkIdent 
			FROM MainData
			WHERE MainData.Time < @lim_actualTime 
			ORDER BY Time ASC);	
		-- In case an error occurs, store the error id in the local variable @del_Protocol 
		IF (@@ERROR <> 0) SELECT @del_Protocol = @@ERROR
		
		-- Actual delete statement
		DELETE FROM ArchiveData
		FROM ArchiveData INNER JOIN MainData ON ArchiveData.FkIdent = MainData.PkIdent  
		WHERE PkIdent IN
			(SELECT TOP (@del) PkIdent 
			FROM MainData
			WHERE MainData.Time < @lim_actualTime 
			ORDER BY Time ASC);	
		-- In case an error occurs, store the error id in the local variable @del_Protocol 
		IF (@@ERROR <> 0) SELECT @del_Protocol = @@ERROR
		
		-- Actual delete statement
		DELETE FROM AdditionalInfo
		FROM AdditionalInfo INNER JOIN MainData ON AdditionalInfo.FkIdent = MainData.PkIdent  
		WHERE PkIdent IN
			(SELECT TOP (@del) PkIdent 
			FROM MainData
			WHERE MainData.Time < @lim_actualTime 
			ORDER BY Time ASC);	
		-- In case an error occurs, store the error id in the local variable @del_Protocol 
		IF (@@ERROR <> 0) SELECT @del_Protocol = @@ERROR
		
		-- Actual delete statement
		DELETE FROM ModesPerMeasurement
		FROM ModesPerMeasurement INNER JOIN MainData ON ModesPerMeasurement.FkIdent = MainData.PkIdent  
		WHERE PkIdent IN
			(SELECT TOP (@del) PkIdent 
			FROM MainData
			WHERE MainData.Time < @lim_actualTime 
			ORDER BY Time ASC);
		-- In case an error occurs, store the error id in the local variable @del_Protocol 
		IF (@@ERROR <> 0) SELECT @del_Protocol = @@ERROR
		
		-- Actual delete statement
		DELETE FROM ProtocolData
		FROM ProtocolData INNER JOIN MainData ON ProtocolData.FkIdent = MainData.PkIdent  
		WHERE PkIdent IN
			(SELECT TOP (@del) PkIdent 
			FROM MainData
			WHERE MainData.Time < @lim_actualTime 
			ORDER BY Time ASC);		
		-- Count number of entries that are affected by the deletion
		SELECT @del_number = @@ROWCOUNT
		-- In case an error occurs, store the error id in the local variable @del_Protocol 
		IF (@@ERROR <> 0) SELECT @del_Protocol = @@ERROR

		-- Actual delete statement
		DELETE FROM MainData 
		WHERE PkIdent IN
			(SELECT TOP (@del) PkIdent 
			FROM MainData
			WHERE MainData.Time < @lim_actualTime 
			ORDER BY Time ASC);
		-- In case an error occurs, store the error id in the local variable @del_Protocol 
		IF (@@ERROR <> 0) SELECT @del_Protocol = @@ERROR
		
		-- In case there are still units left to be deleted for the respective TypeId, then update table and exit script. 
		IF (@lim_del >= 0) BEGIN 
		UPDATE [TimeLimitsOfTypes]
    		SET UnitsToDelete = @lim_del
			WHERE TimeLimitsOfTypes.Id = @lim_id
		If (@lim_del = 0) BEGIN
			UPDATE [TimeLimitsOfTypes]
				SET Deleted = 1
    			WHERE TimeLimitsOfTypes.Id = @lim_id
		END		
		SELECT @del_Stop = GETDATE() - @del_Start
		-- Write into table 'DeleteLog'
		INSERT INTO [DeleteLog] (Day, TimeLimit, Duration, Type, DeletedMain, DeletedProtocol, ErrorId) VALUES (GetDate(), @lim_actualTime, @del_stop, @lim_id, @del, @del_number, @del_Protocol )	 
		GOTO SCRIPT_EXIT 
	END

	-- In case the number of units to be deleted is greater than the actual units for the respective TypeId, 
	-- the cursor gets the next line and deletes from the next Type.Id the remaining number of daily deletes (@lim_del) 
	IF (@lim_del < 0) BEGIN
		UPDATE [TimeLimitsOfTypes]
    		SET UnitsToDelete = 0	
			WHERE TimeLimitsOfTypes.Id = @lim_id
		UPDATE [TimeLimitsOfTypes]
			SET Deleted = 1
    		WHERE TimeLimitsOfTypes.Id = @lim_id
		SELECT @del_Stop = GETDATE() - @del_Start
		INSERT INTO [DeleteLog] (Day, TimeLimit, Duration, Type, DeletedMain, DeletedProtocol, ErrorId) VALUES (GetDate(), @lim_actualTime, @del_stop, @lim_id, @del, @del_number, @del_Protocol )	 
		SELECT @del = abs(@lim_del)	 
	END

    -- Get the next line.
    FETCH NEXT FROM lim_cursor 
	INTO @lim_id, @lim_name, @lim_actualTime, @lim_previousTime, @lim_number, @lim_deleted
	END

---------------------------------------------------------------------------------------------------------------------------------------------
-- Deletion from ProtocolData
---------------------------------------------------------------------------------------------------------------------------------------------
	IF (@lim_id <> 0) BEGIN
	-- Local variable that stores for a specific type the number of units which are left after a routine deletion (default: 0)
	SELECT @lim_del = 0

	-- Local variable that stores the system time before the delete statement is submitted to the SQL-Server
	SELECT @del_Start = GETDATE()
	
	-- Number of units left after deleting @del number of units 
	SELECT @lim_del = @lim_number - @del	
	-- In case @del (number of units to be deleted) is greater than the actual number of units 
	-- to be deleted for a specific type, @del is set to this actual number.  
	IF (@lim_del < 0) SELECT @del = @del + @lim_del	 	 
	
	-- Extract time of the first measurement which follows the delete package (@del) => identified by the '>' operator!
	-- This is set to the new previous time limit for the next delete step. May be NULL.  
	SELECT @temp_previousTime = MAX(t) FROM 
		(SELECT TOP (@del) PkIdent, Time as t
		FROM MainData
		WHERE MainData.Time < @lim_actualTime AND MainData.Time > @lim_previousTime AND MainData.Type = @lim_id
		ORDER BY Time ASC) as dummy

	-- Actual delete statement
	DELETE FROM ProtocolData
	FROM ProtocolData INNER JOIN MainData ON ProtocolData.FkIdent = MainData.PkIdent  
	WHERE PkIdent IN
		(SELECT TOP (@del) PkIdent 
		FROM MainData
		WHERE MainData.Time < @lim_actualTime AND MainData.Time >= @lim_previousTime AND MainData.Type = @lim_id
		ORDER BY Time ASC);	
	-- Count number of entries in ProtocolData that are affected by the deletion
	SELECT @del_number = @@ROWCOUNT
	-- In case an error occurs, store the error id in the local variable @del_Protocol 
	IF (@@ERROR <> 0) SELECT @del_Protocol = @@ERROR
		
	-- In case there are still units left to be deleted for the respective TypeId, then update table and exit script. 
	IF (@lim_del > 0) BEGIN 
		UPDATE [TimeLimitsOfTypes]
    		SET PreviousTimeLimit = @temp_previousTime
			WHERE TimeLimitsOfTypes.Id = @lim_id
		UPDATE [TimeLimitsOfTypes]
    		SET UnitsToDelete = @lim_del
			WHERE TimeLimitsOfTypes.Id = @lim_id	
		SELECT @del_Stop = GETDATE() - @del_Start
		INSERT INTO [DeleteLog] (Day, TimeLimit, Duration, Type, DeletedMain, DeletedProtocol, ErrorId) VALUES (GetDate(), @lim_actualTime, @del_stop, @lim_id, @del, @del_number, @del_Protocol )	 
		GOTO SCRIPT_EXIT 
	END

		-- In case there are exactly 0 units left to be deleted for the respective TypeId, then update table and exit script. 
	IF (@lim_del = 0) BEGIN 
		UPDATE [TimeLimitsOfTypes]
    		SET PreviousTimeLimit = @lim_actualTime
			WHERE TimeLimitsOfTypes.Id = @lim_id
		UPDATE [TimeLimitsOfTypes]
    		SET UnitsToDelete = 0	
			WHERE TimeLimitsOfTypes.Id = @lim_id
		UPDATE [TimeLimitsOfTypes]
			SET Deleted = 1
    		WHERE TimeLimitsOfTypes.Id = @lim_id
		SELECT @del_Stop = GETDATE() - @del_Start 
		INSERT INTO [DeleteLog] (Day, TimeLimit, Duration, Type, DeletedMain, DeletedProtocol, ErrorId) VALUES (GetDate(), @lim_actualTime, @del_stop, @lim_id, @del, @del_number, @del_Protocol )	 
		GOTO SCRIPT_EXIT 
	END

	-- In case the number of units to be deleted is greater than the actual units for the respective TypeId, 
	-- the cursor gets the next line and deletes from the next Type.Id the remaining number of daily deletes (@lim_del) 
	IF (@lim_del < 0) BEGIN
		UPDATE [TimeLimitsOfTypes]
    		SET PreviousTimeLimit = @lim_actualTime
			WHERE TimeLimitsOfTypes.Id = @lim_id
		UPDATE [TimeLimitsOfTypes]
    		SET UnitsToDelete = 0	
			WHERE TimeLimitsOfTypes.Id = @lim_id
		UPDATE [TimeLimitsOfTypes]
			SET Deleted = 1
    		WHERE TimeLimitsOfTypes.Id = @lim_id
		SELECT @del_Stop = GETDATE() - @del_Start 
		INSERT INTO [DeleteLog] (Day, TimeLimit, Duration, Type, DeletedMain, DeletedProtocol, ErrorId) VALUES (GetDate(), @lim_actualTime, @del_stop, @lim_id, @del, @del_number, @del_Protocol )	 
		SELECT @del = abs(@lim_del)	 
	END

    -- Get the next line.
    FETCH NEXT FROM lim_cursor 
	INTO @lim_id, @lim_name, @lim_actualTime, @lim_previousTime, @lim_number, @lim_deleted 
	
	END

END

SCRIPT_EXIT:
If (@status = -1) PRINT 'Table TimeLimitsOfTypes does not specify data to delete.'
ELSE PRINT 'Data deletion completed with ' + CAST(@del_number AS varchar) + ' entries deleted from ProtocolData.' 

-- Checks whether there are any units left to be deleted. If none are left, the flag 'renew' in 'DeleteParameters' is set to 1.
DECLARE @count int
SELECT @count = (SELECT COUNT (*) FROM TimeLimitsOfTypes 
				WHERE TimeLimitsOfTypes.UnitsToDelete <> 0)
If (@count = 0) Begin
	Update [DeleteParameters]
	Set [RenewLimits] = 1
	PRINT 'RenewLimits set to 1: The renewal of the TimeLimitsOfTypes will be triggered by the next delete cylce.'
End

CLOSE lim_cursor
DEALLOCATE lim_cursor

GO
---------------------------------------------------------------------------------------------------------------------------------------------

PRINT 'Stored procedure added'
GO