﻿

---------------------------------------------------------------------------------------------------------------------------------------------
-- Stored procedure TimeLimitsPreview
---------------------------------------------------------------------------------------------------------------------------------------------
-- Procedure which updates the TimeLimitsOfTypesPreview-table
---------------------------------------------------------------------------------------------------------------------------------------------
-- This procedure is intended to provide a preview of the changes that come with changing delete parameters  

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[TimeLimitsPreview_TypeAndBench]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
	DROP PROCEDURE [dbo].[TimeLimitsPreview_TypeAndBench]
GO
---------------------------------------------------------------------------------------------------------------------------------------------

CREATE PROCEDURE [dbo].[TimeLimitsPreview_TypeAndBench] @trans int, @months int, @main int, @prot int, @lim int
AS
SET NOCOUNT ON;

---------------------------------------------------------------------------------------------------------------------------------------------
-- Declaration and initialisation of variables
---------------------------------------------------------------------------------------------------------------------------------------------
DECLARE @count int,
		@date dateTime,
		@minTime dateTime

SELECT	@minTime = (SELECT Min(Time) FROM MainData)

-- In case the database contains no data, Min(Time) equals null. 
-- In this case, minTime is set to the minimum of the two time limits (TimeLimitProtocol and TimeLimitMain). 
IF (@minTime IS NULL) BEGIN
	SELECT	@minTime = MIN(minTime) FROM (SELECT DATEADD("Month",-(@months),GETDATE()) as minTime UNION SELECT DATEADD("Month",-(@lim),GETDATE())) as minTime
END
---------------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------------
-- Replicate TimeLimitsOfTypesPreview to store the original PreviousTimeLimit values before calculating the new TimeLimitOfTypes 		
---------------------------------------------------------------------------------------------------------------------------------------------
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[TimeLimitsOfTypesPreview]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP TABLE [dbo].[TimeLimitsOfTypesPreview]
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[TimeLimitsOfTypesPreview2]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP TABLE [dbo].[TimeLimitsOfTypesPreview2]

-- Replicate TimeLimitsOfTypesPreview 		
SELECT * INTO TimeLimitsOfTypesPreview FROM TimeLimitsOfTypes
SELECT * INTO TimeLimitsOfTypesPreview2 FROM TimeLimitsOfTypes  

-- Remove all rows from TimeLimitsOfTypesPreview table
TRUNCATE TABLE TimeLimitsOfTypesPreview

---------------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------------
-- Check for the different deletion cases and calculation of the ActualTimeLimit and PreviousTimeLimit
---------------------------------------------------------------------------------------------------------------------------------------------
IF (@main = 1) BEGIN
	INSERT INTO [TimeLimitsOfTypesPreview] (Id, Name, BenchId, BenchName, ActualTimeLimit, PreviousTimeLimit)
	SELECT 0, '-', 0, '-', (SELECT DATEADD("MONTH", -(@lim), GETDATE())), @minTime

	UPDATE TimeLimitsOfTypesPreview
	SET UnitsToDelete = (SELECT COUNT (*) FROM MainData WHERE MainData.Time < TimeLimitsOfTypesPreview.ActualTimeLimit)

	UPDATE TimeLimitsOfTypesPreview
	SET UnitsRetained = (SELECT COUNT (*) FROM MainData WHERE MainData.Time >= TimeLimitsOfTypesPreview.ActualTimeLimit) 
END

-- Case 1: Only the @trans number of units are kept which were measured within the last @months months.  
IF (@prot = 1) BEGIN
	INSERT INTO [TimeLimitsOfTypesPreview] (Id, Name, BenchId, BenchName, ActualTimeLimit, PreviousTimeLimit)
	SELECT DISTINCT Type, TypeID.Name, TestBench, BenchID.Name, 
		(select MAX(mt) from
		(
			SELECT MIN(time) as mt FROM 
			(SELECT TOP (@trans) time FROM MainData
			WHERE type = TypeId.Id AND TestBench = BenchID.Id 		 
			ORDER BY [time] DESC) topN 
			union 
			select  DATEADD("Month", -(@months), GETDATE()) as mt
		) topN) as timelimit ,
	(select MIN(time) FROM MainData WHERE type = TypeId.Id AND TestBench = BenchID.Id ) as mintime
	FROM MainData INNER JOIN TypeID ON TypeID.Id = MainData.Type INNER JOIN BenchID ON BenchID.Id = MainData.TestBench
	ORDER BY Type, TestBench	
END

-- Case 2: In addition to keeping all units measured within the last @months months, this case guarantees that at least @trans number of units are kept. 
IF (@prot = 2) BEGIN
	INSERT INTO [TimeLimitsOfTypesPreview] (Id, Name, BenchId, BenchName, ActualTimeLimit, PreviousTimeLimit)
	SELECT DISTINCT Type, TypeID.Name, TestBench, BenchID.Name,
		(select MIN(mt) from
		(
			SELECT MIN(time) as mt FROM 
			(SELECT TOP (@trans) time FROM MainData
			WHERE type = TypeId.Id AND TestBench = BenchID.Id 		 
			ORDER BY [time] DESC) topN 
			union 
			select  DATEADD("Month", -(@months), GETDATE()) as mt
		) topN) as timelimit ,
	(select MIN(time) FROM MainData WHERE type = TypeId.Id AND TestBench = BenchID.Id) as mintime
	FROM MainData INNER JOIN TypeID ON TypeID.Id = MainData.Type INNER JOIN BenchID ON BenchID.Id = MainData.TestBench
	ORDER BY Type, TestBench		
END

-- Case 3: Only the last @trans number of units are kept.
IF (@prot = 3) BEGIN
	INSERT INTO [TimeLimitsOfTypesPreview] (Id, Name, BenchId, BenchName, ActualTimeLimit, PreviousTimeLimit)
	SELECT DISTINCT Type, TypeID.Name, TestBench, BenchID.Name,
		(select MIN(mt) from
		(
			SELECT MIN(time) as mt FROM 
			(SELECT TOP (@trans) time FROM MainData 
			WHERE type = TypeId.Id AND TestBench = BenchID.Id 		 
			ORDER BY [time] DESC) topN 
		) topN) as timelimit ,
		(select MIN(time) FROM MainData WHERE type = TypeId.Id AND TestBench = BenchID.Id) as mintime
	FROM MainData INNER JOIN TypeID ON TypeID.Id = MainData.Type INNER JOIN BenchID ON BenchID.Id = MainData.TestBench
	ORDER BY Type, TestBench	
END

-- Case 4: All units measured within the last @months number of months are kept.
IF (@prot = 4) BEGIN
	INSERT INTO [TimeLimitsOfTypesPreview] (Id, Name, BenchId, BenchName, ActualTimeLimit, PreviousTimeLimit)
	SELECT DISTINCT Type, TypeID.Name, TestBench, BenchID.Name,
		(SELECT DATEADD("MONTH", -(@months), GETDATE())) ,
		(SELECT MIN(time) FROM MainData WHERE type = TypeId.Id AND TestBench = BenchID.Id) as mintime
	FROM MainData INNER JOIN TypeID ON TypeID.Id = MainData.Type INNER JOIN BenchID ON BenchID.Id = MainData.TestBench
	ORDER BY Type, TestBench	
END
---------------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------------
-- Type-specific deletion: Fill in original PreviousTimeLimit values   
---------------------------------------------------------------------------------------------------------------------------------------------
IF (@prot <> 0) BEGIN
	UPDATE [TimeLimitsOfTypesPreview]
	SET TimeLimitsOfTypesPreview.PreviousTimeLimit = TimeLimitsOfTypesPreview2.PreviousTimeLimit
	FROM TimeLimitsOfTypesPreview INNER JOIN TimeLimitsOfTypesPreview2 On TimeLimitsOfTypesPreview.Id = TimeLimitsOfTypesPreview2.Id
	AND TimeLimitsOfTypesPreview.BenchId = TimeLimitsOfTypesPreview2.BenchId 
	WHERE TimeLimitsOfTypesPreview.Id <> 0		
END
---------------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------------
-- MainData deletion: In case data is deleted from both ProtocolData and MainData  
---------------------------------------------------------------------------------------------------------------------------------------------
-- If the PreviousTimeLimit (PTL) of the type-specific deletion is less than the ActualTimeLimit of the MainData deletion, 
-- then set the PreviousTimeLimit to the ActualTimeLimit
IF (@main = 1) BEGIN
	SELECT @date = (SELECT ActualTimeLimit FROM TimeLimitsOfTypesPreview WHERE Id = 0)
	UPDATE [TimeLimitsOfTypesPreview]
	SET PreviousTimeLimit = @date
	WHERE Id <> 0 AND PreviousTimeLimit <= @date
END
---------------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------------
-- Type-specific deletion: Validity queries and calculation of the units to delete and to keep
---------------------------------------------------------------------------------------------------------------------------------------------
IF (@prot <> 0) BEGIN

-- If one of the queries above returns a null value for the ActualTimeLimit (e.g. in case there are no measurements 
-- for a type which has already been initialised), then set the ActualTimeLimit to @minTime
	UPDATE [TimeLimitsOfTypesPreview]
		Set PreviousTimeLimit = @minTime
		WHERE PreviousTimeLimit IS NULL
	UPDATE [TimeLimitsOfTypesPreview]
		Set ActualTimeLimit = @minTime
		WHERE ActualTimeLimit IS NULL

-- In case the ActualTimeLimit is less than the PreviousTimeLimit then set ActualTimeLimit to the PreviousTimeLimit
-- Otherwise the following calculation will result in an erroneous number of kept units
	UPDATE [TimeLimitsOfTypesPreview]
		Set ActualTimeLimit = PreviousTimeLimit
		WHERE ActualTimeLimit < PreviousTimeLimit

-- Calculation of the number of units to be deleted and to be kept 
	UPDATE [TimeLimitsOfTypesPreview]
	SET UnitsToDelete = (SELECT COUNT (*) FROM MainData WHERE MainData.Type = TimeLimitsOfTypesPreview.Id AND
		MainData.TestBench = TimeLimitsOfTypesPreview.BenchId AND
		MainData.Time < TimeLimitsOfTypesPreview.ActualTimeLimit AND MainData.Time >= TimeLimitsOfTypesPreview.PreviousTimeLimit)
	WHERE Id <> 0
	UPDATE [TimeLimitsOfTypesPreview]
	SET UnitsRetained = (SELECT COUNT (*) FROM MainData WHERE MainData.Type = TimeLimitsOfTypesPreview.Id AND
		MainData.TestBench = TimeLimitsOfTypesPreview.BenchId AND 
		MainData.Time >= TimeLimitsOfTypesPreview.ActualTimeLimit)
	WHERE Id <> 0
END
---------------------------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------------------------
-- Set delete flag of TimeLimitsOfTypesPreview
---------------------------------------------------------------------------------------------------------------------------------------------
-- In case the UnitsToDelete is zero (for a given type), then set column 'Deleted' to 1 
UPDATE [TimeLimitsOfTypesPreview]
SET Deleted = 1
WHERE TimeLimitsOfTypesPreview.UnitsToDelete = 0

-- In case the UnitsToDelete is NOT zero (for a given type), then set column 'Deleted' to 1 
UPDATE [TimeLimitsOfTypesPreview]
SET Deleted = 0
WHERE TimeLimitsOfTypesPreview.UnitsToDelete <> 0
---------------------------------------------------------------------------------------------------------------------------------------------

-- Drop duplicate table TimeLimitsOfTypesPreview2 
DROP TABLE TimeLimitsOfTypesPreview2

GO
---------------------------------------------------------------------------------------------------------------------------------------------

PRINT 'Stored procedure added'
GO