
------------------------------------------------------------------------------------------------------------------------------------------
------------ Procedure: SpGmkErrorReportOfSerial
------------------------------------------------------------------------------------------------------------------------------------------

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[SpGmkErrorReportOfSerial]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
	DROP PROCEDURE [dbo].[SpGmkErrorReportOfSerial]
GO

CREATE PROC [dbo].[SpGmkErrorReportOfSerial]
	@serial		nvarchar (200),			-- serial number
	@whichTest	int,					-- 0: All tests, -1: The last test, else the specific test
	@showCategories	int,				-- 1: Show error category numbetr and text  else show error code and text
	@completeReport	int					-- 1: Show complete report line for each error, else only show shortened error line

AS

-- ShowCategories? Which test entry?
DECLARE @errorCondition nvarchar (100)
DECLARE @whichTestCondition nvarchar (100)


-- Categories
SET @errorCondition = ''
IF @showCategories = 0
BEGIN
  SET @errorCondition = N'
  	 Nr As Nr,
	 ErrorCode AS ErrorCode,
	 ErrorText AS ErrorText,'
END
ELSE
BEGIN
  SET @errorCondition = N'
     Nr As Nr,
	 ErrorCategoryCode AS ErrorCode,
	 ErrorCategoryText AS ErrorText,'
END

-- WhichTest

IF @whichTest = -1	-- Last Test
BEGIN
  SET @whichTestCondition = N'
	 AND [ThisLast] = 1'
END
ELSE IF @whichTest = 0  --All tests
BEGIN
SET @whichTestCondition = ''
END
ELSE
BEGIN
SET @whichTestCondition = N'
	 AND [NTest] = ' + CONVERT(char(15),@whichTest)
END



-- Define Query
DECLARE @query nvarchar (4000)

-- Short or full error line
IF @completeReport = 1
BEGIN
SET @query = N'
	SELECT
	NTest,
	 TestBench,
	 Type,
	 BaseType,
	 [Time],'
       + @errorCondition
       + N'
	 Mode,
	 Value,
	 Limit,
	 Unit,
	 Instrument,
	 Channel,
	 Position
	FROM [SpGmkCompleteErrorReport]
	WHERE [Serial] = @serial'
      + @whichTestCondition
       + N'  
	ORDER BY NTest, TestBench, Type, [Time], Nr'
END
ELSE
BEGIN

	SET @query = N'
	SELECT
	 NTest,
	 Nr,
	 ErrorCode,
	 ErrorText	
	FROM [SpGmkShortErrorReport]
	WHERE [Serial] = @serial'
      + @whichTestCondition
      + N'
	ORDER BY NTest, Nr'
END


-- execute @query
EXEC sp_executesql @query,

	-- 1. Parameter deklarieren
      N'@serial		nvarchar (200),
	@whichTest	int,
	@showCategories	int',

        -- 2. Werte bergeben
	@serial,
	@whichTest,
	@showCategories
GO
------------------------------------------------------------------------------------------------------------------------------------------

PRINT 'Stored procedure added'
GO