
------------------------------------------------------------------------------------------------------------------------------------------
------------ Procedure: spXmlRejectSort
------------------------------------------------------------------------------------------------------------------------------------------
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[spXmlRejectSort]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
	DROP PROCEDURE [dbo].spXmlRejectSort
GO

CREATE PROCEDURE [dbo].spXmlRejectSort 
(
	 @fkIdent int
)
 AS
BEGIN

SET NOCOUNT ON

DECLARE 
	@rotasErrors int, @xmlErrors int, @testStandId int

	SELECT @testStandId = [Id] FROM [LocationId] WHERE [Text] = 'TestStand'

	SELECT @rotasErrors = COUNT(*) FROM ErrorData WHERE  [FkIdent] = @fkIdent AND [FkLocation] <>  @testStandId
	SELECT @xmlErrors = COUNT(*) FROM ErrorData WHERE  [FkIdent] = @fkIdent AND [FkLocation] =  @testStandId

	IF (@rotasErrors = 0 OR @xmlErrors = 0) RETURN;
	
DECLARE 
	@rotasPrio int, @xmlPrio int
	SET @rotasPrio = 256
	SET @xmlPrio = 255


	DECLARE @RejectWithPrio TABLE 
	(
		[FkIdent]			[int]	NOT NULL,  -- Nummer der Messung
		[Nr]				[int]	NOT NULL,  -- Nummer der Grenzwertberschreitung
		[Prio]				[int]	NOT NULL,  -- Prioritt
		[FkInternCode]			[int]	NOT NULL   -- Fehlernummer
	)

	-- insert XML Errors
	INSERT INTO @RejectWithPrio (		
		[FkIdent],		
		[Nr],
		[FkInternCode],			
		[Prio]			
	)		
	SELECT 
		[FkIdent],		
		[Nr],
		[FkInternCode],			
		@xmlPrio			
	FROM 
		ErrorData		
	WHERE  
		[FkIdent] = @fkIdent AND [FkLocation] =  @testStandId 


	-- insert Rotas Errors
	INSERT INTO @RejectWithPrio (		
		[FkIdent],		
		[Nr],			
		[FkInternCode],			
		[Prio]			
	)		
	SELECT 
		[FkIdent],		
		[Nr],			
		[FkInternCode],			
		@rotasPrio			
	FROM 
		ErrorData		
	WHERE  
		[FkIdent] = @fkIdent AND [FkLocation] <>  @testStandId 

		 
	-- set prios for error codes listed in ErrorPriorityInfo
	UPDATE t 
	SET	[Prio] = e.[Priority] 
	FROM @RejectWithPrio t
	INNER JOIN [ErrorPriorityInfo] AS e ON t.[FkIdent] = e.[FkIdent] AND t.[Nr] = e.[Nr]

	-- modify prios seleted error codes listed in ErrorPrioOverride
	UPDATE t 
	SET	[Prio] = e.[Prio] 
	FROM @RejectWithPrio t
	INNER JOIN [ErrorPrioOverride] AS e ON t.[FkInternCode] = e.[Id]


	-- Re-enumerate Errors on Priority
	
	DECLARE @RejectWithPrioSorted TABLE 
	(
		[FkIdent]			[int]	NOT NULL,  -- Nummer der Messung
		[NewNr]				[int]	NOT NULL,  -- Nummer der Grenzwertberschreitung neu
		[Nr]				[int]	NOT NULL  -- Nummer der Grenzwertberschreitung
	)
	-- resort, [Nr] will hold new position due to IDENTITY
	INSERT INTO @RejectWithPrioSorted (		
		[FkIdent],		
		[NewNr],			
		[Nr]	
	)
	SELECT 	a.[FkIdent], 
		NewNr= (
			SELECT COUNT (*) 
			FROM @RejectWithPrio b 
			WHERE a.[Prio]<b.[Prio] or (a.[Prio]=b.[Prio] AND a.[Nr]>b.[Nr])
			),
		a.[Nr] 
	FROM @RejectWithPrio a 
	ORDER BY [Prio] DESC,[Nr] ASC

	-- Update: Write back re-sorted entries

	UPDATE e 
	SET [Nr] = t.[NewNr] 
	FROM [ErrorData] e 
	INNER JOIN @RejectWithPrioSorted AS t ON e.[FkIdent] = t.[FkIdent] AND e.[Nr] = t.[Nr]

END -- procedure spXmlRejectSort
GO
------------------------------------------------------------------------------------------------------------------------------------------

PRINT 'Stored procedure added'
GO