------------------------------------------------------------------------------------------------------------------------------------------
------------ MainData: Check existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[MainData]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN

	CREATE TABLE [dbo].[MainData] (
		[PkIdent]	[int]		PRIMARY KEY CLUSTERED IDENTITY (1, 1) NOT NULL,
		[Serial]	[nvarchar] (200)	COLLATE Latin1_General_CI_AS NOT NULL,
		[Ordinal]	[int]		NOT NULL,
		[Type]		[int]		NOT NULL,
		[BaseType]	[int]		NULL,
		[TypeClass]	[int]		NOT NULL,
		[Property]	[int]		NOT NULL,
		[TestBench]	[int]		NOT NULL,
		[Factory]	[int]		NOT NULL,
		[Time]		[datetime]	NOT NULL,	
		[TestKind]	[int]		NOT NULL,
		[Result]	[int]		NOT NULL,
		[ThisLast]	[int]		NOT NULL,
		[TimeOfNextMeasurement]	[datetime]
	) ON [PRIMARY]
	
PRINT 'Table MainData added'
END
ELSE PRINT 'Table MainData already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ MainData: Check index existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT * FROM sysindexes WHERE name = N'IX_Time_MainData')
BEGIN
	CREATE NONCLUSTERED INDEX [IX_Time_MainData] ON [dbo].[MainData]([Time]) ON [PRIMARY]

PRINT 'Non-clustered index IX_Time_MainData added'
END
ELSE PRINT 'Non-clustered index IX_Time_MainData already exists'
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE name = N'IX_Type_MainData')
BEGIN
 	CREATE NONCLUSTERED INDEX [IX_Type_MainData] ON [dbo].[MainData]([Type]) ON [PRIMARY]

PRINT 'Non-clustered index IX_Type_MainData added' 
END
ELSE PRINT 'Non-clustered index IX_Type_MainData already exists'
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE name = N'IX_Serial_MainData')
BEGIN
 	CREATE NONCLUSTERED INDEX [IX_Serial_MainData] ON [dbo].[MainData]([Serial]) ON [PRIMARY]

PRINT 'Non-clustered index IX_Serial_MainData added'
END
ELSE PRINT 'Non-clustered index IX_Serial_MainData already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------


------------------------------------------------------------------------------------------------------------------------------------------
------------ MainData: Check constraint existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF ((SELECT dbo.fnDsDefConstraintName('MainData','TestKind')) IS NULL) 
BEGIN
	ALTER TABLE [dbo].[MainData] WITH NOCHECK ADD 
	CONSTRAINT [DF_MainData_TestType] DEFAULT (1) FOR [TestKind]

PRINT 'Table constraint DF_MainData_TestType added'
END
ELSE PRINT 'Table constraint DF_MainData_TestType already exists'
GO

IF ((SELECT dbo.fnDsDefConstraintName('MainData','ThisLast')) IS NULL) 
BEGIN
	ALTER TABLE [dbo].[MainData] WITH NOCHECK ADD 
	CONSTRAINT [DF_MainData_ThisLast] DEFAULT (1) FOR [ThisLast]

PRINT 'Table constraint DF_MainData_ThisLast added'
END
ELSE PRINT 'Table constraint DF_MainData_ThisLast already exists'
GO

IF ((SELECT dbo.fnDsDefConstraintName('MainData','TimeOfNextMeasurement')) IS NULL) 
BEGIN
	ALTER TABLE [dbo].[MainData] WITH NOCHECK ADD 
	CONSTRAINT [DF_MainData_TimeOfNextMeasurement] DEFAULT (NULL) FOR [TimeOfNextMeasurement]

PRINT 'Table constraint DF_MainData_TimeOfNextMeasurement added'
END
ELSE PRINT 'Table constraint DF_MainData_TimeOfNextMeasurement already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------



