

---------------------------------------------------------------------------------------------------------------------------
/* 1. Drop all Foreign Key constraints */
---------------------------------------------------------------------------------------------------------------------------

DECLARE @name VARCHAR(128)
DECLARE @constraint VARCHAR(254)
DECLARE @SQL VARCHAR(254)

SELECT @name = (SELECT TOP 1 TABLE_NAME FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS WHERE constraint_catalog=DB_NAME() AND CONSTRAINT_TYPE = 'FOREIGN KEY' ORDER BY TABLE_NAME)

WHILE @name is not null
BEGIN
    SELECT @constraint = (SELECT TOP 1 CONSTRAINT_NAME FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS WHERE constraint_catalog=DB_NAME() AND CONSTRAINT_TYPE = 'FOREIGN KEY' AND TABLE_NAME = @name ORDER BY CONSTRAINT_NAME)
    WHILE @constraint IS NOT NULL
    BEGIN
        SELECT @SQL = 'ALTER TABLE [dbo].[' + RTRIM(@name) +'] DROP CONSTRAINT [' + RTRIM(@constraint) +']'
		EXEC (@SQL)
		PRINT 'Dropped FK Constraint: ' + @constraint + ' on ' + @name
        
		SELECT @constraint = (SELECT TOP 1 CONSTRAINT_NAME FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS WHERE constraint_catalog=DB_NAME() AND CONSTRAINT_TYPE = 'FOREIGN KEY' AND CONSTRAINT_NAME <> @constraint AND TABLE_NAME = @name ORDER BY CONSTRAINT_NAME)
    END
SELECT @name = (SELECT TOP 1 TABLE_NAME FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS WHERE constraint_catalog=DB_NAME() AND CONSTRAINT_TYPE = 'FOREIGN KEY' ORDER BY TABLE_NAME)
END
GO

---------------------------------------------------------------------------------------------------------------------------
/* 2. Truncate all tables except DeleteParameters */
---------------------------------------------------------------------------------------------------------------------------

DECLARE @name VARCHAR(128)
DECLARE @SQL VARCHAR(254)

SELECT @name = (SELECT TOP 1 [name] FROM sysobjects WHERE [type] = 'U' AND category = 0 ORDER BY [name])

WHILE @name IS NOT NULL
BEGIN
    -- Exlcude the following tables
	IF (@name <> 'DeleteParameters' AND @name <> 'DatabaseVersion' AND @name <> 'DatabaseChangeLog')
	BEGIN
		SELECT @SQL = 'TRUNCATE TABLE [dbo].[' + RTRIM(@name) +']'
		EXEC (@SQL)
		PRINT 'Truncated Table: ' + @name
	END
	SELECT @name = (SELECT TOP 1 [name] FROM sysobjects WHERE [type] = 'U' AND category = 0 AND [name] > @name ORDER BY [name])
END
GO

---------------------------------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------------------------------
/* 3. Re-establish foreign key constraints */
---------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ AceErrorSelection: Check foreign key constraint existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_AceErrorSelection_ErrorGroup')
BEGIN
	ALTER TABLE [dbo].[AceErrorSelection] WITH CHECK
	ADD CONSTRAINT [FK_AceErrorSelection_ErrorGroup] FOREIGN KEY([GroupId])
	REFERENCES [dbo].[AceErrorGroup]([Id])
	ON DELETE CASCADE
	ON UPDATE CASCADE

	PRINT 'Added FK constraint FK_AceErrorSelection_ErrorGroup'
END
ELSE PRINT 'FK constraint FK_AceErrorSelection_ErrorGroup already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ AdditionalInfo: Check foreign key constraint existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_AdditionalInfo_MainData')
BEGIN
	ALTER TABLE [dbo].[AdditionalInfo] 
	ADD CONSTRAINT [FK_AdditionalInfo_MainData] FOREIGN KEY([FkIdent])
	REFERENCES [dbo].[MainData]([PkIdent])

PRINT 'Added FK constraint FK_AceErrorSelection_ErrorGroup'
END
ELSE PRINT 'FK constraint FK_AceErrorSelection_ErrorGroup already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_AdditionalInfo_AdditionalKeys')
BEGIN
	ALTER TABLE [dbo].[AdditionalInfo] 
	ADD CONSTRAINT [FK_AdditionalInfo_AdditionalKeys] FOREIGN KEY([KeyId])
	REFERENCES [dbo].[AdditionalKeys]([Id])

PRINT 'Added FK constraint FK_AdditionalInfo_AdditionalKeys'
END
ELSE PRINT 'FK constraint FK_AdditionalInfo_AdditionalKeys already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_AdditionalInfo_AdditionalValues')
BEGIN
	ALTER TABLE [dbo].[AdditionalInfo] 
	ADD CONSTRAINT [FK_AdditionalInfo_AdditionalValues]	FOREIGN KEY([ValueId])
	REFERENCES [dbo].[AdditionalValues]([Id])

PRINT 'Added FK constraint FK_AdditionalInfo_AdditionalValues'
END
ELSE PRINT 'FK constraint FK_AdditionalInfo_AdditionalValues already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ ArchiveData: Check foreign key constraint existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ArchiveData_MainData')
BEGIN
	ALTER TABLE [dbo].[ArchiveData] 
	ADD CONSTRAINT [FK_ArchiveData_MainData] FOREIGN KEY([FkIdent])
	REFERENCES [dbo].[MainData]([PkIdent])

PRINT 'Added FK constraint FK_ArchiveData_MainData'
END
ELSE PRINT 'FK constraint FK_ArchiveData_MainData already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ArchiveData_StoreModules')
BEGIN
	ALTER TABLE [dbo].[ArchiveData] 
	ADD CONSTRAINT [FK_ArchiveData_StoreModules] FOREIGN KEY([FkStoreModule])
	REFERENCES [dbo].[StoreModules]([Id])

PRINT 'Added FK constraint FK_ArchiveData_StoreModules'
END
ELSE PRINT 'FK constraint FK_ArchiveData_StoreModules already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ArchiveData_Quinters')
BEGIN
	ALTER TABLE [dbo].[ArchiveData]
	ADD CONSTRAINT [FK_ArchiveData_Quinters] FOREIGN KEY([FkQuinter])
	REFERENCES [dbo].[Quinters]([Id])

PRINT 'Added FK constraint FK_ArchiveData_Quinters'
END
ELSE PRINT 'FK constraint FK_ArchiveData_Quinters already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ArchiveData_Files')
BEGIN
	ALTER TABLE [dbo].[ArchiveData] 
	ADD CONSTRAINT [FK_ArchiveData_Files] FOREIGN KEY([FkFilename])
	REFERENCES [dbo].[Files]([Id])

PRINT 'Added FK constraint FK_ArchiveData_Files'
END
ELSE PRINT 'FK constraint FK_ArchiveData_Files already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ BaseTypeID: Check foreign key constraint existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE TABLE_NAME = 'BaseTypeId' AND COLUMN_NAME = 'TypeClass')
BEGIN
	ALTER TABLE [dbo].[BaseTypeId] 
	ADD CONSTRAINT [FK_BaseTypeID_TypeClassID] FOREIGN KEY([TypeClass])
	REFERENCES [dbo].[TypeClassId]([ID])

PRINT 'Added FK constraint FK_BaseTypeID_TypeClassID'
END
ELSE PRINT 'FK constraint FK_BaseTypeID_TypeClassID already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ BenchID: Check foreign key constraint existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_BenchID_BenchGroupID')
BEGIN
	ALTER TABLE [dbo].[BenchId] 
	ADD CONSTRAINT [FK_BenchID_BenchGroupID] FOREIGN KEY([BenchGroup])
	REFERENCES [dbo].[BenchGroupId]([Id])

PRINT 'Added FK constraint FK_BenchID_BenchGroupID'
END
ELSE PRINT 'FK constraint FK_BenchID_BenchGroupID already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ BlobData: Check foreign key constraint existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_BlobData_BlobType')
BEGIN
	ALTER TABLE [dbo].[BlobData] 
	ADD CONSTRAINT [FK_BlobData_BlobType] FOREIGN KEY([BlobType])
	REFERENCES [dbo].[BlobType]([Id])

PRINT 'Added FK constraint FK_BlobData_BlobType'
END
ELSE PRINT 'FK constraint FK_BlobData_BlobType already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_BlobData_MainData')
BEGIN
	ALTER TABLE [dbo].[BlobData] 
	ADD CONSTRAINT [FK_BlobData_MainData] FOREIGN KEY([FKident])
	REFERENCES [dbo].[MainData]([PkIdent])
	ON DELETE CASCADE
	ON UPDATE CASCADE

PRINT 'Added FK constraint FK_BlobData_MainData'
END
ELSE PRINT 'FK constraint FK_BlobData_MainData already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ BlobInfo: Check foreign key constraint existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_BlobInfo_BackupMedium')
BEGIN
	ALTER TABLE [dbo].[BlobInfo] 
	ADD CONSTRAINT [FK_BlobInfo_BackupMedium] FOREIGN KEY([BackupMedium])
	REFERENCES [dbo].[BackupMedium]([Id])

PRINT 'Added FK constraint FK_BlobInfo_BackupMedium'
END
ELSE PRINT 'FK constraint FK_BlobInfo_BackupMedium already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_BlobInfo_BlobStatus')
BEGIN
	ALTER TABLE [dbo].[BlobInfo] 
	ADD CONSTRAINT [FK_BlobInfo_BlobStatus] FOREIGN KEY([BackupStatus])
	REFERENCES [dbo].[BlobStatus]([Id])

PRINT 'Added FK constraint FK_BlobInfo_BlobStatus'
END
ELSE PRINT 'FK constraint FK_BlobInfo_BlobStatus already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_BlobInfo_BlobType')
BEGIN
	ALTER TABLE [dbo].[BlobInfo] 
	ADD CONSTRAINT [FK_BlobInfo_BlobType] FOREIGN KEY([BlobType])
	REFERENCES [dbo].[BlobType]([Id])

PRINT 'Added FK constraint FK_BlobInfo_BlobType'
END
ELSE PRINT 'FK constraint FK_BlobInfo_BlobType already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_BlobInfo_MainData')
BEGIN
	ALTER TABLE [dbo].[BlobInfo] 
	ADD CONSTRAINT [FK_BlobInfo_MainData] FOREIGN KEY([FkIdent])
	REFERENCES [dbo].[MainData]([PkIdent])
	ON DELETE CASCADE
	ON UPDATE CASCADE

PRINT 'Added FK constraint FK_BlobInfo_MainData'
END
ELSE PRINT 'FK constraint FK_BlobInfo_MainData already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ ErrorCategoryMapping: Check foreign key constraint existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ErrorCategoryMapping_ErrorCodes')
BEGIN
	ALTER TABLE [dbo].[ErrorCategoryMapping] 
	ADD CONSTRAINT [FK_ErrorCategoryMapping_ErrorCodes] FOREIGN KEY([FkInternCode])
	REFERENCES [dbo].[ErrorCodes]([PkInternCode]) 
	ON UPDATE CASCADE 
	ON DELETE CASCADE

PRINT 'Added FK constraint FK_ErrorCategoryMapping_ErrorCodes'
END
ELSE PRINT 'FK constraint FK_ErrorCategoryMapping_ErrorCodes already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ErrorCategoryMapping_ErrorCategoryCodes')
BEGIN
	ALTER TABLE [dbo].[ErrorCategoryMapping] 
	ADD CONSTRAINT [FK_ErrorCategoryMapping_ErrorCategoryCodes] FOREIGN KEY([FkCategoryCode])
	REFERENCES [dbo].[ErrorCategoryCodes]([Id]) 
	ON UPDATE CASCADE 
	ON DELETE CASCADE

PRINT 'Added FK constraint FK_ErrorCategoryMapping_ErrorCategoryCodes'
END
ELSE PRINT 'FK constraint FK_ErrorCategoryMapping_ErrorCategoryCodes already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ ErrorCategoryMapping: Check foreign key constraint existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ErrorData_MainData')
BEGIN
	ALTER TABLE [dbo].[ErrorData] 
	ADD CONSTRAINT [FK_ErrorData_MainData] FOREIGN KEY([FkIdent])
	REFERENCES [dbo].[MainData]([PkIdent])

PRINT 'Added FK constraint FK_ErrorData_MainData'
END
ELSE PRINT 'FK constraint FK_ErrorData_MainData already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ErrorData_ModeId')
BEGIN
	ALTER TABLE [dbo].[ErrorData] 
	ADD CONSTRAINT [FK_ErrorData_ModeId] FOREIGN KEY([FkMode])
	REFERENCES [dbo].[ModeId]([Id])

PRINT 'Added FK constraint FK_ErrorData_ModeId'
END
ELSE PRINT 'FK constraint FK_ErrorData_ModeId already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ErrorData_InstanceId')
BEGIN
	ALTER TABLE [dbo].[ErrorData] 
	ADD CONSTRAINT [FK_ErrorData_InstanceId] FOREIGN KEY([FkChannel])
	REFERENCES [dbo].[InstanceId]([Id])

PRINT 'Added FK constraint FK_ErrorData_InstanceId'
END
ELSE PRINT 'FK constraint FK_ErrorData_InstanceId already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ErrorData_LocationId')
BEGIN
	ALTER TABLE [dbo].[ErrorData] 
	ADD CONSTRAINT [FK_ErrorData_LocationId] FOREIGN KEY([FkLocation])
	REFERENCES [dbo].[LocationId]([Id])

PRINT 'Added FK constraint FK_ErrorData_LocationId'
END
ELSE PRINT 'FK constraint FK_ErrorData_LocationId already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ErrorData_InstrumentId')
BEGIN
	ALTER TABLE [dbo].[ErrorData] 
	ADD CONSTRAINT [FK_ErrorData_InstrumentId] FOREIGN KEY([FkInstrument])
	REFERENCES [dbo].[InstrumentId]([Id])

PRINT 'Added FK constraint FK_ErrorData_InstrumentId'
END
ELSE PRINT 'FK constraint FK_ErrorData_InstrumentId already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ErrorData_InstrParamId')
BEGIN
	ALTER TABLE [dbo].[ErrorData] 
	ADD CONSTRAINT [FK_ErrorData_InstrParamId] FOREIGN KEY([FkParameter])
	REFERENCES [dbo].[InstrParamId]([Id])

PRINT 'Added FK constraint FK_ErrorData_InstrParamId'
END
ELSE PRINT 'FK constraint FK_ErrorData_InstrParamId already exists'
GO
		
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ErrorData_ErrorCodes')
BEGIN
	ALTER TABLE [dbo].[ErrorData] 
	ADD CONSTRAINT [FK_ErrorData_ErrorCodes] FOREIGN KEY([FkInternCode])
	REFERENCES [dbo].[ErrorCodes]([PkInternCode])

PRINT 'Added FK constraint FK_ErrorData_ErrorCodes'
END
ELSE PRINT 'FK constraint FK_ErrorData_ErrorCodes already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ ErrorPriorityInfo: Check foreign key constraint existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ErrorPriorityInfo_ErrorData')
BEGIN
	ALTER TABLE [dbo].[ErrorPriorityInfo] 
	ADD CONSTRAINT [FK_ErrorPriorityInfo_ErrorData] FOREIGN KEY([FkIdent], [Nr])
	REFERENCES [dbo].[ErrorData]([FkIdent], [Nr]) 
	ON UPDATE CASCADE 
	ON DELETE CASCADE

PRINT 'Added FK constraint FK_ErrorData_MainData'
END
ELSE PRINT 'FK constraint FK_ErrorData_MainData already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ DatabaseChangeLog: Check foreign key constraint existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_DatabaseChangeLog_DatabaseVersion')
BEGIN
	ALTER TABLE [dbo].[DatabaseChangeLog] 
	ADD CONSTRAINT [FK_DatabaseChangeLog_DatabaseVersion] FOREIGN KEY([DbVersionId])
	REFERENCES [dbo].[DatabaseVersion]([Id]) 
	ON UPDATE CASCADE 
	ON DELETE CASCADE

PRINT 'Added FK constraint FK_DatabaseChangeLog_DatabaseVersion'
END
ELSE PRINT 'FK constraint FK_DatabaseChangeLog_DatabaseVersion already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ MainData: Check foreign key constraint existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_MainData_BaseTypeID')
BEGIN
	ALTER TABLE [dbo].[MainData] 
	ADD CONSTRAINT [FK_MainData_BaseTypeID] FOREIGN KEY([BaseType])
	REFERENCES [dbo].[BaseTypeId]([Id])

PRINT 'Added FK constraint FK_MainData_BaseTypeID'
END
ELSE PRINT 'FK constraint FK_MainData_BaseTypeID already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_MainData_BenchID')
BEGIN
	ALTER TABLE [dbo].[MainData] 
	ADD CONSTRAINT [FK_MainData_BenchID] FOREIGN KEY([TestBench])
	REFERENCES [dbo].[BenchId]([Id])

PRINT 'Added FK constraint FK_MainData_BenchID'
END
ELSE PRINT 'FK constraint FK_MainData_BenchID already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_MainData_FactoryID')
BEGIN
	ALTER TABLE [dbo].[MainData] 
	ADD CONSTRAINT [FK_MainData_FactoryID] FOREIGN KEY([Factory])
	REFERENCES [dbo].[FactoryId]([ID])

PRINT 'Added FK constraint FK_MainData_FactoryID'
END
ELSE PRINT 'FK constraint FK_MainData_FactoryID already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_MainData_ResultID')
BEGIN
	ALTER TABLE [dbo].[MainData] 
	ADD CONSTRAINT [FK_MainData_ResultID] FOREIGN KEY([Result])
	REFERENCES [dbo].[ResultId]([Id])

PRINT 'Added FK constraint FK_MainData_ResultID'
END
ELSE PRINT 'FK constraint FK_MainData_ResultID already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_MainData_TestTypeID')
BEGIN
	ALTER TABLE [dbo].[MainData] 
	ADD CONSTRAINT [FK_MainData_TestTypeID] FOREIGN KEY([TestKind])
	REFERENCES [dbo].[TestKindId]([Id])

PRINT 'Added FK constraint FK_MainData_TestTypeID'
END
ELSE PRINT 'FK constraint FK_MainData_TestTypeID already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_MainData_TypeClassID')
BEGIN
	ALTER TABLE [dbo].[MainData] 
	ADD CONSTRAINT [FK_MainData_TypeClassID] FOREIGN KEY([TypeClass])
	REFERENCES [dbo].[TypeClassId]([ID])

PRINT 'Added FK constraint FK_MainData_TypeClassID'
END
ELSE PRINT 'FK constraint FK_MainData_TypeClassID already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_MainData_TypeID')
BEGIN
	ALTER TABLE [dbo].[MainData] 
	ADD CONSTRAINT [FK_MainData_TypeID] FOREIGN KEY([Type])	
	REFERENCES [dbo].[TypeId]([Id])
	ON DELETE CASCADE
	ON UPDATE CASCADE

PRINT 'Added FK constraint FK_MainData_TypeID'
END
ELSE PRINT 'FK constraint FK_MainData_TypeID already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ ModesPerMeasurement: Check foreign key constraint existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ModesPerMeasurement_ModeId')
BEGIN
	ALTER TABLE [dbo].[ModesPerMeasurement] 
	ADD CONSTRAINT [FK_ModesPerMeasurement_ModeId] FOREIGN KEY([FkMode])
	REFERENCES [dbo].[ModeId]([Id])

PRINT 'Added FK constraint FK_ModesPerMeasurement_ModeId'
END
ELSE PRINT 'FK constraint FK_ModesPerMeasurement_ModeId already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ModesPerMeasurement_MainData')
BEGIN
	ALTER TABLE [dbo].[ModesPerMeasurement] 
	ADD CONSTRAINT [FK_ModesPerMeasurement_MainData] FOREIGN KEY([FkIdent])
	REFERENCES [dbo].[MainData]([PkIdent])

PRINT 'Added FK constraint FK_ModesPerMeasurement_MainData'
END
ELSE PRINT 'FK constraint FK_ModesPerMeasurement_MainData already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ ProtocolCrossReference: Check foreign key constraint existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ProtocolCrossReference_InstanceId')
BEGIN
	ALTER TABLE [dbo].[ProtocolCrossReference] 
	ADD CONSTRAINT [FK_ProtocolCrossReference_InstanceId] FOREIGN KEY([FkInstance])
	REFERENCES [dbo].[InstanceId]([Id])

PRINT 'Added FK constraint FK_ProtocolCrossReference_InstanceId'
END
ELSE PRINT 'FK constraint FK_ProtocolCrossReference_InstanceId already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ProtocolCrossReference_InstrParamId')
BEGIN
	ALTER TABLE [dbo].[ProtocolCrossReference] 
	ADD CONSTRAINT [FK_ProtocolCrossReference_InstrParamId] FOREIGN KEY([FkParam])
	REFERENCES [dbo].[InstrParamId]([Id])

PRINT 'Added FK constraint FK_ProtocolCrossReference_InstrParamId'
END
ELSE PRINT 'FK constraint FK_ProtocolCrossReference_InstrParamId already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ProtocolCrossReference_InstrumentId')
BEGIN
	ALTER TABLE [dbo].[ProtocolCrossReference] 
	ADD CONSTRAINT [FK_ProtocolCrossReference_InstrumentId] FOREIGN KEY([FkInstrument])
	REFERENCES [dbo].[InstrumentId]([Id])

PRINT 'Added FK constraint FK_ProtocolCrossReference_InstrumentId'
END
ELSE PRINT 'FK constraint FK_ProtocolCrossReference_InstrumentId already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ProtocolCrossReference_LocationId')
BEGIN
	ALTER TABLE [dbo].[ProtocolCrossReference] 
	ADD CONSTRAINT [FK_ProtocolCrossReference_LocationId] FOREIGN KEY([FkLocation])
	REFERENCES [dbo].[LocationId]([Id])

PRINT 'Added FK constraint FK_ProtocolCrossReference_LocationId'
END
ELSE PRINT 'FK constraint FK_ProtocolCrossReference_LocationId already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ProtocolCrossReference_ModeId')
BEGIN
	ALTER TABLE [dbo].[ProtocolCrossReference] 
	ADD CONSTRAINT [FK_ProtocolCrossReference_ModeId] FOREIGN KEY([FkMode])
	REFERENCES [dbo].[ModeId]([Id])

PRINT 'Added FK constraint FK_ProtocolCrossReference_ModeId'
END
ELSE PRINT 'FK constraint FK_ProtocolCrossReference_ModeId already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ ProtocolData: Check foreign key constraint existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ProtocolData_MainData')
BEGIN
	ALTER TABLE [dbo].[ProtocolData] 
	ADD CONSTRAINT [FK_ProtocolData_MainData] FOREIGN KEY([FkIdent])
	REFERENCES [dbo].[MainData]([PkIdent])

PRINT 'Added FK constraint FK_ProtocolData_MainData'
END
ELSE PRINT 'FK constraint FK_ProtocolData_MainData already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ProtocolData_ModeId')
BEGIN
	ALTER TABLE [dbo].[ProtocolData] 
	ADD CONSTRAINT [FK_ProtocolData_ModeId] FOREIGN KEY([FkMode])
	REFERENCES [dbo].[ModeId]([Id])

PRINT 'Added FK constraint FK_ProtocolData_ModeId'
END
ELSE PRINT 'FK constraint FK_ProtocolData_ModeId already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ProtocolData_InstanceId')
BEGIN
	ALTER TABLE [dbo].[ProtocolData] 
	ADD CONSTRAINT [FK_ProtocolData_InstanceId] FOREIGN KEY([FkChannel])
	REFERENCES [dbo].[InstanceId]([Id])

PRINT 'Added FK constraint FK_ProtocolData_InstanceId'
END
ELSE PRINT 'FK constraint FK_ProtocolData_InstanceId already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ProtocolData_LocationId')
BEGIN
	ALTER TABLE [dbo].[ProtocolData] 
	ADD CONSTRAINT [FK_ProtocolData_LocationId] FOREIGN KEY([FkLocation])
	REFERENCES [dbo].[LocationId]([Id])

PRINT 'Added FK constraint FK_ProtocolData_LocationId'
END
ELSE PRINT 'FK constraint FK_ProtocolData_LocationId already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ProtocolData_InstrumentId')
BEGIN
	ALTER TABLE [dbo].[ProtocolData]
	ADD CONSTRAINT [FK_ProtocolData_InstrumentId] FOREIGN KEY([FkInstrument])
	REFERENCES [dbo].[InstrumentId]([Id])

PRINT 'Added FK constraint FK_ProtocolData_InstrumentId'
END
ELSE PRINT 'FK constraint FK_ProtocolData_InstrumentId already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ProtocolData_InstrParamId')
BEGIN
	ALTER TABLE [dbo].[ProtocolData] 
	ADD CONSTRAINT [FK_ProtocolData_InstrParamId] FOREIGN KEY([FkParameter])
	REFERENCES [dbo].[InstrParamId]([Id])

PRINT 'Added FK constraint FK_ProtocolData_InstrParamId'
END
ELSE PRINT 'FK constraint FK_ProtocolData_InstrParamId already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ProtocolData_UnitId')
BEGIN
	ALTER TABLE [dbo].[ProtocolData] 
	ADD CONSTRAINT [FK_ProtocolData_UnitId] FOREIGN KEY([FkUnit])
	REFERENCES [dbo].[UnitId]([Id])

PRINT 'Added FK constraint FK_ProtocolData_UnitId'
END
ELSE PRINT 'FK constraint FK_ProtocolData_UnitId already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ProtocolData_ExtraPInfoId')
BEGIN
	ALTER TABLE [dbo].[ProtocolData] 
	ADD CONSTRAINT [FK_ProtocolData_ExtraPInfoId] FOREIGN KEY([FkExtra])
	REFERENCES [dbo].[ExtraPInfoId]([Id])

PRINT 'Added FK constraint FK_ProtocolData_ExtraPInfoId'
END
ELSE PRINT 'FK constraint FK_ProtocolData_ExtraPInfoId already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ Referenzgetriebe: Check foreign key constraint existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_Referenzgetriebe_TypeID')
BEGIN
	ALTER TABLE [dbo].[Referenzgetriebe] 
	ADD CONSTRAINT [FK_Referenzgetriebe_TypeID] FOREIGN KEY([Type])
	REFERENCES [dbo].[TypeID]([Id])

PRINT 'Added FK constraint FK_Referenzgetriebe_TypeID'
END
ELSE PRINT 'FK constraint FK_Referenzgetriebe_TypeID already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------


------------------------------------------------------------------------------------------------------------------------------------------
------------ ShiftMinutes: Check foreign key constraint existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ShiftMinutes_ShiftNames')
BEGIN
	ALTER TABLE [dbo].[ShiftMinutes] WITH CHECK 
	ADD CONSTRAINT [FK_ShiftMinutes_ShiftNames] FOREIGN KEY([ShiftId])
	REFERENCES [dbo].[ShiftNames] ([Id])
	ON UPDATE CASCADE
	ON DELETE CASCADE
PRINT 'Added FK constraint FK_ShiftMinutes_ShiftNames'
END
ELSE PRINT 'FK constraint FK_ShiftMinutes_ShiftNames already exists'
GO

ALTER TABLE [dbo].[ShiftMinutes] CHECK CONSTRAINT [FK_ShiftMinutes_ShiftNames]
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ TypeID: Check foreign key constraint existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS  (SELECT * FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE  WHERE TABLE_NAME = 'TypeId' AND COLUMN_NAME = 'BaseType')
BEGIN
	ALTER TABLE [dbo].[TypeId] 
	ADD CONSTRAINT [FK_TypeID_BaseTypeID] FOREIGN KEY([BaseType])
	REFERENCES [dbo].[BaseTypeId]([Id])
	
PRINT 'Added FK constraint FK_TypeID_BaseTypeID'
END
ELSE PRINT 'FK constraint FK_TypeID_BaseTypeID already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ WeeklyResultDetails: Check foreign key constraint existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_WeeklyResultDetails_BenchID')
BEGIN
	ALTER TABLE [dbo].WeeklyResultDetails 
	ADD CONSTRAINT [FK_WeeklyResultDetails_BenchID] FOREIGN KEY([TestBench])
	REFERENCES [dbo].[BenchId]([Id])

PRINT 'Added FK constraint FK_WeeklyResultDetails_BenchID'
END
ELSE PRINT 'FK constraint FK_WeeklyResultDetails_BenchID already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_WeeklyResultDetails_TypeID')
BEGIN
	ALTER TABLE [dbo].WeeklyResultDetails 
	ADD CONSTRAINT [FK_WeeklyResultDetails_TypeID] FOREIGN KEY([Type])
	REFERENCES [dbo].[TypeId]([Id])

PRINT 'Added FK constraint FK_WeeklyResultDetails_TypeID'
END
ELSE PRINT 'FK constraint FK_WeeklyResultDetails_TypeID already exists'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_WeeklyResultDetails_CountID')
BEGIN
	ALTER TABLE [dbo].WeeklyResultDetails 
	ADD CONSTRAINT [FK_WeeklyResultDetails_CountID] FOREIGN KEY([CountType])
	REFERENCES [dbo].[ResultCountType]([Id])

PRINT 'Added FK constraint FK_WeeklyResultDetails_CountID'
END
ELSE PRINT 'FK constraint FK_WeeklyResultDetails_CountID already exists'
GO

------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
/* 4. Initialize table values */
------------------------------------------------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------------------------------
------------ AceErrorGroup: Default entries
----------------------------------------------------------------------------------------------------------------------------
DECLARE 
	@nrGroups int

SELECT @nrGroups = count ( * ) from AceErrorGroup

IF (@nrGroups = 0) 
BEGIN
	PRINT 'No entries in table AceErrorGroup. Setting Default Values'
	INSERT INTO [AceErrorGroup] VALUES (100, 'Acoustic Rejects')
	INSERT INTO [AceErrorGroup] VALUES (200, 'Function Rjects')
	INSERT INTO [AceErrorGroup] VALUES (900, 'System Rejects')
	INSERT INTO [AceErrorGroup] VALUES (1100, 'Extra quiet transmissions')
END
ELSE PRINT 'Table AceErrorGroup already contains entries'
GO
----------------------------------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------------------------------
------------ BackupMedium: Default entries
----------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT * FROM [BackupMedium] WHERE Id = 0) 
BEGIN
	INSERT INTO [BackupMedium] ([Id], [ArchiveName]) VALUES (0, '(medium missing)')
END
ELSE PRINT 'Table BackupMedium already contains entries'
GO
----------------------------------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------------------------------
------------ BenchGroupID: Default entries
----------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT * FROM [BenchGroupId] WHERE Id = 0) 
BEGIN 
	PRINT 'No entries in table BenchGroupID. Setting Default Values'
	INSERT INTO [BenchGroupId] VALUES ('*')
	IF NOT EXISTS (SELECT * FROM [BenchGroupId] WHERE Id = 1) INSERT INTO [BenchGroupId] VALUES ('Standard')
END
ELSE PRINT 'Table BenchGroupID already contains entries'
GO
----------------------------------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------------------------------
------------ BlobStatus: Default entries
----------------------------------------------------------------------------------------------------------------------------
DECLARE 
	@Count int

SELECT @Count = count ( * ) from BlobStatus

IF (@Count = 0) 
BEGIN
	PRINT 'No entries in table BlobStatus. Setting Default Values'
	INSERT INTO [BlobStatus] VALUES (0, 'only in DB')
	INSERT INTO [BlobStatus] VALUES (1, 'archived and in DB')
	INSERT INTO [BlobStatus] VALUES (2, 'only archived')
END
ELSE PRINT 'Table BlobStatus already contains entries'
GO
----------------------------------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------------------------------
------------ BlobType: Default entries
----------------------------------------------------------------------------------------------------------------------------
DECLARE 
	@Count int

SELECT @Count = count ( * ) from BlobType

IF (@Count = 0) 
BEGIN
	PRINT 'No entries in table BlobType. Setting Default Values'
	INSERT INTO [BlobType] VALUES (0, '?')
	INSERT INTO [BlobType] VALUES (1, 'Fehlerberichte')
	INSERT INTO [BlobType] VALUES (2, 'Protokolldaten')
	INSERT INTO [BlobType] VALUES (3, 'End-Spektren')
	INSERT INTO [BlobType] VALUES (4, 'Ordnungskurven')
	INSERT INTO [BlobType] VALUES (5, 'Berichtstexte')
	INSERT INTO [BlobType] VALUES (6, 'Schaltkraft')
	INSERT INTO [BlobType] VALUES (7, 'Spektogramme')
	INSERT INTO [BlobType] VALUES (77, 'Prfstandsdaten (XML)')
END
ELSE PRINT 'Table BlobType already contains entries'
GO
----------------------------------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------------------------------
------------ FactoryId: Default entries
----------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT * FROM [FactoryId] WHERE id = 0) 
BEGIN
	PRINT 'No entries in table FactoryId. Setting Default Values'
	INSERT INTO [FactoryId] VALUES (0, '*')
	IF NOT EXISTS (SELECT * FROM [FactoryId] WHERE id = 1) INSERT INTO [FactoryId] VALUES (1, 'Standard')
END
ELSE PRINT 'Table FactoryId already contains entries'
GO
----------------------------------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------------------------------
------------ Quinters: Default entries
----------------------------------------------------------------------------------------------------------------------------
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[Quinters]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
	IF ((SELECT COUNT(*) FROM [dbo].Quinters) = 0)
	BEGIN
		PRINT 'No entries in table Quinters. Setting Default Values'
		INSERT INTO Quinters VALUES ('','','','','')	-- Id = 0, All values = ''.
	END
	ELSE PRINT 'Table Quinters already contains entries'
END
GO
----------------------------------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------------------------------
------------ ResultCountType: Default entries
----------------------------------------------------------------------------------------------------------------------------
DECLARE 
	@nrCount int

SELECT @nrCount = count ( * ) from [ResultCountType]

IF (@nrCount = 0) 
BEGIN
	PRINT 'No entries in table ResultCountType. Setting Default Values'
	-- String constants Identical to SelectionStore.cs
	INSERT INTO [ResultCountType] VALUES (1, 'All')
	INSERT INTO [ResultCountType] VALUES (2, 'Last')
	INSERT INTO [ResultCountType] VALUES (3, 'First')
	INSERT INTO [ResultCountType] VALUES (4, 'CurrentReject')
	INSERT INTO [ResultCountType] VALUES (5, 'Direct')
END
ELSE PRINT 'Table ResultCountType already contains entries'

GO
----------------------------------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------------------------------
------------ ResultId: Default entries
----------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT * FROM [ResultId] WHERE id = 0) 
BEGIN
	PRINT 'No entries in table ResultId. Setting Default Values'
	INSERT INTO [ResultId] VALUES (0, '*')
	IF NOT EXISTS (SELECT * FROM [ResultId] WHERE id = 1) INSERT INTO [ResultId] VALUES (1, 'Good')
	IF NOT EXISTS (SELECT * FROM [ResultId] WHERE id = 2) INSERT INTO [ResultId] VALUES (2, 'Bad')
	IF NOT EXISTS (SELECT * FROM [ResultId] WHERE id = 3) INSERT INTO [ResultId] VALUES (3, 'Not Evaluated')
	IF NOT EXISTS (SELECT * FROM [ResultId] WHERE id = 4) INSERT INTO [ResultId] VALUES (4, 'System Error')
END
ELSE PRINT 'Table ResultId already contains entries'
GO
----------------------------------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------------------------------
------------ ShiftNames: Default entries
----------------------------------------------------------------------------------------------------------------------------
DECLARE 
	@nrShifts int

SELECT @nrShifts = count ( * ) from ShiftNames

IF (@nrShifts = 0) 
BEGIN
	PRINT 'No entries in table ShiftNames. Setting Default Values'
	INSERT INTO [ShiftNames] VALUES (1, 'Morning')
	INSERT INTO [ShiftNames] VALUES (2, 'Day')
	INSERT INTO [ShiftNames] VALUES (3, 'Night')
END
ELSE PRINT 'Table ShiftNames already contains entries'
GO
----------------------------------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------------------------------
------------ ShiftMinutes: Default entries
----------------------------------------------------------------------------------------------------------------------------
DECLARE 
	@nrShifts int

SELECT @nrShifts = count ( * ) from [ShiftMinutes]

IF (@nrShifts = 0) 
BEGIN
	PRINT 'No entries in table ShiftMinutes. Setting Default Values'
	INSERT INTO [ShiftMinutes] VALUES (1, 3, 0, 390, 1, '1,2,3,4,5,6,7')
	INSERT INTO [ShiftMinutes] VALUES (2, 1, 390, 870, 0, '1,2,3,4,5,6,7')
	INSERT INTO [ShiftMinutes] VALUES (3, 2, 870, 1350, 0, '1,2,3,4,5,6,7')
	INSERT INTO [ShiftMinutes] VALUES (4, 3, 1350, 1440, 0, '1,2,3,4,5,6,7')
END
ELSE PRINT 'Table ShiftMinutes already contains entries'
GO
----------------------------------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------------------------------
------------ StoreModules: Default entry
----------------------------------------------------------------------------------------------------------------------------
-- Einfgen des speziellen Werts Null: Wurzel des Storage-Modul-Baums
IF NOT EXISTS (SELECT * FROM [StoreModules] WHERE id = 0) 
BEGIN
	PRINT 'No entries in table StoreModules. Setting Default Values'
	INSERT INTO StoreModules VALUES ('')	-- Id = 0, Name = ''.
END
ELSE PRINT 'Table StoreModules already contains entries'
GO
----------------------------------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------------------------------
------------ TypeClassId: Default entries !!!!!!! Order is crucial: TypeClassId before BaseTypeId before TypeId !!!!!!!!!!!!
----------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT Id FROM [TypeClassId] WHERE Id = 0)
BEGIN
	PRINT 'No entries in table TypeClassId. Setting Default Values'
	INSERT INTO [TypeClassId] VALUES (0, '*')
	IF NOT EXISTS (SELECT Id FROM [TypeClassId] WHERE Id = 1) INSERT INTO [TypeClassId] VALUES (1, 'Not Assigned')
END
ELSE PRINT 'Table TypeClassId already contains entries'
GO
----------------------------------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------------------------------
------------ BaseTypeId: Default entries !!!!!!! Order is crucial: TypeClassId before BaseTypeId before TypeId !!!!!!!!!!!!
----------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT Id FROM [BaseTypeId] WHERE Id = 0)
BEGIN
	PRINT 'No entries in table BaseTypeId. Setting Default Values'
	INSERT INTO [BaseTypeId] (text, typeClass)  VALUES ('*', 0)
END
ELSE PRINT 'Table BaseTypeId already contains entries'
GO
----------------------------------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------------------------------
------------ TypeId default entries !!!!!!!!!!! Order is crucial: TypeClassId before BaseTypeId before TypeId !!!!!!!!!!!!!!
----------------------------------------------------------------------------------------------------------------------------
IF  NOT EXISTS (SELECT Id FROM [TypeId] WHERE Id = 0)
BEGIN
	PRINT 'No entries in table TypeId. Setting Default Values'
	INSERT INTO [TypeId] VALUES (0, '*', '*', 0, 0)
END
ELSE PRINT 'Table TypeId already contains entries'
GO
----------------------------------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------------------------------
------------ BenchId default entries
----------------------------------------------------------------------------------------------------------------------------
IF  NOT EXISTS (SELECT Id FROM [BenchId] WHERE Id = 0)
BEGIN
	PRINT 'No entries in table BenchId. Setting Default Values'
	INSERT INTO [BenchId] VALUES (0, '*', '*', 0)
END
ELSE PRINT 'Table BenchId already contains entries'
GO
----------------------------------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------------------------------
------------ Referenzgetriebe default entries
----------------------------------------------------------------------------------------------------------------------------
IF  NOT EXISTS (SELECT Type FROM [Referenzgetriebe] WHERE Type = 0)
BEGIN
	PRINT 'No entries in table Referenzgetriebe. Setting Default Values'
	INSERT INTO [Referenzgetriebe] VALUES (0, 0, '*')
END
ELSE PRINT 'Table Referenzgetriebe already contains entries'
GO
----------------------------------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------------------------------
------------ InstanceId default entries
----------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT Id FROM [InstanceId] WHERE Id = 0)
BEGIN
	PRINT 'No entries in table InstanceId. Setting Default Values'
	INSERT INTO [InstanceId] VALUES ('?')
END
ELSE PRINT 'Table InstanceId already contains entries'
GO
----------------------------------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------------------------------
------------ InstrParamId default entries
----------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT Id FROM [InstrParamId] WHERE Id = 0)
BEGIN
	PRINT 'No entries in table InstrParamId. Setting Default Values'
	INSERT INTO [InstrParamId] VALUES ('?')
END
ELSE PRINT 'Table InstrParamId already contains entries'
GO
----------------------------------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------------------------------
------------ InstrumentId default entries
----------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT Id FROM [InstrumentId] WHERE Id = 0)
BEGIN
	PRINT 'No entries in table InstrumentId. Setting Default Values'
	INSERT INTO [InstrumentId] VALUES ('?')
END
ELSE PRINT 'Table InstrumentId already contains entries'
GO
----------------------------------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------------------------------
------------ LocationId default entries
----------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT Id FROM [LocationId] WHERE Id = 0)
BEGIN
	PRINT 'No entries in table LocationId. Setting Default Values'
	INSERT INTO [LocationId] VALUES ('?')
END
ELSE PRINT 'Table LocationId already contains entries'
GO
----------------------------------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------------------------------
------------ ModeId default entries
----------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT Id FROM [ModeId] WHERE Id = 0)
BEGIN
	PRINT 'No entries in table ModeId. Setting Default Values'
	INSERT INTO [ModeId] VALUES ('?')
END
ELSE PRINT 'Table ModeId already contains entries'
GO
----------------------------------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------------------------------
------------ ExtraPInfoId default entries
----------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT Id FROM [ExtraPInfoId] WHERE Id = 0)
BEGIN
	PRINT 'No entries in table ExtraPInfoId. Setting Default Values'
	INSERT INTO [ExtraPInfoId] VALUES (0, '?')
	IF NOT EXISTS (SELECT Id FROM [ExtraPInfoId] WHERE Id = 1) INSERT INTO [ExtraPInfoId] VALUES (1, '')
END
ELSE PRINT 'Table ExtraPInfoId already contains entries'
GO
----------------------------------------------------------------------------------------------------------------------------
