


------------------------------------------------------------------------------------------------------------------------------------------
------------ AceErrorSelection: Check foreign key constraint existence, else build
------------------------------------------------------------------------------------------------------------------------------------------
-- Drop foreign key constraint
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[FK_AceErrorSelection_ErrorGroup]') AND OBJECTPROPERTY(id, N'IsForeignKey') = 1)
BEGIN
	ALTER TABLE [dbo].[AceErrorSelection] DROP CONSTRAINT FK_AceErrorSelection_ErrorGroup
	PRINT 'Dropping constraint FK_AceErrorSelection_ErrorGroup from table AceErrorSelection' 
END

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_AceErrorSelection_ErrorGroup')
BEGIN
	ALTER TABLE [dbo].[AceErrorSelection] WITH CHECK
	ADD CONSTRAINT [FK_AceErrorSelection_ErrorGroup] FOREIGN KEY([GroupId])
	REFERENCES [dbo].[AceErrorGroup]([Id])
	ON DELETE CASCADE
	ON UPDATE CASCADE
	
PRINT 'Added FK constraint FK_AceErrorSelection_ErrorGroup'
END
ELSE PRINT 'FK constraint FK_AceErrorSelection_ErrorGroup already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------


------------------------------------------------------------------------------------------------------------------------------------------
------------ AdditionalValues: Change length of name column 
------------------------------------------------------------------------------------------------------------------------------------------
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[AdditionalValues]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
	IF (SELECT COL_LENGTH ('AdditionalValues','Name')) < 512
	BEGIN
		ALTER TABLE [AdditionalValues]
		ALTER COLUMN [Name] [nvarchar](256) NOT NULL

		PRINT 'Field size of column Name in table AdditionalValues was increased to size 256'
	END
END
GO

------------------------------------------------------------------------------------------------------------------------------------------
------------ BaseTypeId: Update the Id column of BaseTypeId 
------------------------------------------------------------------------------------------------------------------------------------------
IF( COLUMNPROPERTY(object_id('BaseTypeId'), 'Id', 'IsIdentity') != 1 )

BEGIN
	-- Change the Id column to identity
	BEGIN TRANSACTION
	SET QUOTED_IDENTIFIER ON
	SET TRANSACTION ISOLATION LEVEL SERIALIZABLE
	SET ARITHABORT ON
	SET NUMERIC_ROUNDABORT OFF
	SET CONCAT_NULL_YIELDS_NULL ON
	SET ANSI_NULLS ON
	SET ANSI_PADDING ON
	SET ANSI_WARNINGS ON
	COMMIT
	
	BEGIN TRANSACTION
	
		IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[Tmp_BaseTypeId]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
			CREATE TABLE dbo.Tmp_BaseTypeId	(
			[Id]		int NOT NULL IDENTITY (0, 1),
			[Text]		[nvarchar] (80)	COLLATE Latin1_General_CI_AS NULL,
			[TypeClass]	[int]		NULL 	) ON [PRIMARY]

		IF EXISTS (SELECT dbo.fnDsDefConstraintName('BaseTypeId','TypeClass'))
			ALTER TABLE BaseTypeId DROP CONSTRAINT [DF_BaseTypeID_TypeClass]
		
		IF NOT EXISTS (SELECT dbo.fnDsDefConstraintName('Tmp_BaseTypeId','TypeClass'))
			ALTER TABLE dbo.Tmp_BaseTypeID ADD CONSTRAINT DF_BaseTypeID_TypeClass DEFAULT (0) FOR TypeClass
		
		-- Switch off identity, the copy all fields to temp table, then re-establish identity 
		SET IDENTITY_INSERT dbo.Tmp_BaseTypeId ON

		IF EXISTS(SELECT * FROM dbo.BaseTypeID)
			EXEC('INSERT INTO dbo.Tmp_BaseTypeID (Id, Text, TypeClass)
				SELECT Id, Text, TypeClass FROM dbo.BaseTypeID TABLOCKX')

		SET IDENTITY_INSERT dbo.Tmp_BaseTypeId OFF
		
		EXECUTE SpDsDropAllConstraintsOfTable @table = 'BaseTypeId', @database = '{DbName}'
		
		DROP TABLE dbo.BaseTypeId

		EXECUTE sp_rename N'dbo.Tmp_BaseTypeId', N'BaseTypeId', 'OBJECT'

		ALTER TABLE dbo.BaseTypeId ADD CONSTRAINT
			PK_BaseTypeId PRIMARY KEY CLUSTERED (Id) ON [PRIMARY]			
	
		IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[TypeClassId]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
			ALTER TABLE dbo.BaseTypeID WITH NOCHECK ADD CONSTRAINT
			FK_BaseTypeID_TypeClassID FOREIGN KEY (TypeClass) 
			REFERENCES dbo.TypeClassID (ID)		
	
		IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[TypeId]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
			ALTER TABLE dbo.TypeID WITH NOCHECK ADD CONSTRAINT
			FK_TypeID_BaseTypeID FOREIGN KEY (BaseType) 
			REFERENCES dbo.BaseTypeID (Id)
			
		IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[MainData]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
			ALTER TABLE dbo.MainData WITH NOCHECK ADD CONSTRAINT
			FK_MainData_BaseTypeID FOREIGN KEY (BaseType) 
			REFERENCES dbo.BaseTypeID (Id)
					
	COMMIT
	PRINT 'Table BaseTypeId column Id is now an identity column'	

END
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ BenchGroupId: Update table and remove DLD column
------------------------------------------------------------------------------------------------------------------------------------------
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[BenchGroupId]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
	-- Remove the DLD (DirektLauferdefinition) FK constraint and Column 
	
	DECLARE @BgId_ConstraintName varchar (100)
	-- Remove the DLD (DirektLauferdefinition) FK constraint and Column 
	SELECT @BgId_ConstraintName = CONSTRAINT_NAME FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE TABLE_NAME = 'BenchGroupId' AND COLUMN_NAME = 'DLD'
	IF ( @BgId_ConstraintName IS NOT NULL)
		EXEC ('ALTER TABLE [dbo].[BenchGroupId] DROP CONSTRAINT ' +  @BgId_ConstraintName)

	IF EXISTS (SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = 'BenchGroupId' AND COLUMN_NAME = 'DLD')
	BEGIN
		ALTER TABLE [dbo].[BenchGroupId] DROP COLUMN [DLD]
		PRINT 'Column DLD was removed from table BenchGroupId '
	END

END
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ DeleteLog: Add column if it does not exist 
------------------------------------------------------------------------------------------------------------------------------------------
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[DeleteLog]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
	IF ((SELECT name FROM sys.columns WHERE object_id = (SELECT object_id FROM sys.objects WHERE name = 'DeleteLog') AND name = 'Bench') IS NULL)
	BEGIN
		ALTER TABLE [DeleteLog]
		ADD Bench int
		PRINT 'Adding column Bench to table DeleteLog'  
	END
END
GO
------------------------------------------------------------------------------------------------------------------------------------------
------------ DirektLaufDefinition: Drop table
------------------------------------------------------------------------------------------------------------------------------------------
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[DirektLaufDefinition]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
	DROP TABLE [DirektLaufDefinition]
	PRINT 'Table DirektLaufDefinition was removed from the database'
END
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ BlobInfo: Drop table
------------------------------------------------------------------------------------------------------------------------------------------
--IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[BlobInfo]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
--BEGIN
--	DROP TABLE [BlobInfo]
--	PRINT 'Table BlobInfo was removed from the database'
--END
--GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ BlobStatus: Drop table
------------------------------------------------------------------------------------------------------------------------------------------
--IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[BlobStatus]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
--BEGIN
--	DROP TABLE [BlobStatus]
--	PRINT '
--	Table BlobStatus was removed from the database'
--END
--GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ BackupMedium: Drop table
------------------------------------------------------------------------------------------------------------------------------------------
--IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[BackupMedium]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
--BEGIN
--	DROP TABLE [dbo].[BackupMedium]
--	PRINT 'Table BackupMedium was removed from the database'
--END
--GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ ErrorCodes: Drop columns ExternCode and ErrorGroup
------------------------------------------------------------------------------------------------------------------------------------------
IF EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME = 'ExternCode' AND TABLE_NAME = 'ErrorCodes')
BEGIN
	ALTER TABLE [dbo].[ErrorCodes] DROP COLUMN [ExternCode]
	PRINT 'Column ExternCode was removed from table ErrorCodes'
END
GO

IF EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME = 'ErrorGroup' AND TABLE_NAME = 'ErrorCodes')
BEGIN
	ALTER TABLE [dbo].[ErrorCodes] DROP COLUMN [ErrorGroup]
	PRINT 'Column ErrorGroup was removed from table ErrorCodes'
END
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ Files: Change length of name column 
------------------------------------------------------------------------------------------------------------------------------------------
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[Files]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
	IF (SELECT COL_LENGTH ('Files','Name')) < 512
	BEGIN
		ALTER TABLE [Files]
		ALTER COLUMN [Name] [nvarchar](256) NOT NULL

		PRINT 'Field size of column Name in table Files was increased to size 256'
	END
END
GO
------------------------------------------------------------------------------------------------------------------------------------------
------------ InstanceId: Update the Id column 
------------------------------------------------------------------------------------------------------------------------------------------
IF( COLUMNPROPERTY(object_id('InstanceId'), 'Id', 'IsIdentity') != 1 )

BEGIN
	-- Change the Id column to identity
	BEGIN TRANSACTION
	SET QUOTED_IDENTIFIER ON
	SET TRANSACTION ISOLATION LEVEL SERIALIZABLE
	SET ARITHABORT ON
	SET NUMERIC_ROUNDABORT OFF
	SET CONCAT_NULL_YIELDS_NULL ON
	SET ANSI_NULLS ON
	SET ANSI_PADDING ON
	SET ANSI_WARNINGS ON
	COMMIT
	
	BEGIN TRANSACTION
	
		IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[Tmp_InstanceId]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
			CREATE TABLE dbo.Tmp_InstanceId	(
			Id int NOT NULL IDENTITY (1, 1),
			Text nvarchar(80) NULL
			)  ON [PRIMARY]
			
		-- Switch off identity, the copy all fields to temp table, then reestablish identity 
		SET IDENTITY_INSERT dbo.Tmp_InstanceId ON

		IF EXISTS(SELECT * FROM dbo.InstanceId)
			EXEC('INSERT INTO dbo.Tmp_InstanceId (Id, Text)
				SELECT Id, Text FROM dbo.InstanceId TABLOCKX')

		SET IDENTITY_INSERT dbo.Tmp_InstanceId OFF
		
		EXECUTE SpDsDropAllConstraintsOfTable @table = 'InstanceId', @database = '{DbName}'
		
		DROP TABLE dbo.InstanceId

		EXECUTE sp_rename N'dbo.Tmp_InstanceId', N'InstanceId', 'OBJECT'

		ALTER TABLE dbo.InstanceId ADD CONSTRAINT
			PK_InstanceId PRIMARY KEY CLUSTERED ( Id ) ON [PRIMARY]			
	
		IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[ErrorData]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
			ALTER TABLE dbo.ErrorData WITH NOCHECK ADD CONSTRAINT
			FK_ErrorData_InstanceId FOREIGN KEY	(FkChannel) REFERENCES dbo.InstanceId (Id)
	
		IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[ProtocolData]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
			ALTER TABLE [dbo].[ProtocolData] WITH NOCHECK ADD CONSTRAINT 
			[FK_ProtocolData_InstanceId] FOREIGN KEY([FkChannel])	
			REFERENCES [dbo].[InstanceId]([Id])

		IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[ProtocolCrossReference]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
			ALTER TABLE dbo.ProtocolCrossReference WITH NOCHECK ADD CONSTRAINT
			FK_ProtocolCrossReference_InstanceId FOREIGN KEY (FkInstance) 
			REFERENCES dbo.InstanceId (Id)

	COMMIT
	PRINT 'Table InstanceId column Id is now an identity column'	

END
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ InstrParamId: Update the Id column 
------------------------------------------------------------------------------------------------------------------------------------------
IF( COLUMNPROPERTY(object_id('InstrParamId'), 'Id', 'IsIdentity') != 1 )

BEGIN
	-- change the Id column to identity
	BEGIN TRANSACTION
	SET QUOTED_IDENTIFIER ON
	SET TRANSACTION ISOLATION LEVEL SERIALIZABLE
	SET ARITHABORT ON
	SET NUMERIC_ROUNDABORT OFF
	SET CONCAT_NULL_YIELDS_NULL ON
	SET ANSI_NULLS ON
	SET ANSI_PADDING ON
	SET ANSI_WARNINGS ON
	COMMIT
	
	BEGIN TRANSACTION
	
		IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[Tmp_InstrParamId]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
			CREATE TABLE dbo.Tmp_InstrParamId	(
			Id int NOT NULL IDENTITY (1, 1),
			Text nvarchar(80) NULL
			)  ON [PRIMARY]
			
		-- switch off identity, the copy all fields to temp table, then reestablish identity 
		SET IDENTITY_INSERT dbo.Tmp_InstrParamId ON

		IF EXISTS(SELECT * FROM dbo.InstrParamId)
			EXEC('INSERT INTO dbo.Tmp_InstrParamId (Id, Text)
				SELECT Id, Text FROM dbo.InstrParamId TABLOCKX')

		SET IDENTITY_INSERT dbo.Tmp_InstrParamId OFF
		
		EXECUTE SpDsDropAllConstraintsOfTable @table = 'InstrParamId', @database = '{DbName}'
		
		DROP TABLE dbo.InstrParamId

		EXECUTE sp_rename N'dbo.Tmp_InstrParamId', N'InstrParamId', 'OBJECT'

		ALTER TABLE dbo.InstrParamId ADD CONSTRAINT
			PK_InstrParamId PRIMARY KEY CLUSTERED ( Id ) ON [PRIMARY]			
	
		IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[ErrorData]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
			ALTER TABLE [dbo].[ErrorData] ADD CONSTRAINT 
			[FK_ErrorData_InstrParamId] FOREIGN KEY([FkParameter])	
			REFERENCES [dbo].[InstrParamId]([Id])
	
		IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[ProtocolData]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
			ALTER TABLE [dbo].[ProtocolData] WITH NOCHECK ADD CONSTRAINT 
			[FK_ProtocolData_InstrParamId] FOREIGN KEY([FkParameter])	
			REFERENCES [dbo].[InstrParamId]([Id])


		IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[ProtocolCrossReference]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
			ALTER TABLE dbo.ProtocolCrossReference WITH NOCHECK ADD CONSTRAINT
			FK_ProtocolCrossReference_InstrParamId FOREIGN KEY (FkParam) 
			REFERENCES dbo.InstrParamId (Id)

	COMMIT
	PRINT 'Table InstrParamId  column Id is now an identity column'	

END
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ InstrumentId: Update the Id column 
------------------------------------------------------------------------------------------------------------------------------------------
IF( COLUMNPROPERTY(object_id('InstrumentId'), 'Id', 'IsIdentity') != 1 )

BEGIN
	-- change the Id column to identity
	BEGIN TRANSACTION
	SET QUOTED_IDENTIFIER ON
	SET TRANSACTION ISOLATION LEVEL SERIALIZABLE
	SET ARITHABORT ON
	SET NUMERIC_ROUNDABORT OFF
	SET CONCAT_NULL_YIELDS_NULL ON
	SET ANSI_NULLS ON
	SET ANSI_PADDING ON
	SET ANSI_WARNINGS ON
	COMMIT
	
	BEGIN TRANSACTION
	
		IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[Tmp_InstrumentId]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
			CREATE TABLE dbo.Tmp_InstrumentId	(
			Id int NOT NULL IDENTITY (1, 1),
			Text nvarchar(80) NULL
			)  ON [PRIMARY]
			
		-- switch off identity, the copy all fields to temp table, then reestablish identity 
		SET IDENTITY_INSERT dbo.Tmp_InstrumentId ON

		IF EXISTS(SELECT * FROM dbo.InstrumentId)
			EXEC('INSERT INTO dbo.Tmp_InstrumentId (Id, Text)
				SELECT Id, Text FROM dbo.InstrumentId TABLOCKX')

		SET IDENTITY_INSERT dbo.Tmp_InstrumentId OFF
		
		EXECUTE SpDsDropAllConstraintsOfTable @table = 'InstrumentId', @database = '{DbName}'
		
		DROP TABLE dbo.InstrumentId

		EXECUTE sp_rename N'dbo.Tmp_InstrumentId', N'InstrumentId', 'OBJECT'

		ALTER TABLE dbo.InstrumentId ADD CONSTRAINT
			PK_InstrumentId PRIMARY KEY CLUSTERED ( Id ) ON [PRIMARY]			
	
		IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[ErrorData]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
			ALTER TABLE [dbo].[ErrorData] ADD CONSTRAINT 
			[FK_ErrorData_InstrumentId] FOREIGN KEY([FkInstrument])	
			REFERENCES [dbo].[InstrumentId]([Id])
	
		IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[ProtocolData]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
			ALTER TABLE [dbo].[ProtocolData] WITH NOCHECK ADD CONSTRAINT 
			[FK_ProtocolData_InstrumentId] FOREIGN KEY([FkInstrument])	
			REFERENCES [dbo].[InstrumentId]([Id])


		IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[ProtocolCrossReference]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
			ALTER TABLE dbo.ProtocolCrossReference WITH NOCHECK ADD CONSTRAINT
			FK_ProtocolCrossReference_InstrumentId FOREIGN KEY (FkInstrument) 
			REFERENCES dbo.InstrumentId (Id)

	COMMIT
	PRINT 'Table InstrumentId  column Id is now an identity column'	

END
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ InstrumentId: Add the column EnumId
------------------------------------------------------------------------------------------------------------------------------------------

-- IF ((SELECT name FROM sys.columns WHERE object_id = (SELECT object_id FROM sys.objects WHERE name = 'InstrumentId') AND name = 'EnumId') IS NULL)
-- BEGIN
	-- ALTER TABLE [InstrumentId]
	-- ADD EnumId int
-- END
-- GO

------------------------------------------------------------------------------------------------------------------------------------------
------------ LocationId: Update the Id column 
------------------------------------------------------------------------------------------------------------------------------------------
IF( COLUMNPROPERTY(object_id('LocationId'), 'Id', 'IsIdentity') != 1 )

BEGIN
	-- change the Id column to identity
	BEGIN TRANSACTION
	SET QUOTED_IDENTIFIER ON
	SET TRANSACTION ISOLATION LEVEL SERIALIZABLE
	SET ARITHABORT ON
	SET NUMERIC_ROUNDABORT OFF
	SET CONCAT_NULL_YIELDS_NULL ON
	SET ANSI_NULLS ON
	SET ANSI_PADDING ON
	SET ANSI_WARNINGS ON
	COMMIT
	
	BEGIN TRANSACTION
	
		IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[Tmp_LocationId]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
			CREATE TABLE dbo.Tmp_LocationId	(
			Id int NOT NULL IDENTITY (1, 1),
			Text nvarchar(80) NULL
			)  ON [PRIMARY]
			
		-- switch off identity, the copy all fields to temp table, then reestablish identity 
		SET IDENTITY_INSERT dbo.Tmp_LocationId ON

		IF EXISTS(SELECT * FROM dbo.LocationId)
			EXEC('INSERT INTO dbo.Tmp_LocationId (Id, Text)
				SELECT Id, Text FROM dbo.LocationId TABLOCKX')

		SET IDENTITY_INSERT dbo.Tmp_LocationId OFF
		
		EXECUTE SpDsDropAllConstraintsOfTable @table = 'LocationId', @database = '{DbName}'
		
		DROP TABLE dbo.LocationId

		EXECUTE sp_rename N'dbo.Tmp_LocationId', N'LocationId', 'OBJECT'

		ALTER TABLE dbo.LocationId ADD CONSTRAINT
			PK_LocationId PRIMARY KEY CLUSTERED ( Id ) ON [PRIMARY]			
	
		IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[ErrorData]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
			ALTER TABLE [dbo].[ErrorData] ADD CONSTRAINT 
			[FK_ErrorData_LocationId] FOREIGN KEY([FkLocation])	
			REFERENCES [dbo].[LocationId]([Id])
	
		IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[ProtocolData]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
			ALTER TABLE [dbo].[ProtocolData] WITH NOCHECK ADD CONSTRAINT 
			[FK_ProtocolData_LocationId] FOREIGN KEY([FkLocation])	
			REFERENCES [dbo].[LocationId]([Id])


		IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[ProtocolCrossReference]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
			ALTER TABLE dbo.ProtocolCrossReference WITH NOCHECK ADD CONSTRAINT
			FK_ProtocolCrossReference_LocationId FOREIGN KEY (FkLocation) 
			REFERENCES dbo.LocationId (Id)

	COMMIT
	PRINT 'Table LocationId column Id is now an identity column'	

END
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ MainData: Update indexes, constraints and columns
------------------------------------------------------------------------------------------------------------------------------------------
-- Drop non-clustered index NCI_MainData
IF EXISTS (SELECT * FROM sysindexes WHERE name = N'NCI_MainData') DROP INDEX [NCI_MainData] ON [dbo].[MainData]

-- Drop foreign key constraint for PropertyId
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[FK_MainData_PropertyID]') AND OBJECTPROPERTY(id, N'IsForeignKey') = 1)
BEGIN
	ALTER TABLE [dbo].[MainData] DROP CONSTRAINT FK_MainData_PropertyID
	PRINT 'Dropping constraint FK_MainData_PropertyID from table MainData' 
END
GO

------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ ModeId: Update the Id column 
------------------------------------------------------------------------------------------------------------------------------------------
IF( COLUMNPROPERTY(object_id('ModeId'), 'Id', 'IsIdentity') != 1 )

BEGIN
	-- change the Id column to identity
	BEGIN TRANSACTION
	SET QUOTED_IDENTIFIER ON
	SET TRANSACTION ISOLATION LEVEL SERIALIZABLE
	SET ARITHABORT ON
	SET NUMERIC_ROUNDABORT OFF
	SET CONCAT_NULL_YIELDS_NULL ON
	SET ANSI_NULLS ON
	SET ANSI_PADDING ON
	SET ANSI_WARNINGS ON
	COMMIT
	
	BEGIN TRANSACTION
	
		IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[Tmp_ModeId]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
			CREATE TABLE dbo.Tmp_ModeId	(
			Id int NOT NULL IDENTITY (1, 1),
			Text nvarchar(80) NULL
			)  ON [PRIMARY]
			
		-- switch off identity, the copy all fields to temp table, then reestablish identity 
		SET IDENTITY_INSERT dbo.Tmp_ModeId ON

		IF EXISTS(SELECT * FROM dbo.ModeId)
			EXEC('INSERT INTO dbo.Tmp_ModeId (Id, Text)
				SELECT Id, Text FROM dbo.ModeId TABLOCKX')

		SET IDENTITY_INSERT dbo.Tmp_ModeId OFF
		
		EXECUTE SpDsDropAllConstraintsOfTable @table = 'ModeId', @database = '{DbName}'
		
		DROP TABLE dbo.ModeId

		EXECUTE sp_rename N'dbo.Tmp_ModeId', N'ModeId', 'OBJECT'

		ALTER TABLE dbo.ModeId ADD CONSTRAINT
			PK_ModeId PRIMARY KEY CLUSTERED ( Id ) ON [PRIMARY]			
	
		IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[ErrorData]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
			ALTER TABLE dbo.ErrorData WITH NOCHECK ADD CONSTRAINT
			[FK_ErrorData_ModeId] FOREIGN KEY([FkMode])	
			REFERENCES [dbo].[ModeId]([Id])


		IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[ProtocolData]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
			ALTER TABLE [dbo].[ProtocolData] WITH NOCHECK ADD CONSTRAINT 
			[FK_ProtocolData_ModeId] FOREIGN KEY([FkMode])	
			REFERENCES [dbo].[ModeId]([Id])


		IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[ProtocolCrossReference]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
			ALTER TABLE dbo.ProtocolCrossReference WITH NOCHECK ADD CONSTRAINT
				FK_ProtocolCrossReference_ModeId FOREIGN KEY (FkMode) 
				REFERENCES dbo.ModeId (Id)

	COMMIT
	PRINT 'Table ModeId column Id is now an identity column'	

END
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ TimeLimitsOfTypes: Add columns if they do not exist 
------------------------------------------------------------------------------------------------------------------------------------------
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[TimeLimitsOfTypes]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
	IF ((SELECT name FROM sys.columns WHERE object_id = (SELECT object_id FROM sys.objects WHERE name = 'TimeLimitsOfTypes') AND name = 'BenchId') IS NULL)
	BEGIN
		ALTER TABLE [TimeLimitsOfTypes]
		ADD BenchId int NULL 
		ALTER TABLE [TimeLimitsOfTypes]
		ADD BenchName nvarchar(50) COLLATE Latin1_General_CI_AS NULL
		
		PRINT 'Adding columns BenchId and BenchName to table TimeLimitsOfTypes' 
	END
END
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ ShiftMinutes: Drop foreign key constraint and add with CASCADE keyword
------------------------------------------------------------------------------------------------------------------------------------------
IF EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ShiftMinutes_ShiftNames')
BEGIN
	ALTER TABLE [dbo].[ShiftMinutes] DROP CONSTRAINT [FK_ShiftMinutes_ShiftNames]
PRINT 'FK constraint FK_ShiftMinutes_ShiftNames dropped successfully'
END
ELSE PRINT 'FK constraint FK_ShiftMinutes_ShiftNames does not exist'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ShiftMinutes_ShiftNames')
BEGIN
	ALTER TABLE [dbo].[ShiftMinutes] WITH CHECK 
	ADD CONSTRAINT [FK_ShiftMinutes_ShiftNames] FOREIGN KEY([ShiftId])
	REFERENCES [dbo].[ShiftNames] ([Id])
	ON UPDATE CASCADE
	ON DELETE CASCADE
PRINT 'Added FK constraint FK_ShiftMinutes_ShiftNames'
END
ELSE PRINT 'FK constraint FK_ShiftMinutes_ShiftNames already exists'
GO

ALTER TABLE [dbo].[ShiftMinutes] CHECK CONSTRAINT [FK_ShiftMinutes_ShiftNames]
GO
------------------------------------------------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------------------------------
------------ ResultCountType update with new entry 'Direct'
----------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT Id FROM [ResultCountType] WHERE Id = 5)
BEGIN
	INSERT INTO [ResultCountType] VALUES (5, 'Direct')
	PRINT 'Entry Direct with Id 5 entered into the table ResultCountType'
END
ELSE PRINT 'Table ResultCountType already contains the entry Direct with Id 5'
GO
----------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ MainData: Change length of Serial number column 
------------------------------------------------------------------------------------------------------------------------------------------
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[MainData]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
	IF (SELECT COL_LENGTH ('MainData','Serial')) <= 512
	BEGIN
		ALTER TABLE [MainData]
		ALTER COLUMN [Serial] [nvarchar](200) NOT NULL

		PRINT 'Field size of column Serial in table MainData was increased to size 200'
	END
END
GO

------------------------------------------------------------------------------------------------------------------------------------------
------------ Quinters: Change length of quinter columns 
------------------------------------------------------------------------------------------------------------------------------------------
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[Quinters]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
	IF (SELECT COL_LENGTH ('Quinters','Mode')) < 160
	BEGIN
		ALTER TABLE [Quinters]
		ALTER COLUMN [Mode] [nvarchar](80) NOT NULL

		PRINT 'Field size of column Mode in table Quinters was increased to size 80'
	END
	IF (SELECT COL_LENGTH ('Quinters','Channel')) < 160
	BEGIN
		ALTER TABLE [Quinters]
		ALTER COLUMN [Channel] [nvarchar](80) NOT NULL

		PRINT 'Field size of column Channel in table Quinters was increased to size 80'
	END
		IF (SELECT COL_LENGTH ('Quinters','Location')) < 160
	BEGIN
		ALTER TABLE [Quinters]
		ALTER COLUMN [Location] [nvarchar](80) NOT NULL

		PRINT 'Field size of column Location in table Quinters was increased to size 80'
	END
		IF (SELECT COL_LENGTH ('Quinters','Instrument')) < 160
	BEGIN
		ALTER TABLE [Quinters]
		ALTER COLUMN [Instrument] [nvarchar](80) NOT NULL

		PRINT 'Field size of column Instrument in table Quinters was increased to size 80'
	END
		IF (SELECT COL_LENGTH ('Quinters','Parameter')) < 160
	BEGIN
		ALTER TABLE [Quinters]
		ALTER COLUMN [Parameter] [nvarchar](80) NOT NULL

		PRINT 'Field size of column Parameter in table Quinters was increased to size 80'
	END
END
GO

------------------------------------------------------------------------------------------------------------------------------------------
------------ AceErrorSelection: Drop foreign key constraint and re-add with ON DELETE/UPDATE CASCADE
------------------------------------------------------------------------------------------------------------------------------------------
-- Drop foreign key constraint
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[FK_AceErrorSelection_ErrorGroup]') AND OBJECTPROPERTY(id, N'IsForeignKey') = 1)
BEGIN
	ALTER TABLE [dbo].[AceErrorSelection] DROP CONSTRAINT FK_AceErrorSelection_ErrorGroup
	PRINT 'Dropping constraint FK_AceErrorSelection_ErrorGroup from table AceErrorSelection' 
END

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_AceErrorSelection_ErrorGroup')
BEGIN
	ALTER TABLE [dbo].[AceErrorSelection] WITH CHECK
	ADD CONSTRAINT [FK_AceErrorSelection_ErrorGroup] FOREIGN KEY([GroupId])
	REFERENCES [dbo].[AceErrorGroup]([Id])
	ON DELETE CASCADE
	ON UPDATE CASCADE
	
PRINT 'Added FK constraint FK_AceErrorSelection_ErrorGroup'
END
ELSE PRINT 'FK constraint FK_AceErrorSelection_ErrorGroup already exists'
GO
------------------------------------------------------------------------------------------------------------------------------------------


------------------------------------------------------------------------------------------------------------------------------------------
------------ Function: FirstError
------------------------------------------------------------------------------------------------------------------------------------------

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[FirstError]') AND [type] = 'FN')
	DROP FUNCTION [dbo].FirstError
GO

CREATE FUNCTION [dbo].[FirstError] (@PkIdent INT)
RETURNS int
AS
BEGIN
DECLARE @ErrorCode INT
-- in case of good transmissions with error entries
--IF (SELECT Result From MainData WHERE MainData.PkIdent = @PkIdent) <> 2
--RETURN -1
SET @ErrorCode = (
SELECT MAX(FkInternCode)
FROM ErrorData
WHERE ErrorData.FkIdent = @PkIdent AND ErrorData.Nr = 0
)
-- Return the result of the function
RETURN ISNULL(@ErrorCode,-1)
END

GO
------------------------------------------------------------------------------------------------------------------------------------------


------------------------------------------------------------------------------------------------------------------------------------------
------------ Function: FirstAndNErrors
------------------------------------------------------------------------------------------------------------------------------------------

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[FirstAndNErrors]') AND [type] = 'TF')
	DROP FUNCTION [dbo].[FirstAndNErrors]
GO


CREATE FUNCTION [dbo].[FirstAndNErrors] (@PkIdent INT, @TopN INT)
RETURNS  @idTable TABLE 
(
    -- columns returned by the function
    Id INT NOT NULL
)
AS
BEGIN
DECLARE @TempTable table (id INT)

-- in case of good transmissions with error entries
--IF (SELECT Result From MainData WHERE MainData.PkIdent = @PkIdent) <> 2
--BEGIN
--	INSERT INTO @TempTable 
--	SELECT - 1

--	INSERT INTO  @idTable
--	SELECT Id FROM @TempTable 
--	RETURN
--END

-- If @TopN equals 0 then fill the @TempTable with all errors associated with the respective @PkIdent
IF (@TopN) = 0
BEGIN
	INSERT INTO @TempTable 
	SELECT FkInternCode
	FROM ErrorData
	WHERE ErrorData.FkIdent = @PkIdent 
END
ELSE
BEGIN
	INSERT INTO @TempTable 
	SELECT FkInternCode
	FROM ErrorData
	WHERE ErrorData.FkIdent = @PkIdent AND ErrorData.Nr IN (SELECT TOP (@TopN) Nr FROM (SELECT DISTINCT Nr FROM ErrorData) t)
END


-- In case the above SELECT query returns NULL
IF (SELECT COUNT(*) FROM @TempTable) = 0
INSERT INTO @idTable 
SELECT -1

INSERT INTO  @idTable
SELECT Id FROM @TempTable 
RETURN

END
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ Function: FirstErrorRejectHistory
------------------------------------------------------------------------------------------------------------------------------------------

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[FirstErrorRejectHistory]') AND [type] = 'FN')
	DROP FUNCTION [dbo].FirstErrorRejectHistory
GO

CREATE FUNCTION [dbo].[FirstErrorRejectHistory] (@PkIdent INT)
RETURNS int
AS
BEGIN
DECLARE @ErrorCode INT
-- in case of good transmissions with error entries
IF (SELECT Result From MainData WHERE MainData.PkIdent = @PkIdent) = 1
RETURN -1
SET @ErrorCode = (
SELECT MAX(FkInternCode)
FROM ErrorData
WHERE ErrorData.FkIdent = @PkIdent AND ErrorData.Nr = 0
)
-- Return the result of the function
RETURN ISNULL(@ErrorCode,-1)
END

GO
------------------------------------------------------------------------------------------------------------------------------------------


------------------------------------------------------------------------------------------------------------------------------------------
------------ Function: FirstAndNErrorsRejectHistory
------------------------------------------------------------------------------------------------------------------------------------------

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[FirstAndNErrorsRejectHistory]') AND [type] = 'TF')
	DROP FUNCTION [dbo].[FirstAndNErrorsRejectHistory]
GO


CREATE FUNCTION [dbo].[FirstAndNErrorsRejectHistory] (@PkIdent INT, @TopN INT)
RETURNS  @idTable TABLE 
(
    -- columns returned by the function
    Id INT NOT NULL
)
AS
BEGIN
DECLARE @TempTable table (id INT)

-- in case of good transmissions with error entries
IF (SELECT Result From MainData WHERE MainData.PkIdent = @PkIdent) <> 2
BEGIN
	INSERT INTO @TempTable 
	SELECT - 1

	INSERT INTO  @idTable
	SELECT Id FROM @TempTable 
	RETURN
END

-- If @TopN equals 0 then fill the @TempTable with all errors associated with the respective @PkIdent
IF (@TopN) = 0
BEGIN
	INSERT INTO @TempTable 
	SELECT FkInternCode
	FROM ErrorData
	WHERE ErrorData.FkIdent = @PkIdent 
END
ELSE
BEGIN
	INSERT INTO @TempTable 
	SELECT FkInternCode
	FROM ErrorData
	WHERE ErrorData.FkIdent = @PkIdent AND ErrorData.Nr IN (SELECT TOP (@TopN) Nr FROM (SELECT DISTINCT Nr FROM ErrorData) t)
END


-- In case the above SELECT query returns NULL
IF (SELECT COUNT(*) FROM @TempTable) = 0
INSERT INTO @idTable 
SELECT -1

INSERT INTO  @idTable
SELECT Id FROM @TempTable 
RETURN

END
GO
------------------------------------------------------------------------------------------------------------------------------------------


------------------------------------------------------------------------------------------------------------------------------------------
------------ Type: TopErrorGroupTableType
------------------------------------------------------------------------------------------------------------------------------------------

--IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[TopErrorGroupVarTable]') AND [type] = 'FN')
--	DROP FUNCTION [dbo].[TopErrorGroupVarTable]
--GO

--IF EXISTS (SELECT * FROM sys.types WHERE is_table_type = 1 AND name ='TopErrorGroupTableType') 
--	DROP TYPE [dbo].[TopErrorGroupTableType]
--GO

--CREATE TYPE TopErrorGroupTableType AS TABLE( 
--	[GroupId] 	[int]	NOT NULL,  -- Id of error selection group
--	[Nr]		[int]	NOT NULL,  -- Nr of entry
--	[StartErrorId]	[int]	NOT NULL,  -- start of error id that belongs to this group
--	[EndErrorId]	[int]	NOT NULL   -- end of error id that belongs to this group

--)
--GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------
------------ Function: TopErrorGroupVarTable
------------------------------------------------------------------------------------------------------------------------------------------
-- =============================================
-- Author:		Thomas Lewien (modified by Pyare Pueschel)
-- Create date: 2015-06-23 (2017-11-24)
-- Description:	Returns error group. Accepts a table with the error id's
-- =============================================

ALTER FUNCTION [dbo].[TopErrorGroupVarTable]
(
-- Add the parameters for the function here
@pkIdent int,
@Teg TopErrorGroupTableType READONLY,
@TopN INT,
@FirstOrNErrors bit
)
RETURNS int
AS
BEGIN

DECLARE @GroupId int

IF @FirstOrNErrors = 1 
BEGIN
	DECLARE @errorCode int
	SET @errorCode = dbo.FirstErrorRejectHistory(@pkIdent)

	IF (@errorCode = -1) RETURN -1	-- good units

	SET @GroupId = (
	SELECT TOP 1 GroupId from @Teg
	WHERE GroupId = @errorCode
	ORDER BY GroupId ASC
	)

	-- Return the result of the function or -2 for all not assigned groups
	RETURN ISNULL(@GroupId,-2)

END
ELSE BEGIN
	DECLARE @TempTable table (Id INT)
	INSERT INTO @TempTable 
	SELECT * FROM dbo.FirstAndNErrorsRejectHistory(@pkIdent, @TopN)

	IF (SELECT MIN(Id) FROM @TempTable) = -1 RETURN -1	-- good units

	SET @GroupId = (
		SELECT TOP 1 GroupId from @Teg
		WHERE GroupId IN (SELECT Id FROM @TempTable)
		ORDER BY GroupId ASC
		)

	-- Return the result of the function or -2 for all not assigned groups
	RETURN ISNULL(@GroupId,-2)
END

RETURN @GroupId

END
GO

--CREATE FUNCTION [dbo].[TopErrorGroupVarTable]
--(
---- Add the parameters for the function here
--@pkIdent int,
--@Teg TopErrorGroupTableType READONLY,
--@TopN INT,
--@FirstOrNErrors bit
--)
--RETURNS int
--AS
--BEGIN

--DECLARE @GroupId int

--IF @FirstOrNErrors = 1 
--BEGIN
--	DECLARE @errorCode int
--	SET @errorCode = dbo.FirstErrorRejectHistory(@pkIdent)

--	IF (@errorCode = -1) RETURN -1	-- good units

--	SET @GroupId = (
--	SELECT TOP 1 GroupId from @Teg
--	WHERE GroupId = @errorCode
--	ORDER BY GroupId ASC
--	)

--	-- Return the result of the function or -2 for all not assigned groups
--	RETURN ISNULL(@GroupId,-2)

--END
--ELSE BEGIN
--	DECLARE @TempTable table (Id INT)
--	INSERT INTO @TempTable 
--	SELECT * FROM dbo.FirstAndNErrorsRejectHistory(@pkIdent, @TopN)

--	IF (SELECT MIN(Id) FROM @TempTable) = -1 RETURN -1	-- good units

--	SET @GroupId = (
--		SELECT TOP 1 GroupId from @Teg
--		WHERE GroupId IN (SELECT Id FROM @TempTable)
--		ORDER BY GroupId ASC
--		)

--	-- Return the result of the function or -2 for all not assigned groups
--	RETURN ISNULL(@GroupId,-2)
--END

--RETURN @GroupId

--END
--GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ Function: TopErrorGroupVarTable
------------------------------------------------------------------------------------------------------------------------------------------
--IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[TopErrorGroup]') AND [type] = 'FN')
--	DROP FUNCTION [dbo].[TopErrorGroup]
--GO

ALTER FUNCTION [dbo].[TopErrorGroup]
(
-- Add the parameters for the function here
@pkIdent int
)
RETURNS int
AS
BEGIN
DECLARE @errorCode int
SET @errorCode = dbo.FirstErrorRejectHistory(@pkIdent)
DECLARE @GroupId int
IF (@errorCode = -1) RETURN -1	-- good units
SET @GroupId = (
SELECT TOP 1 GroupId from dbo.AceErrorSelection
WHERE @errorCode >= StartErrorId
AND @errorCode <= EndErrorId
ORDER BY GroupId ASC
)
-- Return the result of the function or -2 for all not assigned groups
RETURN ISNULL(@GroupId,-2)
END
GO

--CREATE FUNCTION [dbo].[TopErrorGroup]
--(
---- Add the parameters for the function here
--@pkIdent int
--)
--RETURNS int
--AS
--BEGIN
--DECLARE @errorCode int
--SET @errorCode = dbo.FirstErrorRejectHistory(@pkIdent)
--DECLARE @GroupId int
--IF (@errorCode = -1) RETURN -1	-- good units
--SET @GroupId = (
--SELECT TOP 1 GroupId from dbo.AceErrorSelection
--WHERE @errorCode >= StartErrorId
--AND @errorCode <= EndErrorId
--ORDER BY GroupId ASC
--)
---- Return the result of the function or -2 for all not assigned groups
--RETURN ISNULL(@GroupId,-2)
--END

--GO

------------------------------------------------------------------------------------------------------------------------------------------
------------ ShiftMinutes: Drop foreign key constraint and add with CASCADE keyword
------------------------------------------------------------------------------------------------------------------------------------------
IF EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ShiftMinutes_ShiftNames')
BEGIN
	ALTER TABLE [dbo].[ShiftMinutes] DROP CONSTRAINT [FK_ShiftMinutes_ShiftNames]
PRINT 'FK constraint FK_ShiftMinutes_ShiftNames dropped successfully'
END
ELSE PRINT 'FK constraint FK_ShiftMinutes_ShiftNames does not exist'
GO

IF NOT EXISTS (SELECT 'True' FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = 'FK_ShiftMinutes_ShiftNames')
BEGIN
	ALTER TABLE [dbo].[ShiftMinutes] WITH CHECK 
	ADD CONSTRAINT [FK_ShiftMinutes_ShiftNames] FOREIGN KEY([ShiftId])
	REFERENCES [dbo].[ShiftNames] ([Id])
	ON UPDATE CASCADE
	ON DELETE CASCADE
PRINT 'Added FK constraint FK_ShiftMinutes_ShiftNames'
END
ELSE PRINT 'FK constraint FK_ShiftMinutes_ShiftNames already exists'
GO

ALTER TABLE [dbo].[ShiftMinutes] CHECK CONSTRAINT [FK_ShiftMinutes_ShiftNames]
GO
------------------------------------------------------------------------------------------------------------------------------------------


----------------------------------------------------------------------------------------------------------------------------
------------ ResultCountType update with new entry 'Direct'
----------------------------------------------------------------------------------------------------------------------------
IF NOT EXISTS (SELECT Id FROM [ResultCountType] WHERE Id = 5)
BEGIN
	INSERT INTO [ResultCountType] VALUES (5, 'Direct')
	PRINT 'Entry Direct with Id 5 entered into the table ResultCountType'
END
ELSE PRINT 'Table ResultCountType already contains the entry Direct with Id 5'
GO
----------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ Procedure: spAllTests
------------------------------------------------------------------------------------------------------------------------------------------

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[spAllTests]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
	DROP PROCEDURE [dbo].spAllTests
GO

/* 
	
*/ 
CREATE PROCEDURE [dbo].spAllTests 
(
	 @startTime datetime,
	 @endTime datetime,
	 @fixDaysOffset int,
	 @GoodUnitsOut int OUTPUT,
	 @BadUnitsOut int OUTPUT,
	 @AllUnitsOut int OUTPUT

)
 AS
BEGIN

SET NOCOUNT ON
DECLARE 
	@fixWeekTime datetime

	SET @fixWeekTime   = DATEADD(day,@fixDaysOffset,@endTime)

	SELECT 
		 @GoodUnitsOut = SUM(dbo.HasResult(MainData.Result,1)), 
		 @BadUnitsOut = SUM(dbo.HasResult(MainData.Result,2)) + SUM(dbo.HasResult(MainData.Result,4)),  
		 @AllUnitsOut = COUNT(*)  FROM Maindata  
		
		INNER JOIN dbo.TypeID ON MainData.Type = TypeId.Id
		WHERE  
		MainData.Serial NOT IN ( SELECT [Name] FROM [ReferenceSerials])  
		AND TypeID.Name NOT IN ( SELECT [Name] FROM [ReferenceTypes]) 	
		AND (MainData.TestKind = 1)
		AND  ((MainData.Time >= @startTime)  AND  (MainData.Time < @endTime) ) 


END -- procedure spAllTests
GO
------------------------------------------------------------------------------------------------------------------------------------------


------------------------------------------------------------------------------------------------------------------------------------------
------------ Procedure: spAllTestsGroup
------------------------------------------------------------------------------------------------------------------------------------------

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[spAllTestsGroup]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
	DROP PROCEDURE [dbo].spAllTestsGroup
GO

/* 
	
*/ 
CREATE PROCEDURE [dbo].spAllTestsGroup 
(
	 @startTime datetime,
	 @endTime datetime,
	 @fixDaysOffset int,
	 @weekNr int,
	 @isUpdate int		-- --0: insert, 1: update
)
 AS
BEGIN

SET NOCOUNT ON
DECLARE 
	@fixWeekTime datetime,
	@thisCountType int


	DECLARE @TempDetails TABLE 
	(
			[WeekNr]		[int]		NOT NULL,
			[Type]			[int]		NOT NULL,	
			[TestBench]		[int]		NOT NULL,	
			[CountType]		[int]		NOT NULL,	
			[All]			[int]		NULL,
			[Good]			[int]		NULL,
			[Bad]			[int]		NULL
	)

	SELECT @thisCountType = [Id] FROM [dbo].[ResultCountType] WHERE [Name] = 'All'

	SET @fixWeekTime   = DATEADD(day,@fixDaysOffset,@endTime)

	INSERT INTO @TempDetails (
		[WeekNr], 
		[Type],		
		[TestBench],	
		[CountType],	
		[All],		
		[Good],		
		[Bad]
	)		
	SELECT 
		@weekNr,
		t.Id,
		b.Id,
		@thisCountType,
		COUNT(*),  
		SUM(dbo.HasResult(MainData.Result,1)), 
		SUM(dbo.HasResult(MainData.Result,2)) + SUM(dbo.HasResult(MainData.Result,4)) 
	FROM 
		Maindata  
		INNER JOIN dbo.TypeID t ON MainData.Type = t.Id
		INNER JOIN dbo.BenchId b ON MainData.TestBench = b.Id		
		
	WHERE  
		t.Id > 0 AND b.Id > 0 -- not the '*' entries
		AND MainData.Serial NOT IN ( SELECT [Name] FROM [ReferenceSerials])  
	    AND (MainData.TestKind = 1)
		AND t.Name NOT IN ( SELECT [Name] FROM [ReferenceTypes]) 	
		AND  ((MainData.Time >= @startTime)  AND  (MainData.Time < @endTime) ) 


	GROUP BY t.Id, b.Id WITH ROLLUP -- cube
	HAVING GROUPING (t.Id) = 0 AND  GROUPING (b.Id) = 0 -- avoid null entries
	
	IF (@isUpdate = 1)
	BEGIN
		BEGIN TRAN
			DELETE FROM [WeeklyResultDetails] WHERE [WeeklyResultDetails].[WeekNr] = @weekNr AND [CountType] = @thisCountType
	 		INSERT INTO [WeeklyResultDetails] SELECT * FROM @TempDetails	
		COMMIT
	END ELSE
		INSERT INTO [WeeklyResultDetails] SELECT * FROM @TempDetails	

END -- procedure spAllTestsGroup
GO
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------
------------ Procedure: spFirstTest
------------------------------------------------------------------------------------------------------------------------------------------

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[spFirstTest]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
	DROP PROCEDURE [dbo].spFirstTest
GO

/* 
	
*/ 
CREATE PROCEDURE [dbo].spFirstTest 
(
	 @startTime datetime,
	 @endTime datetime,
	 @fixDaysOffset int,
	 @GoodUnitsOut int OUTPUT,
	 @BadUnitsOut int OUTPUT,
	 @AllUnitsOut int OUTPUT

)
 AS
BEGIN

SET NOCOUNT ON
DECLARE 
	@fixWeekTime datetime

	SET @fixWeekTime   = DATEADD(day,@fixDaysOffset,@endTime)

	SELECT 
		 @GoodUnitsOut = SUM(dbo.HasResult(MainData.Result,1)), 
		 @BadUnitsOut = SUM(dbo.HasResult(MainData.Result,2)) + SUM(dbo.HasResult(MainData.Result,4)),  
		 @AllUnitsOut = COUNT(*)  FROM Maindata  
		
		INNER JOIN dbo.TypeID ON MainData.Type = TypeId.Id
		WHERE  
		MainData.Serial NOT IN ( SELECT [Name] FROM [ReferenceSerials])  
		AND TypeID.Name NOT IN ( SELECT [Name] FROM [ReferenceTypes]) 	
		AND  ((MainData.Time >= @startTime)  AND  (MainData.Time < @endTime) ) 
		AND
		(  
			MainData.Ordinal =1 
			OR (
				(SELECT Count(*) FROM MainData t
				WHERE t.Serial = MainData.Serial  AND t.Type = MainData.Type 
					AND t.Time < MainData.Time 
					AND t.TestKind = MainData.TestKind 
					AND t.Time >= @startTime AND t.time < MainData.Time )
				 = 0        
			) 
		)

		AND (MainData.TestKind = 1)



END -- procedure spFirstTest
GO
------------------------------------------------------------------------------------------------------------------------------------------


------------------------------------------------------------------------------------------------------------------------------------------
------------ Procedure: spFirstTestGroup
------------------------------------------------------------------------------------------------------------------------------------------

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[spFirstTestGroup]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
	DROP PROCEDURE [dbo].spFirstTestGroup
GO

/* 
	
*/ 
CREATE PROCEDURE [dbo].spFirstTestGroup 
(
	 @startTime datetime,
	 @endTime datetime,
	 @fixDaysOffset int,
	 @weekNr int,
	 @isUpdate int		-- --0: insert, 1: update
)
 AS
BEGIN

SET NOCOUNT ON
DECLARE 
	@fixWeekTime datetime,
	@thisCountType int

	DECLARE @TempDetails TABLE 
	(
			[WeekNr]		[int]		NOT NULL,
			[Type]			[int]		NOT NULL,	
			[TestBench]		[int]		NOT NULL,	
			[CountType]		[int]		NOT NULL,	
			[All]			[int]		NULL,
			[Good]			[int]		NULL,
			[Bad]			[int]		NULL
	)
	
	
	SELECT @thisCountType = [Id] FROM [dbo].[ResultCountType] WHERE [Name] = 'First'

	SET @fixWeekTime   = DATEADD(day,@fixDaysOffset,@endTime)

	INSERT INTO @TempDetails (
		[WeekNr], 
		[Type],		
		[TestBench],	
		[CountType],	
		[All],		
		[Good],		
		[Bad]
	)		

	SELECT 
		@weekNr,
		t.Id,
		b.Id,
		@thisCountType,
		COUNT(*),  
		SUM(dbo.HasResult(MainData.Result,1)), 
		SUM(dbo.HasResult(MainData.Result,2)) + SUM(dbo.HasResult(MainData.Result,4)) 
	FROM 
		Maindata  
		INNER JOIN dbo.TypeID t ON MainData.Type = t.Id
		INNER JOIN dbo.BenchId b ON MainData.TestBench = b.Id
	WHERE  
 		t.Id > 0 AND b.Id > 0 -- not the '*' entries
		AND MainData.Serial NOT IN ( SELECT [Name] FROM [ReferenceSerials])  
		AND (MainData.TestKind = 1)
		AND t.Name NOT IN ( SELECT [Name] FROM [ReferenceTypes]) 	
		AND  ((MainData.Time >= @startTime)  AND  (MainData.Time < @endTime) ) 
		AND
		(  
			MainData.Ordinal =1 
			OR (
				(SELECT Count(*) FROM MainData t
				WHERE t.Serial = MainData.Serial  AND t.Type = MainData.Type 
					AND t.Time < MainData.Time 
					AND t.TestKind = MainData.TestKind 
					AND t.Time >= @startTime AND t.time < MainData.Time )
				 = 0        
			) 
		)

	GROUP BY t.Id, b.Id WITH ROLLUP -- cube
	HAVING GROUPING (t.Id) = 0 AND  GROUPING (b.Id) = 0 -- avoid null entries
	
	IF (@isUpdate = 1)
	BEGIN
		BEGIN TRAN
			DELETE FROM [WeeklyResultDetails] WHERE [WeeklyResultDetails].[WeekNr] = @weekNr AND [CountType] = @thisCountType
	 		INSERT INTO [WeeklyResultDetails] SELECT * FROM @TempDetails	
		COMMIT
	END ELSE
		INSERT INTO [WeeklyResultDetails] SELECT * FROM @TempDetails	
	
		
END -- procedure spFirstTestGroup
GO
------------------------------------------------------------------------------------------------------------------------------------------


------------------------------------------------------------------------------------------------------------------------------------------
------------ Procedure: spLastTest
------------------------------------------------------------------------------------------------------------------------------------------

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[spLastTest]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
	DROP PROCEDURE [dbo].spLastTest
GO

/* 
	
*/ 
CREATE PROCEDURE [dbo].spLastTest 
(
	 @startTime datetime,
	 @endTime datetime,
	 @fixDaysOffset int,
	 @GoodUnitsOut int OUTPUT,
	 @BadUnitsOut int OUTPUT,
	 @AllUnitsOut int OUTPUT

)
 AS
BEGIN

SET NOCOUNT ON
DECLARE 
	@fixWeekTime datetime

	SET @fixWeekTime   = DATEADD(day,@fixDaysOffset,@endTime)

	SELECT 
		 @GoodUnitsOut = SUM(dbo.HasResult(MainData.Result,1)), 
		 @BadUnitsOut = SUM(dbo.HasResult(MainData.Result,2)) + SUM(dbo.HasResult(MainData.Result,4)),  
		 @AllUnitsOut = COUNT(*)  FROM Maindata  
		
		INNER JOIN dbo.TypeID ON MainData.Type = TypeId.Id
		WHERE  
		MainData.Serial NOT IN ( SELECT [Name] FROM [ReferenceSerials])  
		AND TypeID.Name NOT IN ( SELECT [Name] FROM [ReferenceTypes]) 	
		AND (MainData.TestKind = 1)
		AND  ((MainData.Time >= @startTime)  AND  (MainData.Time < @endTime) ) 
		AND  (  (MainData.TimeOfNextMeasurement IS NULL) OR (MainData.TimeOfNextMeasurement > @fixWeekTime) ) 


END -- procedure spLastTest
GO
------------------------------------------------------------------------------------------------------------------------------------------


------------------------------------------------------------------------------------------------------------------------------------------
------------ Procedure: spLastTestGroup
------------------------------------------------------------------------------------------------------------------------------------------

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[spLastTestGroup]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
	DROP PROCEDURE [dbo].spLastTestGroup
GO

/* 
	
*/ 
CREATE PROCEDURE [dbo].spLastTestGroup
(
	 @startTime datetime,
	 @endTime datetime,
	 @fixDaysOffset int,
	 @weekNr int,
	 @isUpdate int		-- --0: insert, 1: update

)
 AS
BEGIN

SET NOCOUNT ON
DECLARE 
	@fixWeekTime datetime,
	@thisCountType int


	
	DECLARE @TempDetails TABLE 
	(
			[WeekNr]		[int]		NOT NULL,
			[Type]			[int]		NOT NULL,	
			[TestBench]		[int]		NOT NULL,	
			[CountType]		[int]		NOT NULL,	
			[All]			[int]		NULL,
			[Good]			[int]		NULL,
			[Bad]			[int]		NULL
	)

	SELECT @thisCountType = [Id] FROM [dbo].[ResultCountType] WHERE [Name] = 'Last'

	SET @fixWeekTime   = DATEADD(day,@fixDaysOffset,@endTime)

	INSERT INTO @TempDetails (
		[WeekNr], 
		[Type],		
		[TestBench],	
		[CountType],	
		[All],		
		[Good],		
		[Bad]
	)		

	SELECT 
		@weekNr,
		t.Id,
		b.Id,
		@thisCountType,
		COUNT(*),  
		SUM(dbo.HasResult(MainData.Result,1)), 
		SUM(dbo.HasResult(MainData.Result,2)) + SUM(dbo.HasResult(MainData.Result,4)) 
	FROM 
		Maindata  
		INNER JOIN dbo.TypeID t ON MainData.Type = t.Id
		INNER JOIN dbo.BenchId b ON MainData.TestBench = b.Id

	WHERE
		t.Id > 0 AND b.Id > 0 -- not the '*' entries  
		AND MainData.Serial NOT IN ( SELECT [Name] FROM [ReferenceSerials])  
		AND t.Name NOT IN ( SELECT [Name] FROM [ReferenceTypes]) 
		AND (MainData.TestKind = 1)	
		AND  ((MainData.Time >= @startTime)  AND  (MainData.Time < @endTime) ) 
		AND  (  (MainData.TimeOfNextMeasurement IS NULL) OR (MainData.TimeOfNextMeasurement > @fixWeekTime) ) 

	GROUP BY t.Id, b.Id WITH ROLLUP -- cube
	HAVING GROUPING (t.Id) = 0 AND  GROUPING (b.Id) = 0 -- avoid null entries
	
	IF (@isUpdate = 1)
	BEGIN
		BEGIN TRAN
			DELETE FROM [WeeklyResultDetails] WHERE [WeeklyResultDetails].[WeekNr] = @weekNr AND [CountType] = @thisCountType
	 		INSERT INTO [WeeklyResultDetails] SELECT * FROM @TempDetails	
		COMMIT
	END ELSE
		INSERT INTO [WeeklyResultDetails] SELECT * FROM @TempDetails	
	
		
END -- procedure spLastTestGroup
GO
------------------------------------------------------------------------------------------------------------------------------------------


------------------------------------------------------------------------------------------------------------------------------------------
------------ Procedure: spCurrentReject
------------------------------------------------------------------------------------------------------------------------------------------

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[spCurrentReject]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
	DROP PROCEDURE [dbo].spCurrentReject
GO

/* 
	
*/ 
CREATE PROCEDURE [dbo].spCurrentReject 
(
	 @startTime datetime,
	 @endTime datetime,
	 @fixDaysOffset int,
	 @GoodUnitsOut int OUTPUT,
	 @BadUnitsOut int OUTPUT,
	 @AllUnitsOut int OUTPUT

)
 AS
BEGIN

SET NOCOUNT ON
DECLARE 
	@fixWeekTime datetime

	SET @fixWeekTime   = DATEADD(day,@fixDaysOffset,@endTime)

	SELECT 
		 @GoodUnitsOut = SUM(dbo.HasResult(MainData.Result,1)), 
		 @BadUnitsOut = SUM(dbo.HasResult(MainData.Result,2)) + SUM(dbo.HasResult(MainData.Result,4)),  
		 @AllUnitsOut = COUNT(*)  FROM Maindata  
		
		INNER JOIN dbo.TypeID ON MainData.Type = TypeId.Id
		WHERE  
		MainData.Serial NOT IN ( SELECT [Name] FROM [ReferenceSerials])  
		AND TypeID.Name NOT IN ( SELECT [Name] FROM [ReferenceTypes]) 
		AND (MainData.TestKind = 1)
		
		AND  
		((MainData.Time >= @startTime)  AND  (MainData.Time < @endTime) ) 
		AND
		  
		( 
		   -- Only Test 
		  ((MainData.ThisLast = 1) AND (MainData.Ordinal = 1) ) 
		
		  OR  
		  -- First test and same as any test there after
		  (
			(MainData.ThisLast = 0) AND (MainData.Ordinal = 1)   
			AND  ( SELECT COUNT(*) FROM  MainData e 
				WHERE 
					e.Serial = MainData.Serial  
					AND e.Type = MainData.Type  
					AND e.Time > = MainData.Time  
					AND e.Time <  @fixWeekTime  
					AND e.Result <> MainData.Result)  = 0  
		  )  
		 
		  OR
		   -- Last Test:  different from test before	
		  (
			(MainData.Ordinal > 1) AND (MainData.ThisLast = 1) AND
			(
				-- no earlier tests of same kind
				(
					(Select Count(*) FROM MainData o   WHERE o.Serial = MainData.Serial   
													AND o.Type = MainData.Type 
													AND o.TestKind = MainData.TestKind
													AND o.Time < MainData.Time)
					= 0
				)
				OR
				-- different from last test
				( SELECT COUNT(*) FROM  MainData e 
				WHERE 
					e.Serial = MainData.Serial   
					AND e.Type = MainData.Type  
					AND e.Ordinal =  
					-- (MainData.Ordinal-1) for the same TestKind 
					(Select Max(Ordinal) FROM MainData o   WHERE o.Serial = MainData.Serial   
											AND o.Type = MainData.Type AND o.TestKind = MainData.TestKind
											AND o.Time < MainData.Time
					)
					AND e.Result <> MainData.Result)  > 0   
			)
		  )
					 
		  OR  
		  -- Any test: different from test before and not different from any test thereafter
		  (	MainData.Ordinal > 1 AND
			(				
				(					
					-- no earlier tests of same kind
					(Select Count(*) FROM MainData o   WHERE o.Serial = MainData.Serial   
													AND o.Type = MainData.Type 
													AND o.TestKind = MainData.TestKind
													AND o.Time < MainData.Time)
					= 0
					
					OR  -- different from last test
					( SELECT COUNT(*) FROM  MainData e 
						WHERE 
							e.Serial = MainData.Serial   
							AND e.Type = MainData.Type  
							AND e.Ordinal = 
								-- (MainData.Ordinal-1) for the same TestKind 
								(Select Max(Ordinal) FROM MainData o   WHERE o.Serial = MainData.Serial   
														AND o.Type = MainData.Type 
														AND o.TestKind = MainData.TestKind
														AND o.Time < MainData.Time
								)
							AND e.Result <> MainData.Result)  > 0
				)
				-- no later result that is different
				AND   ( SELECT COUNT(*) FROM  MainData e 
					WHERE e.Serial = MainData.Serial  
						AND e.Type = MainData.Type  
						AND e.Time > = MainData.Time  
						AND  e.Time <  @fixWeekTime  
						AND e.Result <> MainData.Result)  
				= 0  
				 
			)
		  )
		) 
					


END -- procedure spCurrentReject
GO
------------------------------------------------------------------------------------------------------------------------------------------


------------------------------------------------------------------------------------------------------------------------------------------
------------ Procedure: spCurrentRejectGroup
------------------------------------------------------------------------------------------------------------------------------------------

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[spCurrentRejectGroup]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
	DROP PROCEDURE [dbo].spCurrentRejectGroup
GO

/* 
	
*/ 
CREATE PROCEDURE [dbo].spCurrentRejectGroup 
(
	 @startTime datetime,
	 @endTime datetime,
	 @fixDaysOffset int,
	 @weekNr int,
	 @isUpdate int		-- --0: insert, 1: update


)
 AS
BEGIN

SET NOCOUNT ON
DECLARE 
	@fixWeekTime datetime,
	@thisCountType int
	
	DECLARE @TempDetails TABLE 
	(
			[WeekNr]		[int]		NOT NULL,
			[Type]			[int]		NOT NULL,	
			[TestBench]		[int]		NOT NULL,	
			[CountType]		[int]		NOT NULL,	
			[All]			[int]		NULL,
			[Good]			[int]		NULL,
			[Bad]			[int]		NULL
	)

	SELECT @thisCountType = [Id] FROM [dbo].[ResultCountType] WHERE [Name] = 'CurrentReject'

	SET @fixWeekTime   = DATEADD(day,@fixDaysOffset,@endTime)

	INSERT INTO @TempDetails (
		[WeekNr], 
		[Type],		
		[TestBench],	
		[CountType],	
		[All],		
		[Good],		
		[Bad]
	)
		SELECT 
		@weekNr,
		t.Id,
		b.Id,
		@thisCountType,
		COUNT(*),  
		SUM(dbo.HasResult(MainData.Result,1)), 
		SUM(dbo.HasResult(MainData.Result,2)) + SUM(dbo.HasResult(MainData.Result,4)) 
	FROM 
		Maindata  
		INNER JOIN dbo.TypeID t ON MainData.Type = t.Id
		INNER JOIN dbo.BenchId b ON MainData.TestBench = b.Id

		WHERE  
		t.Id > 0 AND b.Id > 0 -- not the '*' entries
		AND MainData.Serial NOT IN ( SELECT [Name] FROM [ReferenceSerials])  
		AND t.Name NOT IN ( SELECT [Name] FROM [ReferenceTypes]) 
		AND (MainData.TestKind = 1)
		
		AND  ((MainData.Time >= @startTime)  AND  (MainData.Time < @endTime) ) 
		AND
		-- Only Test 
		( ((MainData.ThisLast = 1) AND (MainData.Ordinal = 1) ) 
		
		OR  
		-- First test and same as any test there after
		(  (MainData.ThisLast = 0) AND (MainData.Ordinal = 1)   
		AND  ( SELECT COUNT(*) FROM  MainData e 
				WHERE 
					e.Serial = MainData.Serial  
					AND e.Type = MainData.Type  
					AND e.Time > = MainData.Time  
					AND e.Time <  @fixWeekTime  
					AND e.Result <> MainData.Result)  = 0  )  
		 
		  OR
		   -- Last Test:  different from test before	
		  (
			(MainData.Ordinal > 1) AND (MainData.ThisLast = 1) AND
			(
				-- no earlier tests of same kind
				(
					(Select Count(*) FROM MainData o   WHERE o.Serial = MainData.Serial   
													AND o.Type = MainData.Type 
													AND o.TestKind = MainData.TestKind
													AND o.Time < MainData.Time)
					= 0
				)
				OR
				-- different from last test
				( SELECT COUNT(*) FROM  MainData e 
				WHERE 
					e.Serial = MainData.Serial   
					AND e.Type = MainData.Type  
					AND e.Ordinal =  
					-- (MainData.Ordinal-1) for the same TestKind 
					(Select Max(Ordinal) FROM MainData o   WHERE o.Serial = MainData.Serial   
											AND o.Type = MainData.Type AND o.TestKind = MainData.TestKind
											AND o.Time < MainData.Time
					)
					AND e.Result <> MainData.Result)  > 0   
			)
		  )
					 
		 
					 
		  OR  
		  -- Any test: different from test before and not different from any test thereafter
		  (	MainData.Ordinal > 1 AND
			(				
				(					
					-- no earlier tests of same kind
					(Select Count(*) FROM MainData o   WHERE o.Serial = MainData.Serial   
													AND o.Type = MainData.Type 
													AND o.TestKind = MainData.TestKind
													AND o.Time < MainData.Time)
					= 0
					
					OR  -- different from last test
					( SELECT COUNT(*) FROM  MainData e 
						WHERE 
							e.Serial = MainData.Serial   
							AND e.Type = MainData.Type  
							AND e.Ordinal = 
								-- (MainData.Ordinal-1) for the same TestKind 
								(Select Max(Ordinal) FROM MainData o   WHERE o.Serial = MainData.Serial   
														AND o.Type = MainData.Type 
														AND o.TestKind = MainData.TestKind
														AND o.Time < MainData.Time
								)
							AND e.Result <> MainData.Result)  > 0
				)
				-- no later result that is different
				AND   ( SELECT COUNT(*) FROM  MainData e 
					WHERE e.Serial = MainData.Serial  
						AND e.Type = MainData.Type  
						AND e.Time > = MainData.Time  
						AND  e.Time <  @fixWeekTime  
						AND e.Result <> MainData.Result)  
				= 0  
				 
			)
		  )			
    ) 
					
	GROUP BY t.Id, b.Id WITH ROLLUP -- cube
	HAVING GROUPING (t.Id) = 0 AND  GROUPING (b.Id) = 0 -- avoid null entries
	
	IF (@isUpdate = 1)
	BEGIN
		BEGIN TRAN
			DELETE FROM [WeeklyResultDetails] WHERE [WeeklyResultDetails].[WeekNr] = @weekNr AND [CountType] = @thisCountType
	 		INSERT INTO [WeeklyResultDetails] SELECT * FROM @TempDetails	
		COMMIT
	END ELSE
		INSERT INTO [WeeklyResultDetails] SELECT * FROM @TempDetails	
	

END -- procedure spCurrentRejectGroup
GO
------------------------------------------------------------------------------------------------------------------------------------------


------------------------------------------------------------------------------------------------------------------------------------------
------------ Procedure: spDirectTestGroup
------------------------------------------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------------------------------------------------------

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[spDirectTestGroup]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
	DROP PROCEDURE [dbo].spDirectTestGroup
GO

/* 
	
*/ 

CREATE PROCEDURE [dbo].[spDirectTestGroup]
(
@startTime datetime,
@endTime datetime,
@fixDaysOffset int,
@weekNr int,
@isUpdate int,		-- --0: insert, 1: update
@maxMinutes int, 
@maxOrdinal int
)
AS
BEGIN
SET NOCOUNT ON
DECLARE
@fixWeekTime datetime,
@thisCountType int
DECLARE @TempDetails TABLE
(
[WeekNr]		[int]		NOT NULL,
[Type]			[int]		NOT NULL,
[TestBench]		[int]		NOT NULL,
[CountType]		[int]		NOT NULL,
[All]			[int]		NULL,
[Good]			[int]		NULL,
[Bad]			[int]		NULL
)
SELECT @thisCountType = [Id] FROM [dbo].[ResultCountType] WHERE [Name] = 'Direct'
SET @fixWeekTime   = DATEADD(day,@fixDaysOffset,@endTime)

-- Both conditions are met
IF (@maxOrdinal > 0 AND @maxMinutes > 0)
BEGIN
		 INSERT INTO @TempDetails (
		[WeekNr],
		[Type],
		[TestBench],
		[CountType],
		[All],
		[Good],
		[Bad]
		)
		SELECT
		@weekNr,
		t.Id,
		b.Id,
		@thisCountType,
		COUNT(*),
		SUM(dbo.HasResult(MainData.Result,1)),
		SUM(dbo.HasResult(MainData.Result,2)) + SUM(dbo.HasResult(MainData.Result,4))
		FROM
		Maindata
		INNER JOIN dbo.TypeID t ON MainData.Type = t.Id
		INNER JOIN dbo.BenchId b ON MainData.TestBench = b.Id
		WHERE
		t.Id > 0 AND b.Id > 0 -- not the '*' entries
		AND MainData.Serial NOT IN ( SELECT [Name] FROM [ReferenceSerials])
		AND t.Name NOT IN ( SELECT [Name] FROM [ReferenceTypes])
		AND (MainData.TestKind = 1)
		AND  ((MainData.Time >= @startTime)  AND  (MainData.Time < @endTime))
		
		-- Only Test
		AND (((MainData.ThisLast = 1) AND (MainData.Ordinal = 1))
		
            -- first test is BEFORE the time frame and subsequent tests are WITHIN the time frame  
            -- if the result HAS changed count the subsequent test
            -- last test within maxHours is also the last test overall 
            OR PkIdent IN (
                 SELECT b.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result <> a.Result 
                 WHERE 
                 a.Ordinal = 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <=  @maxMinutes
                 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time < @startTime 
                 AND ((b.Ordinal = @maxOrdinal )
                 OR (b.Ordinal > 1 AND b.Ordinal < @maxOrdinal AND b.ThisLast = 1))
                 ) 

            -- first test is BEFORE the time frame and subsequent tests are WITHIN the time frame  
            -- if the result HAS changed count the subsequent test
            -- last test within maxHours which is NOT the last test overall
			-- Get the subsequent test which equals the maxOrdinal and is within the time frame OR is < maxOrdinal and the last one within the time frame
            OR PkIdent IN (
                 SELECT b.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result <> a.Result 
                 WHERE 
                 a.Ordinal = 1 
				 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time < @startTime
				 AND ((b.Ordinal = @maxOrdinal )
                 OR (b.Ordinal > 1 AND b.Ordinal < @maxOrdinal AND b.ThisLast = 1))
                 )

			-- first test is WITHIN the time frame and subsequent tests are AFTER the time frame  
            -- if the result has NOT changed count the first test
            -- last test within maxHours which is also the last test overall
			-- Get the subsequent test which equals the maxOrdinal and is within the time frame OR is < maxOrdinal and the last one within the time frame
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result = a.Result 
                 WHERE 
                 a.Ordinal = 1 
				 AND a.Time > = @startTime AND a.Time < @endTime
				 AND b.Time > = @endTime 
                 AND ((b.Ordinal = @maxOrdinal )
                 OR (b.Ordinal > 1 AND b.Ordinal < @maxOrdinal AND b.ThisLast = 1)) 
                 ) 

            -- first test is WITHIN the time frame and subsequent tests are AFTER the time frame  
            -- if the result has NOT changed count the first test
            -- last test within maxHours which is NOT the last test overall
			-- Get the subsequent test which equals the maxOrdinal and is within the time frame OR is < maxOrdinal and the last one within the time frame
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result = a.Result 
                 WHERE 
                 a.Ordinal = 1 
				 AND a.Time > = @startTime AND a.Time < @endTime
				 AND b.Time > = @endTime
				 AND ((b.Ordinal = @maxOrdinal )
                 OR (b.Ordinal > 1 AND b.Ordinal < @maxOrdinal AND b.ThisLast = 1)) 
                 )

			-- first and subsequent tests are WITHIN the time frame 
            -- if the result has NOT changed count the first test
            -- last test within maxHours which is also the last test overall
			-- Get the subsequent test which equals the maxOrdinal and is within the time frame OR is < maxOrdinal and the last one within the time frame
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result = a.Result 
				 WHERE 
                 a.Ordinal = 1 
				 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time > = @startTime AND a.Time < @endTime
                 AND ((b.Ordinal = @maxOrdinal )
                 OR (b.Ordinal > 1 AND b.Ordinal < @maxOrdinal AND b.ThisLast = 1))
                 ) 

            -- first and subsequent tests are WITHIN the time frame 
            -- if the result has NOT changed count the first test
            -- last test within maxHours which is NOT the last test overall
			-- Get the subsequent test which equals the maxOrdinal and is within the time frame OR is < maxOrdinal and the last one within the time frame
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result = a.Result 
				 WHERE 
                 a.Ordinal = 1 
				 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time > = @startTime AND a.Time < @endTime
				 AND ((b.Ordinal = @maxOrdinal )
                 OR (b.Ordinal > 1 AND b.Ordinal < @maxOrdinal AND b.ThisLast = 1))
                 )

			-- first and subsequent tests are WITHIN the time frame 
            -- if the result HAS changed count the subsequent test
            -- last test within maxHours which is also the last test overall
			-- Get the subsequent test which equals the maxOrdinal and is within the time frame OR is < maxOrdinal and the last one within the time frame
            OR PkIdent IN (
                 SELECT b.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result <> a.Result 
				 WHERE 
                 a.Ordinal = 1 
				 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time > = @startTime AND a.Time < @endTime
                 AND ((b.Ordinal = @maxOrdinal )
                 OR (b.Ordinal > 1 AND b.Ordinal < @maxOrdinal AND b.ThisLast = 1))
                 ) 

            -- first and subsequent tests are WITHIN the time frame 
            -- if the result HAS changed count the subsequent test
            -- last test within maxHours which is NOT the last test overall
			-- Get the subsequent test which equals the maxOrdinal and is within the time frame OR is < maxOrdinal and the last one within the time frame
            OR PkIdent IN (
                 SELECT b.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result <> a.Result 
				 WHERE 
                 a.Ordinal = 1 
				 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time > = @startTime AND a.Time < @endTime
				 AND ((b.Ordinal = @maxOrdinal )
                 OR (b.Ordinal > 1 AND b.Ordinal < @maxOrdinal AND b.ThisLast = 1))
                 ))

		GROUP BY t.Id, b.Id WITH ROLLUP -- cube
		HAVING GROUPING (t.Id) = 0 AND  GROUPING (b.Id) = 0 -- avoid null entries
END
-- Only the max ordinal condition is met
ELSE IF (@maxOrdinal > 0 AND @maxMinutes = 0)
BEGIN
		 INSERT INTO @TempDetails (
		[WeekNr],
		[Type],
		[TestBench],
		[CountType],
		[All],
		[Good],
		[Bad]
		)
		SELECT
		@weekNr,
		t.Id,
		b.Id,
		@thisCountType,
		COUNT(*),
		SUM(dbo.HasResult(MainData.Result,1)),
		SUM(dbo.HasResult(MainData.Result,2)) + SUM(dbo.HasResult(MainData.Result,4))
		FROM
		Maindata
		INNER JOIN dbo.TypeID t ON MainData.Type = t.Id
		INNER JOIN dbo.BenchId b ON MainData.TestBench = b.Id
		WHERE
		t.Id > 0 AND b.Id > 0 -- not the '*' entries
		AND MainData.Serial NOT IN ( SELECT [Name] FROM [ReferenceSerials])
		AND t.Name NOT IN ( SELECT [Name] FROM [ReferenceTypes])
		AND (MainData.TestKind = 1)
		AND  ((MainData.Time >= @startTime)  AND  (MainData.Time < @endTime))
		
		-- Only Test
		AND (((MainData.ThisLast = 1) AND (MainData.Ordinal = 1))
		
			-- First test within the time slot but 2nd test outside the maxHours
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind 
                 WHERE 
                 a.Ordinal = 1 AND b.Ordinal = 2 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) >  @maxMinutes
                 AND a.Time > = @startTime AND a.Time < @endTime 
                 ) 

            -- first test is BEFORE the time frame and subsequent tests are WITHIN the time frame  
            -- if the result HAS changed count the subsequent test
            -- last test within maxHours is also the last test overall 
            OR PkIdent IN (
                 SELECT b.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result <> a.Result 
                 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <=  @maxMinutes
                 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time < @startTime 
                 AND b.ThisLast = 1 
                 ) 

            -- first test is BEFORE the time frame and subsequent tests are WITHIN the time frame  
            -- if the result HAS changed count the subsequent test
            -- last test within maxHours which is NOT the last test overall
            OR PkIdent IN (
                 SELECT b.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result <> a.Result 
                 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <= @maxMinutes
                 AND DATEDIFF(mi, a.[Time], b.[TimeOfNextMeasurement]) >  @maxMinutes
                 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time < @startTime
                 )

			-- first test is WITHIN the time frame and subsequent tests are AFTER the time frame  
            -- if the result has NOT changed count the first test
            -- last test within maxHours which is also the last test overall
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result = a.Result 
                 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <=  @maxMinutes
                 AND a.Time > = @startTime AND a.Time < @endTime
				 AND b.Time > = @endTime 
                 AND b.ThisLast = 1 
                 ) 

            -- first test is WITHIN the time frame and subsequent tests are AFTER the time frame  
            -- if the result has NOT changed count the first test
            -- last test within maxHours which is NOT the last test overall
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result = a.Result 
                 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <= @maxMinutes
                 AND DATEDIFF(mi, a.[Time], b.[TimeOfNextMeasurement]) >  @maxMinutes
                 AND a.Time > = @startTime AND a.Time < @endTime
				 AND b.Time > = @endTime 
                 )

			-- first and subsequent tests are WITHIN the time frame 
            -- if the result has NOT changed count the first test
            -- last test within maxHours which is also the last test overall
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result = a.Result 
				 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <=  @maxMinutes
                 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time > = @startTime AND a.Time < @endTime
                 AND b.ThisLast = 1 
                 ) 

            -- first and subsequent tests are WITHIN the time frame 
            -- if the result has NOT changed count the first test
            -- last test within maxHours which is NOT the last test overall
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result = a.Result 
				 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <= @maxMinutes
                 AND DATEDIFF(mi, a.[Time], b.[TimeOfNextMeasurement]) >  @maxMinutes
                 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time > = @startTime AND a.Time < @endTime
                 )

			-- first and subsequent tests are WITHIN the time frame 
            -- if the result HAS changed count the subsequent test
            -- last test within maxHours which is also the last test overall
            OR PkIdent IN (
                 SELECT b.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result <> a.Result 
				 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <=  @maxMinutes
                 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time > = @startTime AND a.Time < @endTime
                 AND b.ThisLast = 1 
                 ) 

            -- first and subsequent tests are WITHIN the time frame 
            -- if the result HAS changed count the subsequent test
            -- last test within maxHours which is NOT the last test overall
            OR PkIdent IN (
                 SELECT b.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result <> a.Result 
				 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <= @maxMinutes
                 AND DATEDIFF(mi, a.[Time], b.[TimeOfNextMeasurement]) >  @maxMinutes
                 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time > = @startTime AND a.Time < @endTime
                 ))

		GROUP BY t.Id, b.Id WITH ROLLUP -- cube
		HAVING GROUPING (t.Id) = 0 AND  GROUPING (b.Id) = 0 -- avoid null entries

END
--Only the max hours condition is met
ELSE IF (@maxOrdinal = 0 AND @maxMinutes > 0)
BEGIN
		 INSERT INTO @TempDetails (
		[WeekNr],
		[Type],
		[TestBench],
		[CountType],
		[All],
		[Good],
		[Bad]
		)
		SELECT
		@weekNr,
		t.Id,
		b.Id,
		@thisCountType,
		COUNT(*),
		SUM(dbo.HasResult(MainData.Result,1)),
		SUM(dbo.HasResult(MainData.Result,2)) + SUM(dbo.HasResult(MainData.Result,4))
		FROM
		Maindata
		INNER JOIN dbo.TypeID t ON MainData.Type = t.Id
		INNER JOIN dbo.BenchId b ON MainData.TestBench = b.Id
		WHERE
		t.Id > 0 AND b.Id > 0 -- not the '*' entries
		AND MainData.Serial NOT IN ( SELECT [Name] FROM [ReferenceSerials])
		AND t.Name NOT IN ( SELECT [Name] FROM [ReferenceTypes])
		AND (MainData.TestKind = 1)
		AND  ((MainData.Time >= @startTime)  AND  (MainData.Time < @endTime))
		
		-- Only Test
		AND (((MainData.ThisLast = 1) AND (MainData.Ordinal = 1))
		
			-- First test within the time slot but 2nd test outside the maxHours
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind 
                 WHERE 
                 a.Ordinal = 1 AND b.Ordinal = 2 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) >  @maxMinutes
                 AND a.Time > = @startTime AND a.Time < @endTime 
                 ) 

            -- first test is BEFORE the time frame and subsequent tests are WITHIN the time frame  
            -- if the result HAS changed count the subsequent test
            -- last test within maxHours is also the last test overall 
            OR PkIdent IN (
                 SELECT b.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result <> a.Result 
                 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <=  @maxMinutes
                 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time < @startTime 
                 AND b.ThisLast = 1 
                 ) 

            -- first test is BEFORE the time frame and subsequent tests are WITHIN the time frame  
            -- if the result HAS changed count the subsequent test
            -- last test within maxHours which is NOT the last test overall
            OR PkIdent IN (
                 SELECT b.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result <> a.Result 
                 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <= @maxMinutes
                 AND DATEDIFF(mi, a.[Time], b.[TimeOfNextMeasurement]) >  @maxMinutes
                 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time < @startTime
                 )

			-- first test is WITHIN the time frame and subsequent tests are AFTER the time frame  
            -- if the result has NOT changed count the first test
            -- last test within maxHours which is also the last test overall
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result = a.Result 
                 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <=  @maxMinutes
                 AND a.Time > = @startTime AND a.Time < @endTime
				 AND b.Time > = @endTime 
                 AND b.ThisLast = 1 
                 ) 

            -- first test is WITHIN the time frame and subsequent tests are AFTER the time frame  
            -- if the result has NOT changed count the first test
            -- last test within maxHours which is NOT the last test overall
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result = a.Result 
                 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <= @maxMinutes
                 AND DATEDIFF(mi, a.[Time], b.[TimeOfNextMeasurement]) >  @maxMinutes
                 AND a.Time > = @startTime AND a.Time < @endTime
				 AND b.Time > = @endTime 
                 )

			-- first and subsequent tests are WITHIN the time frame 
            -- if the result has NOT changed count the first test
            -- last test within maxHours which is also the last test overall
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result = a.Result 
				 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <=  @maxMinutes
                 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time > = @startTime AND a.Time < @endTime
                 AND b.ThisLast = 1 
                 ) 

            -- first and subsequent tests are WITHIN the time frame 
            -- if the result has NOT changed count the first test
            -- last test within maxHours which is NOT the last test overall
            OR PkIdent IN (
                 SELECT a.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result = a.Result 
				 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <= @maxMinutes
                 AND DATEDIFF(mi, a.[Time], b.[TimeOfNextMeasurement]) >  @maxMinutes
                 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time > = @startTime AND a.Time < @endTime
                 )

			-- first and subsequent tests are WITHIN the time frame 
            -- if the result HAS changed count the subsequent test
            -- last test within maxHours which is also the last test overall
            OR PkIdent IN (
                 SELECT b.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result <> a.Result 
				 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <=  @maxMinutes
                 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time > = @startTime AND a.Time < @endTime
                 AND b.ThisLast = 1 
                 ) 

            -- first and subsequent tests are WITHIN the time frame 
            -- if the result HAS changed count the subsequent test
            -- last test within maxHours which is NOT the last test overall
            OR PkIdent IN (
                 SELECT b.PkIdent FROM MainData a INNER JOIN MainData b ON 
                 b.Serial = a.Serial 
                 AND b.[Type] = a.[Type] 
                 AND b.TestKind = a.TestKind
				 AND b.Result <> a.Result 
				 WHERE 
                 a.Ordinal = 1 AND b.Ordinal > 1 
                 AND DATEDIFF(mi, a.[Time], b.[Time]) <= @maxMinutes
                 AND DATEDIFF(mi, a.[Time], b.[TimeOfNextMeasurement]) >  @maxMinutes
                 AND b.Time > = @startTime AND b.Time < @endTime
				 AND a.Time > = @startTime AND a.Time < @endTime
                 ))

		GROUP BY t.Id, b.Id WITH ROLLUP -- cube
		HAVING GROUPING (t.Id) = 0 AND  GROUPING (b.Id) = 0 -- avoid null entries

END

IF (@isUpdate = 1)
BEGIN
BEGIN TRAN
DELETE FROM [WeeklyResultDetails] WHERE [WeeklyResultDetails].[WeekNr] = @weekNr AND [CountType] = @thisCountType
INSERT INTO [WeeklyResultDetails] SELECT * FROM @TempDetails
COMMIT
END ELSE
INSERT INTO [WeeklyResultDetails] SELECT * FROM @TempDetails
END -- procedure spDirectTestGroup

GO

------------------------------------------------------------------------------------------------------------------------------------------
------------ Procedure: spDsFillWeeklyData
------------------------------------------------------------------------------------------------------------------------------------------

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[spDsFillWeeklyData]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
	DROP PROCEDURE [dbo].spDsFillWeeklyData
GO

/*
Fill WeeklyResults table
A week starts with the first shift offset from Sunday 0:0 + @shiftStartMinutes
and ends with next weeks Sunday + @shiftEndMinutes
IsoWeek And IsoYear are entered per calculation based on the (first) @weekday
Weeks are counted from the base of times (which is 1990-01-01)
*/
CREATE PROCEDURE [dbo].[spDsFillWeeklyData] (@startTime datetime, @endTime datetime, @fixDaysOffset int, @maxMinutes int, @maxOrdinal int) AS
BEGIN
SET NOCOUNT ON
DECLARE
	@weekCnt int,
	@startWeek int,
	@endWeek int,
	@shiftStartMinutes int,
	@shiftEndMinutes int,
	@now datetime,
	@update int
	set @now = GETDATE()
	EXEC dbo.[parSetTime] @key ='WeekData.LastRun',  @setTime =  @now
	-- shift start and end minutes, end minutes minus one day
	SELECT  @shiftStartMinutes = MIN([Day]*24*60 + [Start]) from ShiftMinutes
	SELECT  @shiftEndMinutes =   MAX(([Day]-1)*24*60 + [Stop]) from ShiftMinutes
	set @startWeek = DATEDIFF(wk, 0,  @startTime)
	set @endWeek =   DATEDIFF(wk, 0,  @endTime)
	set @weekCnt = @startWeek
	DECLARE @start datetime, @end datetime, @weekday datetime
	DECLARE @totalCnt int
	WHILE @weekCnt <= @endWeek
	BEGIN
	set @end =   DATEADD(minute,@shiftEndMinutes,DATEADD(wk,1,(7*@weekCnt)-1))
	IF ( ( ( SELECT COUNT(*) FROM [WeeklyResults] WHERE [WeeklyResults].[WeekNr] = @weekCnt) = 0 )
	OR ( DATEADD(day,@fixDaysOffset,@end) > @now)
	)
	BEGIN
	-- There is no entry in [WeeklyResults] or it can still be changed
	set @start = DATEADD(minute,@shiftStartMinutes,DATEADD(wk,0,(7*@weekCnt)-1))		-- StartTime: Day 0, add shift time
	set @weekday =  DATEADD(day,1,@start)
	IF (( (DATEADD(day,@fixDaysOffset,@end) > @now)   AND ( SELECT COUNT(*) FROM [WeeklyResults] WHERE [WeeklyResults].[WeekNr] = @weekCnt) > 0 ) )
	BEGIN
	-- an update operation
	SET @update = 1
	UPDATE [WeeklyResults] SET
	[Year] = dbo.IsoYear(@weekday),
	[WeekInYear] =  dbo.IsoWeek(@weekday),
	[StartTime] = @start,
	[EndTime] = @end
	WHERE WeekNr = @weekCnt
	END ELSE
	BEGIN
	-- an insert operation
	SET @update = 0
	INSERT INTO [WeeklyResults] ([WeekNr],[Year],[WeekInYear],[StartTime],[EndTime])
	VALUES (
	@weekCnt,
	dbo.IsoYear(@weekday),
	dbo.IsoWeek(@weekday),
	@start,
	@end
	)
	END
	-- generate sum terms for test stand and type groups for the different counting methods
	EXEC dbo.spAllTestsGroup
	@startTime =@start,
	@endTime = @end,
	@fixDaysOffset = @fixDaysOffset,
	@weekNr = @weekCnt,
	@isUpdate = @update
	EXEC dbo.spFirstTestGroup
	@startTime =@start,
	@endTime = @end,
	@fixDaysOffset = @fixDaysOffset,
	@weekNr = @weekCnt,
	@isUpdate = @update
	EXEC dbo.spLastTestGroup
	@startTime =@start,
	@endTime = @end,
	@fixDaysOffset = @fixDaysOffset,
	@weekNr = @weekCnt,
	@isUpdate = @update
	EXEC dbo.spCurrentRejectGroup
	@startTime =@start,
	@endTime = @end,
	@fixDaysOffset = @fixDaysOffset,
	@weekNr = @weekCnt,
	@isUpdate = @update
	EXEC dbo.spDirectTestGroup
	@startTime =@start,
	@endTime = @end,
	@fixDaysOffset = @fixDaysOffset,
	@weekNr = @weekCnt,
	@isUpdate = @update,
	@maxMinutes = @maxMinutes,
	@maxOrdinal = @maxOrdinal
	EXEC dbo.spTestRollup				-- generate sum terms for test stands and types
	@weekNr = @weekCnt
	SELECT @totalCnt = 0
	SELECT @totalCnt = [All] FROM [WeeklyResultDetails] WHERE [Type]= 0 AND [TestBench] = 0 AND [WeekNr] = @weekCnt AND [CountType] = 2
	IF (NOT @totalCnt IS NULL)
	Print ('Units:' + CAST(@totalCnt AS nvarchar ) + ' Week: ' + CAST(@weekCnt AS nvarchar) + ' Start: ' + CAST(@start AS nvarchar) + ' End: ' + CAST(@end AS nvarchar))
	END --there is no entry or it can still be changed
	SET @weekCnt = @weekCnt+1
	END
END -- procedure spDsFillWeeklyData

GO
------------------------------------------------------------------------------------------------------------------------------------------
