
PRINT 'Creating time limits of types...'
GO

-- If table xTimeLimitsOfTypes exists
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[xTimeLimitsOfTypes]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
	-- If table is not empty, check its entries
	IF ((SELECT COUNT(Id) FROM xTimeLimitsOfTypes) <> 0)
	BEGIN 
		-- Check whether any units remain to be copied. If all or no units have been transferred, then truncate the table!
		IF ((SELECT MAX(Transferred) FROM xTimeLimitsOfTypes) = (SELECT MIN(Transferred) FROM xTimeLimitsOfTypes))
		BEGIN 
			TRUNCATE TABLE [dbo].[xTimeLimitsOfTypes]

			IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[TimeLimitsOfTypes]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
			BEGIN
				-- If table TimeLimitsOfTypes exists and is not empty, then insert values into xTimeLimitsOfTypes.
				IF ((SELECT COUNT(Id) FROM TimeLimitsOfTypes) <> 0)
				BEGIN
					INSERT INTO xTimeLimitsOfTypes
					(Id, Name, BenchId, BenchName, LimitTime, UnitsToDelete, UnitsRetained)
					SELECT Id, Name, BenchId, BenchName, ActualTimeLimit, UnitsToDelete, UnitsRetained
					FROM TimeLimitsOfTypes	
					WHERE Id <> 0	 
		
					PRINT 'Values inserted into xTimeLimitsOfTypes.'
				END
				-- If table TimeLimitsOfTypes is empty, then raise error.
				ELSE RAISERROR ('Table TimeLimitsOfTypes does not contain any entries. Please setup CleanupDb parameters first.',11,1) WITH NOWAIT	
			END
			-- If table TimeLimitsOfTypes does not exist, then raise error.
			ELSE RAISERROR ('Table TimeLimitsOfTypes does not exist. Please create first.',11,1) WITH NOWAIT		 
		END
	END
	ELSE
	-- If table is empty, insert from TimeLimitsOfTypes
	BEGIN
		IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[TimeLimitsOfTypes]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
		BEGIN
			-- If table TimeLimitsOfTypes exists and is not empty, then insert values into xTimeLimitsOfTypes.
			IF ((SELECT COUNT(Id) FROM TimeLimitsOfTypes WHERE Id <> 0) <> 0)
			BEGIN
				INSERT INTO xTimeLimitsOfTypes
				(Id, Name, BenchId, BenchName, LimitTime, UnitsToDelete, UnitsRetained)
				SELECT Id, Name, BenchId, BenchName, ActualTimeLimit, UnitsToDelete, UnitsRetained
				FROM TimeLimitsOfTypes		 
				WHERE Id <> 0
				
				PRINT 'Values inserted into xTimeLimitsOfTypes.'
			END
			-- If table TimeLimitsOfTypes is empty, then raise error.
			ELSE RAISERROR ('Table TimeLimitsOfTypes does not contain any entries. Please setup CleanupDb parameters first.',11,1) WITH NOWAIT	
		END
		-- If table TimeLimitsOfTypes does not exist, then raise error.
		ELSE RAISERROR ('Table TimeLimitsOfTypes does not exist. Please create first.',11,1) WITH NOWAIT
	END
END
ELSE
-- If table xTimeLimitsOfTypes does not exist
BEGIN
	CREATE TABLE [dbo].[xTimeLimitsOfTypes] (
		[Id] [int] NOT NULL ,
		[Name] [nvarchar] (50) COLLATE Latin1_General_CI_AS NULL ,
		[BenchId] [int] NULL ,
		[BenchName] [nvarchar] (50) COLLATE Latin1_General_CI_AS NULL ,
		[LimitTime] [datetime]  ,
		[UnitsToDelete] [int]  ,
		[UnitsRetained] [int] ,
		[Transferred] [int] DEFAULT 0 , 
		[Duration] [int] ,
		[UnitsProtocol] [bigint] , 
	) ON [PRIMARY]

	PRINT 'xTimeLimitsOfTypes created'

	IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[TimeLimitsOfTypes]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
	BEGIN
		-- If table TimeLimitsOfTypes exists and is not empty, then insert values into xTimeLimitsOfTypes.
		IF ((SELECT COUNT(Id) FROM TimeLimitsOfTypes WHERE Id <> 0) <> 0)
		BEGIN
			INSERT INTO xTimeLimitsOfTypes
			(Id, Name, BenchId, BenchName, LimitTime, UnitsToDelete, UnitsRetained)
			SELECT Id, Name, BenchId, BenchName, ActualTimeLimit, UnitsToDelete, UnitsRetained
			FROM TimeLimitsOfTypes
			WHERE Id <> 0		 
		
			PRINT 'Values inserted into xTimeLimitsOfTypes.'
		END
		-- If table TimeLimitsOfTypes is empty, then raise error.
		ELSE RAISERROR ('Table TimeLimitsOfTypes does not contain any entries. Please setup CleanupDb parameters first.',11,1) WITH NOWAIT	
	END
	-- If table TimeLimitsOfTypes does not exist, then raise error.
	ELSE RAISERROR ('Table TimeLimitsOfTypes does not exist. Please create first.',11,1) WITH NOWAIT
END
GO