PRINT 'Copying ProtocolData...'
GO

---------------------------------------------------------------------------------------------------------------------------------------------
-- TYPE AND BENCH SPECIFIC
---------------------------------------------------------------------------------------------------------------------------------------------
IF ((SELECT Count(BenchId) FROM TimeLimitsOfTypes WHERE BenchId IS NOT NULL) <> 0) 
BEGIN

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[xTimeLimitsOfTypes]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
	IF ((SELECT COUNT(Id) FROM xTimeLimitsOfTypes) <> 0)
	BEGIN

		DECLARE 
			@lim_id int, 
			@lim_name varchar(50),
			@bench_id int, 
			@bench_name varchar(50),
			@lim_limitTime dateTime,
			@lim_Transferred int,
			@duration int,
			@unitsProt bigint, 
			@lineMessage varchar(300),
			@intErrorCode int,
			@del_Start dateTime,
			@unitsTotal int,
			@unitsRest int,
			@unitsCopied int,
			@durationCopied int,
			@durationRest int,
			@factor float,
			@message varchar(300)


---------------------------------------------------------------------------------------------------------------------------
		
		SELECT @unitsTotal = (SELECT SUM(UnitsRetained) FROM xTimeLimitsOfTypes WHERE LimitTime IS NOT NULL AND Transferred = 0)
		SET @message = '1:' + CONVERT(char(10), @unitsTotal)
		RAISERROR(@message,5,1) WITH NOWAIT

		SELECT @unitsCopied = (SELECT SUM(UnitsRetained) FROM xTimeLimitsOfTypes WHERE LimitTime IS NOT NULL AND Transferred = 1)
		SELECT @durationCopied = (SELECT SUM(Duration) FROM xTimeLimitsOfTypes WHERE LimitTime IS NOT NULL AND Transferred = 1)
		
		IF (@unitsCopied <> 0)
		BEGIN
			IF (@durationCopied <> 0)
			BEGIN
				SELECT @factor = CONVERT(float, @durationCopied) / CONVERT(float, @unitsCopied)
				SELECT @durationRest = @factor * @unitsTotal
			END
			ELSE 
			BEGIN
				SELECT @durationRest = 0.005 * @unitsTotal
			END
		END 
		ELSE
		BEGIN
			SELECT @durationRest = 0.005 * @unitsTotal
		END 

		SET @message = '2:' + CONVERT(char(10), @durationRest)
		RAISERROR(@message,5,1) WITH NOWAIT

---------------------------------------------------------------------------------------------------------------------------

		DECLARE lim_cursor CURSOR FOR 
		SELECT Id, Name, BenchId, BenchName, LimitTime, Transferred, Duration, UnitsProtocol
		FROM xTimeLimitsOfTypes
		WHERE LimitTime IS NOT NULL AND Transferred = 0
		ORDER BY Id, BenchId

		OPEN lim_cursor

		FETCH NEXT FROM lim_cursor 
		INTO @lim_id, @lim_name, @bench_id, @bench_name, @lim_limitTime, @lim_Transferred, @duration, @unitsProt 

		WHILE @@FETCH_STATUS = 0
		BEGIN

		   SELECT @lineMessage = @lim_name + ' and bench ' + @bench_name
		   SET @message = '3: Copying data for type ' + @lineMessage
		   RAISERROR(@message,0,1) WITH NOWAIT

		   -- Local variable that stores the system time before the delete statement is submitted to the SQL-Server
		   SELECT @del_Start = GETDATE()

		   BEGIN TRAN	
		   INSERT INTO  [xProtocolData]

    		SELECT FkIdent,FkMode,FkChannel,FkLocation,FkInstrument,FkParameter,Value,ULimit,LLimit,Position,FkUnit,FkExtra 
   			FROM [ProtocolData] INNER JOIN MainData ON ProtocolData.FkIdent = MainData.PkIdent
    		WHERE MainData.Time >= @lim_limitTime AND MainData.Type = @lim_id AND MainData.TestBench = @bench_id	
    
    		SELECT @intErrorCode = @@ERROR	
			IF (@intErrorCode <> 0) GOTO PROBLEM

    		UPDATE [xTimeLimitsOfTypes]
    		SET Transferred = 1
    		WHERE xTimeLimitsOfTypes.Id = @lim_id AND xTimeLimitsOfTypes.BenchId = @bench_id
    		SELECT @intErrorCode = @@ERROR	
			IF (@intErrorCode <> 0) GOTO PROBLEM

			UPDATE [xTimeLimitsOfTypes]
    		SET Duration = DATEDIFF(SECOND, @del_Start, GETDATE())
    		WHERE xTimeLimitsOfTypes.Id = @lim_id AND xTimeLimitsOfTypes.BenchId = @bench_id
    		SELECT @intErrorCode = @@ERROR	
			IF (@intErrorCode <> 0) GOTO PROBLEM

			COMMIT TRAN

---------------------------------------------------------------------------------------------------------------------------
			
			SELECT @unitsCopied = (SELECT SUM(UnitsRetained) FROM xTimeLimitsOfTypes WHERE LimitTime IS NOT NULL AND Transferred = 1)
			SELECT @durationCopied = (SELECT SUM(Duration) FROM xTimeLimitsOfTypes WHERE LimitTime IS NOT NULL AND Transferred = 1)
			SELECT @unitsRest = (SELECT SUM(UnitsRetained) FROM xTimeLimitsOfTypes WHERE LimitTime IS NOT NULL AND Transferred = 0)

			IF (@unitsCopied <> 0)
			BEGIN
				IF (@durationCopied <> 0)
				BEGIN
					SELECT @factor = CONVERT(float, @durationCopied) / CONVERT(float, @unitsCopied)
					SELECT @durationRest = @factor * @unitsRest
				END
				ELSE 
				BEGIN
					SELECT @durationRest = 0.005 * @unitsRest
				END
			END 
			
			SET @message = '4:' + CONVERT(char(10), @unitsCopied)
			RAISERROR(@message,5,1) WITH NOWAIT

			SET @message = '2:' + CONVERT(char(10), @durationRest)
			RAISERROR(@message,5,1) WITH NOWAIT

---------------------------------------------------------------------------------------------------------------------------
			
			PROBLEM:
			IF (@intErrorCode <> 0) BEGIN
			  PRINT 'Unexpected error occurred!'
			  ROLLBACK TRAN
			END

			-- Get the next line.
			FETCH NEXT FROM lim_cursor 
			INTO @lim_id, @lim_name, @bench_id, @bench_name, @lim_limitTime, @lim_Transferred, @duration, @unitsProt

		END

		CLOSE lim_cursor
		DEALLOCATE lim_cursor

		PRINT 'Data copy completed'

	END
	ELSE RAISERROR ('Table xTimeLimitsOfTypes does not contain any entries. Please calculate first.',11,1) WITH NOWAIT	
END
ELSE RAISERROR ('Table xTimeLimitsOfTypes does not exist. Please create first.',11,1) WITH NOWAIT

END
---------------------------------------------------------------------------------------------------------------------------------------------
-- TYPE SPECIFIC
---------------------------------------------------------------------------------------------------------------------------------------------
ELSE
BEGIN

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[xTimeLimitsOfTypes]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
	IF ((SELECT COUNT(Id) FROM xTimeLimitsOfTypes) <> 0)
	BEGIN

		DECLARE 
			@lim_id2 int, 
			@lim_name2 varchar(50),
			@bench_id2 int, 
			@bench_name2 varchar(50),
			@lim_limitTime2 dateTime,
			@lim_Transferred2 int,
			@duration2 int,
			@unitsProt2 bigint, 
			@lineMessage2 varchar(300),
			@intErrorCode2 int,
			@del_Start2 dateTime,
			@unitsTotal2 int,
			@unitsRest2 int,
			@unitsCopied2 int,
			@durationCopied2 int,
			@durationRest2 int,
			@factor2 float,
			@message2 varchar(300)


---------------------------------------------------------------------------------------------------------------------------
		
		SELECT @unitsTotal2 = (SELECT SUM(UnitsRetained) FROM xTimeLimitsOfTypes WHERE LimitTime IS NOT NULL AND Transferred = 0)
		SET @message2 = '1:' + CONVERT(char(10), @unitsTotal2)
		RAISERROR(@message2,5,1) WITH NOWAIT

		SELECT @unitsCopied2 = (SELECT SUM(UnitsRetained) FROM xTimeLimitsOfTypes WHERE LimitTime IS NOT NULL AND Transferred = 1)
		SELECT @durationCopied2 = (SELECT SUM(Duration) FROM xTimeLimitsOfTypes WHERE LimitTime IS NOT NULL AND Transferred = 1)
		
		IF (@unitsCopied2 <> 0)
		BEGIN
			IF (@durationCopied2 <> 0)
			BEGIN
				SELECT @factor2 = CONVERT(float, @durationCopied2) / CONVERT(float, @unitsCopied2)
				SELECT @durationRest2 = @factor2 * @unitsTotal2
			END
			ELSE 
			BEGIN
				SELECT @durationRest2 = 0.005 * @unitsTotal2
			END
		END 
		ELSE
		BEGIN
			SELECT @durationRest2 = 0.005 * @unitsTotal2
		END 

		SET @message2 = '2:' + CONVERT(char(10), @durationRest2)
		RAISERROR(@message2,5,1) WITH NOWAIT

---------------------------------------------------------------------------------------------------------------------------

		DECLARE lim_cursor2 CURSOR FOR 
		SELECT Id, Name, BenchId, BenchName, LimitTime, Transferred, Duration, UnitsProtocol
		FROM xTimeLimitsOfTypes
		WHERE LimitTime IS NOT NULL AND Transferred = 0
		ORDER BY Id, BenchId

		OPEN lim_cursor2

		FETCH NEXT FROM lim_cursor2
		INTO @lim_id2, @lim_name2, @bench_id2, @bench_name2, @lim_limitTime2, @lim_Transferred2, @duration2, @unitsProt2

		WHILE @@FETCH_STATUS = 0
		BEGIN

		   SELECT @lineMessage2 = @lim_name2
		   SET @message2 = '3: Copying data for type ' + @lineMessage2
		   RAISERROR(@message2,0,1) WITH NOWAIT

		   -- Local variable that stores the system time before the delete statement is submitted to the SQL-Server
		   SELECT @del_Start2 = GETDATE()

		   BEGIN TRAN
			INSERT INTO  [xProtocolData]

    			SELECT FkIdent,FkMode,FkChannel,FkLocation,FkInstrument,FkParameter,Value,ULimit,LLimit,Position,FkUnit,FkExtra 
   				FROM [ProtocolData] INNER JOIN MainData ON ProtocolData.FkIdent = MainData.PkIdent
    			WHERE MainData.Time >= @lim_limitTime2 AND MainData.Type = @lim_id2	
    
    			SELECT @intErrorCode2 = @@ERROR	
				IF (@intErrorCode2 <> 0) GOTO PROBLEM2

    			UPDATE [xTimeLimitsOfTypes]
    			SET Transferred = 1
    			WHERE xTimeLimitsOfTypes.Id = @lim_id2
    			SELECT @intErrorCode2 = @@ERROR	
				IF (@intErrorCode2 <> 0) GOTO PROBLEM2

				UPDATE [xTimeLimitsOfTypes]
    			SET Duration = DATEDIFF(SECOND, @del_Start2, GETDATE())
    			WHERE xTimeLimitsOfTypes.Id = @lim_id2
    			SELECT @intErrorCode2 = @@ERROR	
				IF (@intErrorCode2 <> 0) GOTO PROBLEM2

			COMMIT TRAN

---------------------------------------------------------------------------------------------------------------------------
			
			SELECT @unitsCopied2 = (SELECT SUM(UnitsRetained) FROM xTimeLimitsOfTypes WHERE LimitTime IS NOT NULL AND Transferred = 1)
			SELECT @durationCopied2 = (SELECT SUM(Duration) FROM xTimeLimitsOfTypes WHERE LimitTime IS NOT NULL AND Transferred = 1)
			SELECT @unitsRest2 = (SELECT SUM(UnitsRetained) FROM xTimeLimitsOfTypes WHERE LimitTime IS NOT NULL AND Transferred = 0)

			IF (@unitsCopied2 <> 0)
			BEGIN
				IF (@durationCopied2 <> 0)
				BEGIN
					SELECT @factor2 = CONVERT(float, @durationCopied2) / CONVERT(float, @unitsCopied2)
					SELECT @durationRest2 = @factor2 * @unitsRest2
				END
				ELSE 
				BEGIN
					SELECT @durationRest2 = 0.005 * @unitsRest2
				END
			END 
			
			SET @message2 = '4:' + CONVERT(char(10), @unitsCopied2)
			RAISERROR(@message2,5,1) WITH NOWAIT

			SET @message2 = '2:' + CONVERT(char(10), @durationRest2)
			RAISERROR(@message2,5,1) WITH NOWAIT

---------------------------------------------------------------------------------------------------------------------------
			
			PROBLEM2:
			IF (@intErrorCode2 <> 0) BEGIN
			  PRINT 'Unexpected error occurred!'
			  ROLLBACK TRAN
			END

			-- Get the next line.
			FETCH NEXT FROM lim_cursor2
			INTO @lim_id2, @lim_name2, @bench_id2, @bench_name2, @lim_limitTime2, @lim_Transferred2, @duration2, @unitsProt2

		END

		CLOSE lim_cursor2
		DEALLOCATE lim_cursor2

		PRINT 'Data copy completed'

	END
	ELSE RAISERROR ('Table xTimeLimitsOfTypes does not contain any entries. Please calculate first.',11,1) WITH NOWAIT	
END
ELSE RAISERROR ('Table xTimeLimitsOfTypes does not exist. Please create first.',11,1) WITH NOWAIT

END
GO
